const API_BASE_URL = 'https://www.goai-app.com';

class AdminPanel {
    constructor() {
        this.userTableBody = document.getElementById('userTableBody');
        this.deleteButton = document.getElementById('deleteSelectedBtn');
        this.authContainer = document.getElementById('authContainer');
        this.mainContainer = document.getElementById('mainContainer');
        this.addCreditsBtn = document.getElementById('addCreditsBtn');
    }

    async init() {
        try {
            const isAdmin = await this.checkAdminAuth();
            if (isAdmin) {
                this.authContainer.classList.add('hidden');
                this.mainContainer.classList.remove('hidden');
                await this.loadUsers();
                this.setupEventListeners();
            } else {
                this.authContainer.querySelector('.auth-message').textContent = 'Please Login';
            }
        } catch (error) {
            console.error('Auth check failed:', error);
            this.authContainer.querySelector('.auth-message').textContent = 'Please login';
        }
    }

    async checkAdminAuth() {
        try {
            const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
            
            if (!email || !authentication_token) {
                return false;
            }

            const response = await fetch(`${API_BASE_URL}/adminGetUsers?email=${encodeURIComponent(email)}&authToken=${authentication_token}`, {
                method: 'GET',
                headers: {
                    'Content-Type': 'application/json',
                    'Accept': 'application/json',
                    'Origin': chrome.runtime.getURL('')
                }
            });

            const data = await response.json();
            return data.success === true;
        } catch (error) {
            console.error('Admin auth check failed:', error);
            return false;
        }
    }

    async loadUsers() {
        try {
            const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
            
            if (!email || !authentication_token) {
                throw new Error('Missing credentials');
            }

            const response = await fetch(`${API_BASE_URL}/adminGetUsers?email=${encodeURIComponent(email)}&authToken=${authentication_token}`, {
                method: 'GET',
                headers: {
                    'Content-Type': 'application/json',
                    'Accept': 'application/json',
                    'Origin': chrome.runtime.getURL('')
                },
                credentials: 'include'
            });

            if (!response.ok) {
                const errorData = await response.json();
                throw new Error(errorData.error || `HTTP error! status: ${response.status}`);
            }

            const data = await response.json();
            if (!data.success) {
                throw new Error('Failed to load users');
            }
            this.renderUsers(data.users);
        } catch (error) {
            console.error('Error loading users:', error);
            this.userTableBody.innerHTML = `<tr><td colspan="4">Error: ${error.message}</td></tr>`;
        }
    }

    renderUsers(users) {
        this.userTableBody.innerHTML = '';
        
        users.forEach((user, index) => {
            const row = document.createElement('tr');
            row.className = 'user-row';

            // Create LinkedIn profile link
            const linkedinLink = user.linkedin_profile_url ? 
                `<a href="${user.linkedin_profile_url}" target="_blank">${user.linkedin_profile_url}</a>` : 
                'N/A';

            // Format dates
            const formatDate = (dateStr) => {
                return dateStr ? new Date(dateStr).toLocaleString() : 'N/A';
            };

            row.innerHTML = `
                <td>${index + 1}</td>
                <td class="checkbox-cell">
                    <input type="checkbox" class="user-checkbox" data-user-id="${user.user_email_address}">
                </td>
                <td>${user.user_email_address}</td>
                <td>${linkedinLink}</td>
                <td>${user.credits}</td>
                <td>${formatDate(user.last_login)}</td>
                <td>${formatDate(user.account_created)}</td>
                <td>${user.referred_by || 'N/A'}</td>
                <td>${user.magic_code || 'N/A'}</td>
                <td>${formatDate(user.magic_code_expiration)}</td>
                <td>${user.authentication_token || 'N/A'}</td>
            `;

            this.userTableBody.appendChild(row);
        });

        this.updateButtons();
    }

    async deleteUser(userEmail) {
        try {
            const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
            
            if (!email || !authentication_token) {
                throw new Error('Missing admin credentials');
            }

            const response = await fetch(`${API_BASE_URL}/adminDeleteUser?email=${encodeURIComponent(email)}&authToken=${authentication_token}`, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    'Accept': 'application/json',
                    'Origin': chrome.runtime.getURL('')
                },
                body: JSON.stringify({
                    userEmail: userEmail
                })
            });

            const data = await response.json();
            
            if (!data.success) {
                throw new Error(data.error || 'Failed to delete user');
            }

            // Show success message
            this.showMessage(`Successfully deleted user: ${userEmail}`, 'success');
            
            // Refresh the user list
            await this.loadUsers();

        } catch (error) {
            console.error('Error deleting user:', error);
            this.showMessage(`Failed to delete user: ${error.message}`, 'error');
        }
    }

    showMessage(message, type) {
        // Create message element if it doesn't exist
        let messageElement = document.getElementById('adminMessage');
        if (!messageElement) {
            messageElement = document.createElement('div');
            messageElement.id = 'adminMessage';
            document.querySelector('.container').insertBefore(messageElement, document.getElementById('userTable'));
        }

        // Set message content and style
        messageElement.textContent = message;
        messageElement.className = `message ${type}`;

        // Auto-hide after 3 seconds
        setTimeout(() => {
            messageElement.remove();
        }, 3000);
    }

    setupEventListeners() {
        // Listen for checkbox changes
        this.userTableBody.addEventListener('change', (e) => {
            if (e.target.type === 'checkbox') {
                // Uncheck all other checkboxes
                const checkboxes = document.querySelectorAll('.user-checkbox');
                checkboxes.forEach(checkbox => {
                    if (checkbox !== e.target) {
                        checkbox.checked = false;
                    }
                });
                
                // Update both buttons
                this.updateButtons();
            }
        });

        // Delete button click handler
        this.deleteButton.addEventListener('click', async () => {
            const checkedBox = this.userTableBody.querySelector('input[type="checkbox"]:checked');
            if (checkedBox) {
                const userId = checkedBox.dataset.userId;
                const confirmed = await this.showConfirmDialog(`Are you sure you want to delete user: ${userId}?`);
                if (confirmed) {
                    await this.deleteUser(userId);
                }
            }
        });

        this.addCreditsBtn.addEventListener('click', () => {
            const checkedBox = this.userTableBody.querySelector('input[type="checkbox"]:checked');
            if (checkedBox) {
                this.showCreditsModal(checkedBox.dataset.userId);
            }
        });

        document.getElementById('addCreditsConfirm').addEventListener('click', () => {
            this.handleAddCredits();
        });

        document.getElementById('addCreditsCancel').addEventListener('click', () => {
            this.hideCreditsModal();
        });
    }

    updateButtons() {
        const hasChecked = this.userTableBody.querySelector('input[type="checkbox"]:checked');
        this.deleteButton.disabled = !hasChecked;
        this.addCreditsBtn.disabled = !hasChecked;
        
        const viewReportsBtn = document.getElementById('viewReportsBtn');
        if (viewReportsBtn) {
            viewReportsBtn.disabled = !hasChecked;
        }
    }

    async showConfirmDialog(message) {
        return new Promise((resolve) => {
            const modal = document.getElementById('confirmModal');
            const confirmMessage = document.getElementById('confirmMessage');
            const yesButton = document.getElementById('confirmYes');
            const noButton = document.getElementById('confirmNo');

            confirmMessage.textContent = message;
            modal.classList.add('show');

            const handleYes = () => {
                modal.classList.remove('show');
                cleanup();
                resolve(true);
            };

            const handleNo = () => {
                modal.classList.remove('show');
                cleanup();
                resolve(false);
            };

            const cleanup = () => {
                yesButton.removeEventListener('click', handleYes);
                noButton.removeEventListener('click', handleNo);
            };

            yesButton.addEventListener('click', handleYes);
            noButton.addEventListener('click', handleNo);
        });
    }

    showCreditsModal(userEmail) {
        const modal = document.getElementById('creditsModal');
        document.getElementById('creditsUserEmail').textContent = userEmail;
        document.getElementById('creditsAmount').value = '100';
        modal.classList.add('show');
    }

    hideCreditsModal() {
        const modal = document.getElementById('creditsModal');
        modal.classList.remove('show');
    }

    async handleAddCredits() {
        const userEmail = document.getElementById('creditsUserEmail').textContent;
        const credits = parseInt(document.getElementById('creditsAmount').value);

        if (!credits || credits < 1) {
            this.showMessage('Please enter a valid number of credits', 'error');
            return;
        }

        try {
            const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
            
            if (!email || !authentication_token) {
                throw new Error('Missing admin credentials');
            }

            const response = await fetch(`${API_BASE_URL}/adminAddCredits?email=${encodeURIComponent(email)}&authToken=${authentication_token}`, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    'Accept': 'application/json',
                    'Origin': chrome.runtime.getURL('')
                },
                body: JSON.stringify({
                    userEmail: userEmail,
                    credits: credits
                })
            });

            const data = await response.json();
            
            if (!data.success) {
                throw new Error(data.error || 'Failed to add credits');
            }

            this.showMessage(`Successfully added ${credits} credits to ${userEmail}`, 'success');
            this.hideCreditsModal();
            
            // Refresh the user list to show updated credits
            await this.loadUsers();

        } catch (error) {
            console.error('Error adding credits:', error);
            this.showMessage(`Failed to add credits: ${error.message}`, 'error');
        }
    }
}

// Initialize the admin panel when the document is loaded
document.addEventListener('DOMContentLoaded', () => {
    const adminPanel = new AdminPanel();
    adminPanel.init();
});
