class AdminReports {
    constructor() {
        this.viewReportsBtn = document.getElementById('viewReportsBtn');
        this.reportsSection = document.getElementById('reportsSection');
        this.reportUserEmail = document.getElementById('reportUserEmail');
        this.setupEventListeners();
    }

    setupEventListeners() {
        this.viewReportsBtn.addEventListener('click', async () => {
            const checkedBox = document.querySelector('input[type="checkbox"]:checked');
            if (checkedBox) {
                const userEmail = checkedBox.dataset.userId;
                await this.loadUserReports(userEmail);
            }
        });
    }

    async loadUserReports(userEmail) {
        try {
            this.reportUserEmail.textContent = userEmail;
            this.reportsSection.classList.remove('hidden');

            const [profileData, messageGenData, messagesSentData, openAiData] = await Promise.all([
                this.fetchReportData(userEmail, 'profile-captures'),
                this.fetchReportData(userEmail, 'messages-generated'),
                this.fetchReportData(userEmail, 'messages-sent'),
                this.fetchReportData(userEmail, 'openai-usage')
            ]);

            await this.updateProfileScrapeStats(profileData);
            await this.updateMessageStats(messageGenData);
            await this.updateMessagesSentStats(messagesSentData);
            await this.updateOpenAIStats(openAiData);

        } catch (error) {
            console.error('Error loading user reports:', error);
            this.showError(`Failed to load reports: ${error.message}`);
        }
    }

    async fetchReportData(userEmail, reportType) {
        const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
        
        if (!email || !authentication_token) {
            throw new Error('Missing admin credentials');
        }

        const response = await fetch(
            `${API_BASE_URL}/adminGetReportingData?` +
            `email=${encodeURIComponent(email)}&` +
            `authToken=${authentication_token}&` +
            `userEmail=${encodeURIComponent(userEmail)}&` +
            `reportType=${encodeURIComponent(reportType)}`, {
            method: 'GET',
            headers: {
                'Content-Type': 'application/json',
                'Accept': 'application/json',
                'Origin': chrome.runtime.getURL('')
            }
        });

        if (!response.ok) {
            throw new Error(`HTTP error! status: ${response.status}`);
        }

        const data = await response.json();
        if (!data.success) {
            throw new Error(data.error || 'Failed to fetch report data');
        }

        return data.data;
    }

    async updateProfileScrapeStats(reportData) {
        const totals = reportData.reduce((acc, day) => {
            acc.total += day.total;
            acc.successful += day.successful;
            acc.failed += day.failed;
            return acc;
        }, { total: 0, successful: 0, failed: 0 });

        document.getElementById('total-profiles-scraped').textContent = totals.total.toLocaleString();
        document.getElementById('basic-scrapes').textContent = totals.successful.toLocaleString();
        document.getElementById('deep-scrapes').textContent = totals.failed.toLocaleString();
        document.getElementById('scrape-success-rate').textContent = 
            `${((totals.successful / totals.total) * 100).toFixed(1)}%`;

        this.updateDailyScrapeGraph(reportData);
    }

    async updateMessageStats(reportData) {
        const totals = reportData.reduce((acc, day) => {
            acc.total += day.total;
            acc.total_messages += day.total_messages;
            Object.entries(day.byType || {}).forEach(([type, count]) => {
                acc.byType[type] = (acc.byType[type] || 0) + count;
            });
            return acc;
        }, { total: 0, total_messages: 0, byType: {} });

        document.getElementById('total-messages-generated').textContent = totals.total.toLocaleString();
        
        const messageTypes = Object.entries(totals.byType);
        if (messageTypes.length >= 2) {
            document.getElementById('total-connect-messages').textContent = 
                messageTypes[0][1].toLocaleString();
            document.getElementById('total-dm-messages').textContent = 
                messageTypes[1][1].toLocaleString();
        }
        
        document.getElementById('avg-messages-per-contact').textContent = 
            (totals.total_messages / totals.total).toFixed(1);

        this.updateDailyMessagesGraph(reportData);
    }

    async updateMessagesSentStats(reportData) {
        const totals = reportData.reduce((acc, day) => {
            acc.total += day.total;
            Object.entries(day.byType || {}).forEach(([type, count]) => {
                acc.byType[type] = (acc.byType[type] || 0) + count;
            });
            return acc;
        }, { total: 0, byType: {} });

        const successRate = ((totals.total / Object.values(totals.byType).reduce((a, b) => a + b, 0)) * 100).toFixed(1);

        document.getElementById('total-messages-sent').textContent = totals.total.toLocaleString();
        
        const messageTypes = Object.entries(totals.byType);
        if (messageTypes.length >= 2) {
            document.getElementById('total-sent-connect').textContent = 
                messageTypes[0][1].toLocaleString();
            document.getElementById('total-sent-dm').textContent = 
                messageTypes[1][1].toLocaleString();
        }
        
        document.getElementById('messages-sent-success-rate').textContent = `${successRate}%`;

        this.updateMessagesSentGraph(reportData);
    }

    async updateOpenAIStats(reportData) {
        const totals = reportData.reduce((acc, day) => {
            acc.totalTokens += day.totalTokens || 0;
            acc.totalCost += day.totalCost || 0;
            acc.total += day.total || 0;
            acc.successful += day.successful || 0;
            return acc;
        }, { totalTokens: 0, totalCost: 0, total: 0, successful: 0 });

        document.getElementById('total-tokens-used').textContent = totals.totalTokens.toLocaleString();
        document.getElementById('total-cost').textContent = `$${totals.totalCost.toFixed(2)}`;
        document.getElementById('total-requests').textContent = totals.total.toLocaleString();
        
        const avgTokens = totals.total ? Math.round(totals.totalTokens / totals.total) : 0;
        document.getElementById('avg-tokens-per-message').textContent = avgTokens.toLocaleString();

        this.updateOpenAIUsageGraph(reportData);
    }

    updateDailyScrapeGraph(reportData) {
        const graphBars = document.getElementById('daily-scrapes-graph');
        const graphLabels = document.getElementById('daily-scrapes-labels');
        
        graphBars.innerHTML = '';
        graphLabels.innerHTML = '';
        
        const sortedData = [...reportData].sort((a, b) => new Date(a.date) - new Date(b.date));
        const maxCount = Math.max(...sortedData.map(day => day.total));
        
        sortedData.forEach(day => {
            const barContainer = document.createElement('div');
            barContainer.className = 'bar-container';
            
            if (day.total > 0 && maxCount > 0) {
                const barsWrapper = document.createElement('div');
                barsWrapper.className = 'bars-wrapper';
                
                if (day.successful > 0) {
                    const barWrapper = document.createElement('div');
                    barWrapper.className = 'bar-wrapper';
                    
                    const successBar = document.createElement('div');
                    successBar.className = 'bar scrape-bar success';
                    const successHeight = (day.successful / maxCount) * 100;
                    successBar.style.height = `${successHeight}%`;
                    successBar.title = `${day.date}: ${day.successful} successful captures`;
                    
                    barWrapper.appendChild(successBar);
                    barsWrapper.appendChild(barWrapper);
                }
                
                if (day.failed > 0) {
                    const barWrapper = document.createElement('div');
                    barWrapper.className = 'bar-wrapper';
                    
                    const failBar = document.createElement('div');
                    failBar.className = 'bar scrape-bar failed';
                    const failHeight = (day.failed / maxCount) * 100;
                    failBar.style.height = `${failHeight}%`;
                    failBar.title = `${day.date}: ${day.failed} failed captures`;
                    
                    barWrapper.appendChild(failBar);
                    barsWrapper.appendChild(barWrapper);
                }
                
                barContainer.appendChild(barsWrapper);
            }
            
            graphBars.appendChild(barContainer);
            
            const label = document.createElement('div');
            label.className = 'label';
            const date = new Date(day.date);
            label.textContent = `${date.getMonth() + 1}/${date.getDate()}`;
            graphLabels.appendChild(label);
        });
    }

    updateDailyMessagesGraph(reportData) {
        this.updateGraphWithMessageData(
            reportData,
            'message-generation-graph',
            'message-generation-labels',
            document.querySelector('.graph-legend')
        );
    }

    updateMessagesSentGraph(reportData) {
        this.updateGraphWithMessageData(
            reportData,
            'sent-messages-graph',
            'sent-messages-labels',
            document.getElementById('sent-messages-legend')
        );
    }

    updateOpenAIUsageGraph(reportData) {
        const graphBars = document.getElementById('openai-usage-graph');
        const graphLabels = document.getElementById('openai-usage-labels');
        
        graphBars.innerHTML = '';
        graphLabels.innerHTML = '';
        
        const sortedData = [...reportData].sort((a, b) => new Date(a.date) - new Date(b.date));
        const maxTokens = Math.max(...sortedData.map(day => day.totalTokens || 0));
        
        sortedData.forEach(day => {
            const barContainer = document.createElement('div');
            barContainer.className = 'bar-container';
            
            if (day.totalTokens > 0 && maxTokens > 0) {
                const barWrapper = document.createElement('div');
                barWrapper.className = 'bar-wrapper';
                
                const bar = document.createElement('div');
                bar.className = 'bar openai-bar';
                const height = (day.totalTokens / maxTokens) * 100;
                bar.style.height = `${height}%`;
                
                const tooltipDate = new Date(day.date).toLocaleDateString();
                bar.title = `${tooltipDate}:
Tokens: ${day.totalTokens.toLocaleString()}
Cost: $${day.totalCost.toFixed(2)}
Requests: ${day.total}
Success Rate: ${((day.successful / day.total) * 100).toFixed(1)}%`;
                
                barWrapper.appendChild(bar);
                barContainer.appendChild(barWrapper);
            }
            
            graphBars.appendChild(barContainer);
            
            const label = document.createElement('div');
            label.className = 'label';
            const date = new Date(day.date);
            label.textContent = `${date.getMonth() + 1}/${date.getDate()}`;
            graphLabels.appendChild(label);
        });
    }

    updateGraphWithMessageData(reportData, graphBarsId, graphLabelsId, legendContainer) {
        const graphBars = document.getElementById(graphBarsId);
        const graphLabels = document.getElementById(graphLabelsId);
        
        graphBars.innerHTML = '';
        graphLabels.innerHTML = '';
        legendContainer.innerHTML = '';
        
        const sortedData = [...reportData].sort((a, b) => new Date(a.date) - new Date(b.date));
        
        const messageTypes = new Set();
        sortedData.forEach(day => {
            if (day.byType) {
                Object.keys(day.byType).forEach(type => messageTypes.add(type));
            }
        });
        
        const colorMap = {
            'connection-message': '#057642',
            'direct-message': '#0a66c2'
        };
        
        const visibilityState = {};
        messageTypes.forEach(type => {
            visibilityState[type] = true;
        });
        
        const legendItems = [];
        
        [...messageTypes].forEach((type) => {
            const legendItem = document.createElement('div');
            legendItem.className = 'legend-item';
            legendItem.style.cursor = 'pointer';
            legendItem.innerHTML = `
                <div class="legend-color" data-type="${type}" style="background-color: ${colorMap[type]}"></div>
                <span>${type.replace('-', ' ').replace(/\b\w/g, l => l.toUpperCase())}</span>
            `;
            
            legendItem.addEventListener('click', () => {
                visibilityState[type] = !visibilityState[type];
                legendItem.style.opacity = visibilityState[type] ? '1' : '0.5';
                const bars = document.querySelectorAll(`.message-bar[data-type="${type}"]`);
                bars.forEach(bar => {
                    bar.style.opacity = visibilityState[type] ? '1' : '0';
                });
            });
            
            legendItems.push(legendItem);
            legendContainer.appendChild(legendItem);
        });
        
        const maxCount = Math.max(...sortedData.map(day => 
            day.byType ? Object.values(day.byType).reduce((sum, count) => sum + count, 0) : 0
        ));
        
        sortedData.forEach(day => {
            const barContainer = document.createElement('div');
            barContainer.className = 'bar-container';
            
            if (day.byType && maxCount > 0) {
                const barsWrapper = document.createElement('div');
                barsWrapper.className = 'bars-wrapper';
                
                Object.entries(day.byType).forEach(([type, count]) => {
                    if (count > 0) {
                        const barWrapper = document.createElement('div');
                        barWrapper.className = 'bar-wrapper';
                        
                        const bar = document.createElement('div');
                        bar.className = 'bar message-bar';
                        bar.setAttribute('data-type', type);
                        bar.style.backgroundColor = colorMap[type] || '#gray';
                        const height = (count / maxCount) * 100;
                        bar.style.height = `${height}%`;
                        bar.title = `${day.date}: ${count} ${type} messages`;
                        
                        bar.addEventListener('click', this.createBarClickHandler(
                            type, 
                            visibilityState, 
                            messageTypes,
                            legendItems
                        ));
                        
                        barWrapper.appendChild(bar);
                        barsWrapper.appendChild(barWrapper);
                    }
                });
                
                barContainer.appendChild(barsWrapper);
            }
            
            graphBars.appendChild(barContainer);
            
            const label = document.createElement('div');
            label.className = 'label';
            const date = new Date(day.date);
            label.textContent = `${date.getMonth() + 1}/${date.getDate()}`;
            graphLabels.appendChild(label);
        });
    }

    createBarClickHandler(type, visibilityState, messageTypes, legendItems) {
        return () => {
            const isOnlyTypeVisible = Object.entries(visibilityState)
                .every(([t, visible]) => t === type ? visible : !visible);

            if (isOnlyTypeVisible) {
                messageTypes.forEach(t => {
                    visibilityState[t] = true;
                    const bars = document.querySelectorAll(`.message-bar[data-type="${t}"]`);
                    bars.forEach(bar => {
                        bar.style.opacity = '1';
                    });
                    legendItems.forEach(item => {
                        if (item.querySelector(`[data-type="${t}"]`)) {
                            item.style.opacity = '1';
                        }
                    });
                });
            } else {
                messageTypes.forEach(t => {
                    visibilityState[t] = (t === type);
                    const bars = document.querySelectorAll(`.message-bar[data-type="${t}"]`);
                    bars.forEach(bar => {
                        bar.style.opacity = t === type ? '1' : '0';
                    });
                    legendItems.forEach(item => {
                        if (item.querySelector(`[data-type="${t}"]`)) {
                            item.style.opacity = t === type ? '1' : '0.5';
                        }
                    });
                });
            }
        };
    }

    showError(message) {
        const errorDiv = document.createElement('div');
        errorDiv.className = 'error';
        errorDiv.textContent = message;
        this.reportsSection.insertBefore(errorDiv, this.reportsSection.firstChild);
        
        setTimeout(() => {
            errorDiv.remove();
        }, 5000);
    }
}

document.addEventListener('DOMContentLoaded', () => {
    new AdminReports();
}); 