const GA_MEASUREMENT_ID = 'G-2YS98L3EYW';
const API_SECRET = '3IyNRtQWTPeISiz45HSDbA';

async function sendAnalyticsEvent(eventName, eventParams = {}) {
    const endpoint = `https://www.google-analytics.com/mp/collect?measurement_id=${GA_MEASUREMENT_ID}&api_secret=${API_SECRET}`;

    try {
        // Get user and session data from storage
        const { 
            ga_client_id, 
            email,
            session_id,
            session_start_time,
            session_count,
            first_visit_time
        } = await chrome.storage.local.get([
            'ga_client_id', 
            'email',
            'session_id',
            'session_start_time',
            'session_count',
            'first_visit_time'
        ]);
        
        // Generate client ID if not exists
        let clientId = ga_client_id;
        if (!clientId) {
            clientId = Math.random().toString(36).substring(2, 15);
            await chrome.storage.local.set({ ga_client_id: clientId });
        }

        // Create the event data object with enhanced user and session data
        const eventData = {
            client_id: clientId,
            user_id: email,
            non_personalized_ads: false,
            user_properties: {
                email: { value: email },
                first_visit: { value: first_visit_time },
                session_count: { value: session_count || 1 }
            },
            events: [{
                name: eventName,
                params: {
                    ...eventParams,
                    user_email: email,
                    session_id: session_id,
                    session_start_time: session_start_time,
                    engagement_time_msec: Date.now() - (session_start_time || Date.now()),
                    session_number: session_count || 1,
                    debug_mode: process.env?.NODE_ENV === 'development' ? 1 : 0
                }
            }]
        };

        // Send the event to GA4
        const response = await fetch(endpoint, {
            method: 'POST',
            body: JSON.stringify(eventData),
            headers: {
                'Content-Type': 'application/json',
                'Origin': chrome.runtime.getURL(''),
            },
            mode: 'cors',
            credentials: 'omit'
        });

        if (!response.ok) {
            throw new Error(`Analytics request failed: ${response.statusText}`);
        }

        // Log success in development
        if (process.env?.NODE_ENV === 'development') {
            console.log('✅ Analytics event sent:', {
                eventName,
                eventParams,
                userId: email,
                sessionId: session_id
            });
        }

    } catch (error) {
        console.error('❌ Error sending analytics event:', error);
    }
}

// Helper function to start a new session
async function startNewSession() {
    const sessionData = {
        session_id: generateSessionId(),
        session_start_time: Date.now(),
    };

    // Get and increment session count
    const { session_count = 0, first_visit_time } = await chrome.storage.local.get([
        'session_count',
        'first_visit_time'
    ]);

    // Update session data
    await chrome.storage.local.set({
        ...sessionData,
        session_count: session_count + 1,
        first_visit_time: first_visit_time || Date.now()
    });

    return sessionData;
}

// Helper function to generate session ID
function generateSessionId() {
    return 'sess_' + Date.now() + '_' + Math.random().toString(36).substring(2, 15);
}

// Helper function to check if session is expired
async function isSessionExpired() {
    const { session_start_time } = await chrome.storage.local.get('session_start_time');
    if (!session_start_time) return true;

    // Session expires after 30 minutes of inactivity
    const SESSION_TIMEOUT = 30 * 60 * 1000; // 30 minutes in milliseconds
    return Date.now() - session_start_time > SESSION_TIMEOUT;
}

// Message listener for analytics events
chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
    if (message.action === 'sendAnalyticsEvent') {
        sendAnalyticsEvent(message.eventName, message.eventParams);
    } else if (message.action === 'startNewSession') {
        startNewSession();
    } else if (message.action === 'checkSession') {
        isSessionExpired().then(expired => {
            if (expired) startNewSession();
        });
    }
    return true; // Required for async response
});

// Make functions available globally
window.sendAnalyticsEvent = sendAnalyticsEvent;
window.startNewSession = startNewSession;
window.isSessionExpired = isSessionExpired;
