// Add at the top of the file, after imports but before any other code
const state = {
    analysisInProgress: {},
    profileCache: {},
    cacheTimeout: 60 * 60 * 1000,
    clearAnalysisState: (url) => {
        Object.keys(state.analysisInProgress).forEach(key => {
            if (key.includes(url)) {
                delete state.analysisInProgress[key];
            }
        });
    }
};

// Add this after the state definition
setInterval(() => {
    state.profileCache = {};
    console.log('🧹 Cleared profile cache');
}, state.cacheTimeout);

// Add at the top of the file, after imports
const API_BASE_URL = 'https://www.goai-app.com';

console.log('🔄 Extension background script is running!');

// LinkedIn Profile Data Fetching
async function checkLinkedInCookies() {
    const cookie = await chrome.cookies.get({
        url: 'https://www.linkedin.com',
        name: 'li_at'  // LinkedIn's authentication cookie
    });
    return cookie !== null;
}

async function openLinkedInTab() {
    const tab = await chrome.tabs.create({
        url: 'https://www.linkedin.com',
        active: false  // Opens in background
    });
    return tab;
}

async function waitForSeconds(seconds) {
    return new Promise(resolve => setTimeout(resolve, seconds * 1000));
}

async function getLinkedInCookiesWithRetry() {
    try {
        // First attempt to get cookies
        let hasCookies = await checkLinkedInCookies();
        
        if (!hasCookies) {
            console.log("LinkedIn cookies not found. Opening LinkedIn in background...");
            const tab = await openLinkedInTab();
            
            if (!tab) {
                throw new Error("Failed to open LinkedIn tab");
            }
            
            await waitForSeconds(5);
            hasCookies = await checkLinkedInCookies();
            
            try {
                await chrome.tabs.remove(tab.id);
            } catch (e) {
                console.error("Error closing LinkedIn tab:", e);
            }
            
            if (!hasCookies) {
                throw new Error("Failed to get LinkedIn cookies. Please log in to LinkedIn first.");
            }
        }
        
        return await chrome.cookies.getAll({ domain: '.linkedin.com' });
    } catch (error) {
        console.error("Error in getLinkedInCookiesWithRetry:", error);
        throw error;
    }
}

async function makeLinkedInRequest(url, csrfToken) {
    console.log(`🌐 Making LinkedIn request to: ${url}`);
    try {
        const response = await fetch(url, {
            method: 'GET',
            headers: {
                'accept': 'application/vnd.linkedin.normalized+json+2.1',
                'csrf-token': csrfToken,
                'x-li-lang': 'en_US',
                'x-li-track': '{"clientVersion":"1.12.8","mpVersion":"1.12.8"}',
                'x-restli-protocol-version': '2.0.0',
                'x-li-page-instance': 'urn:li:page:d_flagship3_profile_self_edit_top_card;' + Math.random(),
                'sec-ch-ua': '"Not_A Brand";v="8", "Chromium";v="120"',
                'sec-ch-ua-mobile': '?0',
                'sec-ch-ua-platform': '"Windows"',
                'x-li-pem-metadata': 'Voyager - Profile=profile-edit-top-card',
                'Referer': 'https://www.linkedin.com/',
                'Accept-Language': 'en-US,en;q=0.9'
            },
            credentials: 'include'
        });

        if (!response.ok) {
            const errorText = await response.text();
            console.error(`❌ LinkedIn API request failed for ${url}:`, response.status, errorText);
            throw new Error(`LinkedIn API request failed: ${response.status} - ${errorText}`);
        }

        const data = await response.json();
        console.log(`✅ LinkedIn request successful for: ${url}`);
        return data;
    } catch (error) {
        console.error(`🚨 Error in makeLinkedInRequest for ${url}:`, error);
        throw error;
    }
}

async function fetchLinkedInProfileData() {
    try {
        console.log('🔄 Starting LinkedIn profile data fetch...');
        
        const cookies = await getLinkedInCookiesWithRetry();
        console.log('🍪 Retrieved cookies:', cookies.length);
        
        // Get CSRF token from JSESSIONID cookie
        const jsessionCookie = cookies.find(c => c.name === 'JSESSIONID');
        if (!jsessionCookie) {
            console.error('❌ JSESSIONID cookie not found');
            throw new Error('JSESSIONID cookie not found');
        }
        console.log('🔑 Found JSESSIONID cookie');

        // Remove quotes from JSESSIONID value if present
        const csrfToken = jsessionCookie.value.replace(/"/g, '');
        console.log('🔒 CSRF Token prepared');

        // First, make a request to the homepage to get necessary cookies
        console.log('🏠 Making initial homepage request...');
        await fetch('https://www.linkedin.com/', {
            credentials: 'include'
        });
        console.log('✅ Homepage request complete');

        // Fetch basic profile data
        console.log('👤 Fetching basic profile...');
        const basicProfile = await makeLinkedInRequest('https://www.linkedin.com/voyager/api/me', csrfToken);
        console.log('✅ Basic profile retrieved:', basicProfile);

        // Fetch additional profile data using self endpoints
        console.log('📚 Fetching additional profile data...');
        const [contactInfo, profileView, skills] = await Promise.all([
            makeLinkedInRequest('https://www.linkedin.com/voyager/api/identity/profiles/me/profileContactInfo', csrfToken),
            makeLinkedInRequest('https://www.linkedin.com/voyager/api/identity/profiles/me', csrfToken),
            makeLinkedInRequest('https://www.linkedin.com/voyager/api/identity/profiles/me/skillCategory', csrfToken)
        ]);
        console.log('✅ Additional data retrieved');
        console.log('📞 Contact Info:', contactInfo);
        console.log('👥 Profile View:', profileView);
        console.log('🎯 Skills:', skills);

        // Combine all the data
        const enrichedData = {
            basicProfile,
            contactInfo,
            profileView,
            skills,
            timestamp: new Date().toISOString()
        };
        console.log('🎁 Final enriched data:', enrichedData);

        return { success: true, data: enrichedData };
    } catch (error) {
        console.error('🚨 Error in fetchLinkedInProfileData:', error);
        console.error('Error stack:', error.stack);
        return { success: false, error: error.message };
    }
}

chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
    console.group('🎯 Background Script: Message Received');
    console.log('📥 Message type:', request.type || request.action);
    console.log('📥 Message data:', request);
    console.log('📥 Sender:', sender);

    if (request.action === "getLinkedInProfile") {
        // Send immediate acknowledgment
        sendResponse({ received: true });
        
        // Start the fetch
        fetchLinkedInProfileData()
            .then(response => {
                console.log('📦 Raw LinkedIn data:', response);
                
                // Send the data via runtime message
                chrome.runtime.sendMessage({
                    type: 'linkedInProfileData',
                    success: true,
                    data: response
                });
            })
            .catch(error => {
                console.error('🚨 Error in LinkedIn fetch:', error);
                // Send error via runtime message
                chrome.runtime.sendMessage({
                    type: 'linkedInProfileData',
                    success: false,
                    error: error.message
                });
            });
        return true; // Will respond asynchronously with initial acknowledgment
    }

    switch (request.type || request.action) {
        case 'START_ANALYSIS':
        case 'ANALYZE_PROFILE':
            handleAnalysisRequest(request, sender)
                .then(response => {
                    console.log('Analysis completed, sending response:', response);
                    sendResponse(response);
                })
                .catch(error => {
                    console.error('Analysis failed:', error);
                    sendResponse({ 
                        success: false, 
                        error: error.message 
                    });
                });
            return true;
        case 'PROCESS_PROFILE':
            handleProcessProfile(request, sendResponse);
            break;
        case 'UPDATE_CONTACT_TAGS':
            handleUpdateContactTags(request, sendResponse);
            break;
        case 'SCRAPE_ACTIVITY':
            handleScrapeActivity(request, sendResponse);
            break;
        case 'SAVE_PROFILE_PICTURE':
            handleSaveProfilePicture(request, sendResponse);
            break;
        case 'GENERATE_POSTS':
            handleGeneratePosts(request, sendResponse);
            break;
        case 'GENERATE_POST_IMAGE':
            handleGeneratePostImage(request, sendResponse);
            break;
        case 'TRACK_SENT_MESSAGE':
            handleTrackSentMessage(request, sendResponse);
            break;
        case 'URL_CHANGED':
            state.clearAnalysisState(request.from);
            delete state.profileCache[request.from];
            break;
        case 'LINKEDIN_GOAI_CONNECTION_OUTCOME':
            // Forward the connection outcome to all tabs
            chrome.tabs.query({}, (tabs) => {
                tabs.forEach(tab => {
                    chrome.tabs.sendMessage(tab.id, {
                        type: 'CONNECTION_REQUEST_STATUS',
                        success: request.success,
                        error: request.error
                    });
                });
            });
            break;
        case 'LINKEDIN_GOAI_MESSAGE_OUTCOME':
            // Forward the direct message outcome to all tabs
            chrome.tabs.query({}, (tabs) => {
                tabs.forEach(tab => {
                    chrome.tabs.sendMessage(tab.id, {
                        type: 'DIRECT_MESSAGE_STATUS',
                        success: request.success,
                        error: request.error
                    });
                });
            });
            break;
        case 'LINKEDIN_GOAI_VISIT_OUTCOME':
            // Forward the profile visit outcome to all tabs
            chrome.tabs.query({}, (tabs) => {
                tabs.forEach(tab => {
                    chrome.tabs.sendMessage(tab.id, {
                        type: 'PROFILE_VISIT_STATUS',
                        success: request.success,
                        error: request.error
                    });
                });
            });
            break;
    }

    console.groupEnd();
    return true;
});

// Add these new utility functions
async function handleAnalysisRequest(request, sender) {
    const tabId = sender.tab ? sender.tab.id : request.tabId;
    const url = sender.tab ? sender.tab.url : request.url;
    const contactId = url.split('/in/')[1]?.replace(/\/$/, '');
    
    // Use both tabId and exact URL for the analysis key
    const analysisKey = `${tabId}_${url}`;
    
    if (state.analysisInProgress[analysisKey]) {
        console.log('🚫 Analysis already in progress for this tab/URL', analysisKey);
        return { success: false, error: 'Analysis already in progress' };
    }
    
    try {
        // Get auth credentials
        const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);
        
        // Check API for existing contact
        const response = await fetch(`https://www.goai-app.com/getContact?email=${encodeURIComponent(email)}&authToken=${authToken}&contactId=${contactId}`, {
            method: 'GET',
            headers: {
                'Accept': 'application/json'
            }
        });

        const data = await response.json();
        
        if (data.success && data.contact) {
            const lastRefresh = new Date(data.contact.RefreshDatetime);
            const now = new Date();
            const timeDiff = (now - lastRefresh) / 1000 / 60; // difference in minutes
            
            if (timeDiff < 2) {
                console.log('✅ Contact already exists in API and was recently updated');
                return { 
                    success: true, 
                    data: data.contact,
                    alreadyCaptured: true,
                    captureDate: data.contact.RefreshDatetime
                };
            }
        }

        // Set analysis flag
        state.analysisInProgress[analysisKey] = true;
        const result = await handleProfileAnalysis(tabId, url);
        console.log('✅ Profile analysis completed successfully');

        return { success: true, data: result };
    } catch (error) {
        console.error('❌ Error during profile analysis:', error);


        return { success: false, error: error.message };
    } finally {
        delete state.analysisInProgress[analysisKey];
    }
}

async function handleProcessProfile(request, sendResponse) {
    console.log('🔄 Processing profile for URL:', request.data.ContactProfileURL);
    try {
        const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
        const response = await makeApiRequest('/addContact', {
            method: 'POST',
            body: JSON.stringify({ 
                email, 
                authToken: authentication_token, 
                contact: request.data 
            })
        });
        
        console.log('✅ Contact processing completed:', response);
        sendResponse({ success: true, data: response.contact });
    } catch (error) {
        console.error('❌ Error processing contact:', error);
        sendResponse({ success: false, error: error.message });
    }
}

async function handleUpdateContactTags(request, sendResponse) {
    const { profileUrl, tags } = request.data;
    try {
        const result = await findAndProcessContact(profileUrl, { ContactTags: tags });
        sendResponse({ success: true, data: result.contact });
    } catch (error) {
        sendResponse({ success: false, error: error.message });
    }
}

async function handleScrapeActivity(request, sendResponse) {
    console.log('🔄 Starting activity scraping for URL:', request.url);
    try {
        const posts = await handleActivityScraping(request.url, request.settings);
        console.log('✅ Activity scraping completed, found posts:', posts.length);
        sendResponse({ success: true, posts });
    } catch (error) {
        console.error('❌ Error scraping activity:', error);
        sendResponse({ success: false, error: error.message });
    }
}

async function handleSaveProfilePicture(request, sendResponse) {
    try {
        const result = await downloadProfilePicture(request.imageUrl, request.profileId);
        sendResponse(result);
    } catch (error) {
        sendResponse({ success: false, error: error.message });
    }
}

async function handleProfileAnalysis(tabId, url) {
    console.group('🔍handleProfileAnalysis function starting');

    try {
        // Check if profile data is already in cache
        if (state.profileCache[url]) {
            console.log('📋 Using cached profile data');
            await findAndProcessContact(url, state.profileCache[url]);
            return state.profileCache[url];
        }

        // Request profile scraping from content script
        console.log('🔄 Requesting profile data scraping');
        const scrapeResponse = await chrome.tabs.sendMessage(tabId, { 
            type: 'SCRAPE_PROFILE'
        });
        
        if (!scrapeResponse.success) {
            throw new Error('Failed to scrape profile data');
        }

        // Cache the scraped data
        state.profileCache[url] = scrapeResponse.data;

        // Process the scraped data
        console.log('🔄 Processing scraped profile data');
        await findAndProcessContact(url, scrapeResponse.data);

        console.log('✅ Profile analysis completed');
        return scrapeResponse.data;
    } catch (error) {
        console.error('❌ Error in handleProfileAnalysis:', error);
        throw error;
    } finally {
        console.groupEnd();
    }
}


async function handleActivityScraping(url, settings) {
    try {
        // Create background tab
        const tab = await chrome.tabs.create({
            url: url,
            active: false
        });

        // Wait for tab to load
        await new Promise(resolve => {
            chrome.tabs.onUpdated.addListener(function listener(tabId, info) {
                if (tabId === tab.id && info.status === 'complete') {
                    chrome.tabs.onUpdated.removeListener(listener);
                    resolve();
                }
            });
        });

        // First inject the postScraper script
        await chrome.scripting.executeScript({
            target: { tabId: tab.id },
            files: ['services/postScraper.js']
        });

        // Then execute the scraping function
        const result = await chrome.scripting.executeScript({
            target: { tabId: tab.id },
            func: (settings) => {
                return window.LinkedInGoAI.postScraper.scrapePostsFromActivityPage(settings);
            },
            args: [settings]
        });

        // Close the background tab
        await chrome.tabs.remove(tab.id);

        return result[0].result;
    } catch (error) {
        console.error('Error in handleActivityScraping:', error);
        throw error;
    }
}


async function downloadProfilePicture(imageUrl, profileId) {
    try {
        // Fetch the image
        const response = await fetch(imageUrl);
        if (!response.ok) throw new Error('Failed to fetch image');
        
        // Get the image data as array buffer
        const arrayBuffer = await response.arrayBuffer();
        
        // Convert array buffer to base64
        const base64String = btoa(
            new Uint8Array(arrayBuffer)
                .reduce((data, byte) => data + String.fromCharCode(byte), '')
        );
        
        // Create data URL
        const dataUrl = `data:${response.headers.get('content-type') || 'image/jpeg'};base64,${base64String}`;
        
        // Store in chrome.storage.local
        const storageKey = `profilePicture_${profileId}`;
        await chrome.storage.local.set({ [storageKey]: dataUrl });
        
        return { success: true };
    } catch (error) {
        console.error('Error saving profile picture:', error);
        return { success: false, error: error.message };
    }
}


console.log('🚀 Extension background script is running for the first time!');

chrome.runtime.onInstalled.addListener((details) => {
    if (details.reason === 'install') {
        console.log('🎉 Extension installed for the first time!');
        
        // Set initial credits based on installation date
        const initialCredits = 0;
        
        // Initialize credits
        chrome.storage.local.set({ credits: initialCredits });
        chrome.tabs.create({ url: 'thankyou.html' });
    }
});

async function handleGeneratePosts(request, sendResponse) {
    sendResponse({ 
        success: false, 
        error: "Image Generation not currently supported!" 
    });
}

async function handleGeneratePostImage(request, sendResponse) {
    sendResponse({ 
        success: false, 
        error: "Image generation not supported currently!" 
    });
}

async function handleTrackSentMessage(request, sendResponse) {
    try {
        sendResponse({ success: true });
    } catch (error) {
        console.error('Error tracking sent message:', error);
        sendResponse({ success: false, error: error.message });
    }
}

// Add helper function for API calls near the top of the file
async function makeApiRequest(endpoint, options = {}) {
    const defaultOptions = {
        headers: {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            'Origin': chrome.runtime.getURL('')
        },
        credentials: 'include'
    };

    const finalOptions = {
        ...defaultOptions,
        ...options,
        headers: {
            ...defaultOptions.headers,
            ...options.headers
        }
    };

    const response = await fetch(`${API_BASE_URL}${endpoint}`, finalOptions);
    if (!response.ok) {
        throw new Error(`API request failed: ${response.statusText}`);
    }
    return response.json();
}
