window.showResults = function(analysis) {
  console.group('Showing Analysis Results');
  console.log('Raw analysis data:', analysis);
  if (analysis.rawData) {
    console.log('Raw data content:', analysis.rawData);
    console.log('Stringified raw data:', JSON.stringify(analysis.rawData));
  }
  console.groupEnd();
  const modal = document.createElement('div');
  modal.className = 'goai-modal';
  
  modal.innerHTML = createModalContent(analysis);
  document.body.appendChild(modal);

  // Pass analysis to addModalEventListeners
  return addModalEventListeners(modal, analysis);
}

window.showError = function(error) {
  const errorModal = document.createElement('div');
  errorModal.className = 'goai-modal goai-error';
  
  errorModal.innerHTML = `
    <div class="goai-modal-header">
      <h2>Error</h2>
      <button class="goai-close-button">x</button>
    </div>
    <div class="goai-modal-content">
      <p>Sorry, we encountered an error while analyzing this profile.</p>
      <p class="error-details">${error.message}</p>
      ${error.errorCode ? `<p class="error-code">Error code: ${error.errorCode}</p>` : ''}
    </div>
  `;
  
  document.body.appendChild(errorModal);
  // Return cleanup function
  return addModalEventListeners(errorModal);
}

function createModalContent(analysis) {
  return `
    <div class="goai-modal-header">
      <h2>Profile Analysis</h2>
      <button class="goai-close-button">x</button>
    </div>
    <div class="goai-tabs">
      <button class="goai-tab-button active" data-tab="json">JSON Data</button>
      <button class="goai-tab-button" data-tab="ai">AI Messages</button>
    </div>
    <div class="goai-modal-content">
      <div class="goai-tab-content active" id="json-content">
        <div class="goai-json-viewer">
          <pre>${JSON.stringify(analysis.rawData ?? analysis, null, 2)}</pre>
        </div>
      </div>
      <div class="goai-tab-content" id="ai-content">
        <div class="goai-ai-controls">
          <button class="goai-generate-button">Generate AI Messages</button>
        </div>
        <div class="goai-ai-messages-info-handler"></div>
        <div class="goai-ai-messages-api-payload"></div>
        <div class="goai-ai-messages-api-response"></div>
        <div class="goai-ai-messages-container"></div>
      </div>
    </div>
  `;
}

function addModalEventListeners(modal, analysis) {
    const listeners = new Set();
    
    // Tab switching
    const tabHandler = (event) => {
        if (event.target.classList.contains('goai-tab-button')) {
            modal.querySelectorAll('.goai-tab-button').forEach(t => t.classList.remove('active'));
            modal.querySelectorAll('.goai-tab-content').forEach(c => c.classList.remove('active'));
            
            event.target.classList.add('active');
            const contentId = `${event.target.dataset.tab}-content`;
            modal.querySelector(`#${contentId}`).classList.add('active');
        }
    };
    modal.addEventListener('click', tabHandler);
    listeners.add({ element: modal, type: 'click', handler: tabHandler });

    // Close button
    const closeHandler = () => modal.remove();
    const closeButton = modal.querySelector('.goai-close-button');
    closeButton.addEventListener('click', closeHandler);
    listeners.add({ element: closeButton, type: 'click', handler: closeHandler });

    // Outside click
    const outsideClickHandler = (e) => {
        if (e.target === modal) modal.remove();
    };
    modal.addEventListener('click', outsideClickHandler);
    listeners.add({ element: modal, type: 'click', handler: outsideClickHandler });

    // Add AI message generation handler
    const generateButton = modal.querySelector('.goai-generate-button');
    const messagesContainer = modal.querySelector('.goai-ai-messages-container');
    const infoHandler = modal.querySelector('.goai-ai-messages-info-handler');
    
    const generateHandler = async () => {
        try {
            generateButton.disabled = true;
            infoHandler.textContent = 'Generating messages...';
            
            // Add timer element next to the info handler
            const timerSpan = document.createElement('span');
            timerSpan.className = 'goai-timer';
            timerSpan.textContent = '0s';
            infoHandler.appendChild(timerSpan);

            // Start timer
            const startTime = Date.now();
            const timerInterval = setInterval(() => {
                const elapsedSeconds = Math.floor((Date.now() - startTime) / 1000);
                timerSpan.textContent = `${elapsedSeconds}s`;
            }, 1000);

            // Get the scraped data from the modal's dataset
            const scrapedData = modal.dataset.scrapedData ? 
                JSON.parse(modal.dataset.scrapedData) : 
                JSON.stringify(analysis.rawData || analysis, null, 2);

            // Generate messages using the existing function from contentScript.js
            const response = await generateMessages(scrapedData);
            
            // Stop timer
            clearInterval(timerInterval);
            const totalTime = Math.floor((Date.now() - startTime) / 1000);
            timerSpan.textContent = `${totalTime}s`;
            
            // Display the formatted messages using the existing function
            messagesContainer.innerHTML = formatAIMessages(response);
            infoHandler.textContent = `Response received in ${totalTime}s`;

            // Add click handlers for send buttons
            messagesContainer.querySelectorAll('.goai-message-send').forEach(sendButton => {
                sendButton.addEventListener('click', async (e) => {
                    const messageText = e.target.closest('.goai-message')
                        .querySelector('.goai-message-body').textContent;
                    
                    const result = await sendLinkedInMessage(messageText);
                    if (!result.success) {
                        console.error('Failed to send message:', result.error);
                    }
                });
            });

            // Add click handlers for message copying
            messagesContainer.querySelectorAll('.goai-message-body').forEach(messageDiv => {
                messageDiv.addEventListener('click', (e) => {
                    const message = e.target.textContent;
                    navigator.clipboard.writeText(message);
                    const tempSpan = document.createElement('span');
                    tempSpan.textContent = ' (Copied!)';
                    e.target.appendChild(tempSpan);
                    setTimeout(() => {
                        tempSpan.remove();
                    }, 2000);
                });
            });
            
        } catch (error) {
            console.error('Error generating AI messages:', error);
            infoHandler.textContent = `Error: ${error.message}`;
        } finally {
            generateButton.disabled = false;
        }
    };

    generateButton.addEventListener('click', generateHandler);
    listeners.add({ element: generateButton, type: 'click', handler: generateHandler });

    // Return cleanup function
    return () => {
        listeners.forEach(({ element, type, handler }) => {
            element.removeEventListener(type, handler);
        });
        listeners.clear();
    };
}

function formatAIMessages(aiResponse) {
    try {
        // Check if aiResponse is already parsed JSON
        const parsedMessages = typeof aiResponse === 'string' ? JSON.parse(aiResponse) : aiResponse;
        
        if (parsedMessages.messages && Array.isArray(parsedMessages.messages)) {
            return parsedMessages.messages.map(message => `
                <div class="goai-message">
                    <button class="goai-message-send">Send</button>
                    <div class="goai-message-body">${message.messageBody}</div>
                    <div class="goai-message-reasons">
                        ${message.reasons.map(reason => 
                            `<div class="goai-message-reason">${reason}</div>`
                        ).join('')}
                    </div>
                </div>
            `).join('');
        }
        
        return `<div class="goai-message-error">Invalid response format</div>`;
    } catch (error) {
        console.error('Error formatting AI messages:', error);
        return `<div class="goai-message-error">Error formatting messages: ${error.message}</div>`;
    }
}
