import { delay } from './utils.js';
import { fetchGeneratedMessages } from './messageManager.js';
import { showActivitiesModal } from './tableManager.js';

const OPERATION_TIMEOUT = 60000; // 60 seconds in milliseconds

function createTimeout(ms) {
    return new Promise((_, reject) => {
        setTimeout(() => reject(new Error('Operation timed out')), ms);
    });
}

let timerInterval;
let startTime;
let pausedTime = 0;
let isAutomationPaused = false;
let isAutomationStopped = false;
let delaySeconds = 60;

// Timer functions
function startTimer() {
    startTime = Date.now() - pausedTime;
    const timerDisplay = document.querySelector('.timer-display');
    
    timerInterval = setInterval(() => {
        const elapsedTime = Date.now() - startTime;
        const seconds = Math.floor((elapsedTime / 1000) % 60);
        const minutes = Math.floor((elapsedTime / (1000 * 60)) % 60);
        const hours = Math.floor(elapsedTime / (1000 * 60 * 60));
        
        timerDisplay.textContent = `${hours.toString().padStart(2, '0')}:${minutes.toString().padStart(2, '0')}:${seconds.toString().padStart(2, '0')}`;
    }, 1000);
}

function stopTimer() {
    clearInterval(timerInterval);
    pausedTime = 0;
}

function pauseTimer() {
    clearInterval(timerInterval);
    pausedTime = Date.now() - startTime;
}

// Control button functions
function createControlButtons() {
    const controlButtonsContainer = document.querySelector('.control-buttons-container');
    
    const pauseButton = document.createElement('button');
    pauseButton.className = 'primary-button';
    pauseButton.id = 'pauseButton';
    pauseButton.innerHTML = '<i class="fas fa-pause"></i> Pause';
    
    controlButtonsContainer.appendChild(pauseButton);
    
    pauseButton.addEventListener('click', () => {
        if (isAutomationPaused) {
            isAutomationPaused = false;
            pauseButton.innerHTML = '<i class="fas fa-pause"></i> Pause';
            document.querySelector('.progress-status').textContent = 'Processing';
            startTimer();
            
            const delayInput = document.getElementById('delaySeconds');
            delaySeconds = delayInput.value === '' ? 60 : parseInt(delayInput.value);
        } else {
            isAutomationPaused = true;
            pauseButton.innerHTML = '<i class="fas fa-play"></i> Resume';
            document.querySelector('.progress-status').textContent = 'Paused';
            pauseTimer();
        }
    });
}

function resetControlButtons() {
    const pauseButton = document.getElementById('pauseButton');
    if (pauseButton) pauseButton.remove();
}

// Message processing functions
async function checkExistingMessages(messagesCell, statusCell) {
    const existingMessagesButton = messagesCell.querySelector('.messages-count-button');
    if (existingMessagesButton) {
        console.log('Messages already exist for this contact');
        const isConnectionRequest = existingMessagesButton.classList.contains('connection-request');
        statusCell.querySelector('.status-icon').innerHTML = isConnectionRequest ? 
            '<i class="fas fa-comment-dots" style="color: #22c55e;"></i>' : 
            '<i class="fas fa-comment-dots" style="color: #0a66c2;"></i>';
        statusCell.querySelector('.status-icon').title = 'Messages Ready';
        return true;
    }
    return false;
}

async function handleSkipAction(actionType, statusCell) {
    if (actionType === 'skip') {
        console.log('Skipping contact');
        statusCell.querySelector('.status-icon').innerHTML = `<i class="fas fa-forward" style="color: #6c757d;"></i>`;
        statusCell.querySelector('.status-icon').title = 'Skipped';
        return true;
    }
    return false;
}

async function updateSuggestedAction(row, contact) {
    const actionSelect = row.querySelector('.action-select');
    const actionContainer = actionSelect.closest('.action-container');
    let suggestedActionInfo = actionContainer.querySelector('.suggestedActionInfo');

    // Remove existing suggestedActionInfo if it exists
    if (suggestedActionInfo) {
        suggestedActionInfo.remove();
    }

    // Check if data is older than 24 hours
    let needsRefresh = false;
    if (contact.RefreshDatetime) {
        const refreshDate = new Date(contact.RefreshDatetime);
        const now = new Date();
        const diffHours = Math.abs(now - refreshDate) / 36e5;
        if (diffHours > 24) {
            needsRefresh = true;
        }
    }

    // Add new suggestedActionInfo if needed
    if (needsRefresh) {
        suggestedActionInfo = document.createElement('div');
        suggestedActionInfo.className = 'suggestedActionInfo actionListContacts-info';
        suggestedActionInfo.innerHTML = `
            <i class="fas fa-exclamation-triangle"></i>
            >24 hours data
        `;
        actionContainer.appendChild(suggestedActionInfo);
        
        // Update action to visit_profile if available
        const visitProfileOption = actionSelect.querySelector('option[value="visit_profile"]');
        if (visitProfileOption && !visitProfileOption.disabled) {
            actionSelect.value = 'visit_profile';
            // Dispatch change event to trigger any listeners
            actionSelect.dispatchEvent(new Event('change', { bubbles: true }));
        }
    }

    // TODO: Add more suggested action logic here
    // For example:
    // - Check last interaction date
    // - Check connection status changes
    // - Check activity frequency
    // - etc.
}

async function refreshContactOnTable(row, linkedinId, email, authentication_token) {
    const statusCell = row.querySelector('.status-cell');
    let originalIcon = null;
    let originalTitle = null;

    try {
        if (!row || !linkedinId || !email || !authentication_token) {
            throw new Error('Missing required parameters for refreshing contact data');
        }

        // Keep the grey eye icon during refresh if it's a visit_profile action
        if (statusCell) {
            const currentIcon = statusCell.querySelector('.status-icon i');
            originalIcon = currentIcon ? currentIcon.outerHTML : null;
            originalTitle = currentIcon ? statusCell.querySelector('.status-icon').title : null;
            
            const isVisitProfile = currentIcon && currentIcon.classList.contains('fa-eye');
            
            if (!isVisitProfile) {
                statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-sync fa-spin" style="color: #0a66c2;"></i>';
                statusCell.querySelector('.status-icon').title = 'Refreshing Contact Data';
            }
        }

        // Get user preference for profile pictures
        const { showProfilePictures } = await chrome.storage.local.get(['showProfilePictures']);

        console.log('Fetching updated contact data for:', linkedinId);
        const response = await fetch(
            `https://www.goai-app.com/getContact?email=${encodeURIComponent(email)}&authToken=${encodeURIComponent(authentication_token)}&contactId=${encodeURIComponent(linkedinId)}`
        );
        
        if (!response.ok) {
            throw new Error('Failed to fetch updated contact data');
        }

        const data = await response.json();
        if (!data.success || !data.contact) {
            throw new Error('Invalid contact data received');
        }

        const contact = data.contact;
        console.log('Received contact data:', contact);

        // Update profile picture if enabled
        if (showProfilePictures === true) {
            try {
                const profilePicResponse = await fetch(
                    `https://www.goai-app.com/getProfilePicture?email=${encodeURIComponent(email)}&authToken=${encodeURIComponent(authentication_token)}&contactId=${encodeURIComponent(linkedinId)}`,
                    { method: 'GET' }
                );

                if (profilePicResponse.ok) {
                    const blob = await profilePicResponse.blob();
                    const profilePicUrl = URL.createObjectURL(blob);
                    const profilePicImg = row.querySelector('.profile-picture');
                    if (profilePicImg) {
                        profilePicImg.src = profilePicUrl;
                        profilePicImg.setAttribute('data-original-src', profilePicUrl);
                    }
                }
            } catch (error) {
                console.warn('Failed to update profile picture:', error);
            }
        }

        // Update connection status overlay
        const isConnected = contact.existingContact === '1';
        const isPending = contact.pendingConnection === 1;
        const connectionStatusOverlay = row.querySelector('.connection-status-overlay');
        const connectionIcon = row.querySelector('.connection-status-overlay .icon');
        
        if (connectionStatusOverlay && connectionIcon) {
            // Update classes
            connectionStatusOverlay.className = `connection-status-overlay ${isConnected ? 'connected' : isPending ? 'pending' : 'not-connected'}`;
            connectionIcon.className = `fas ${isConnected ? 'fa-check' : isPending ? 'fa-hourglass' : 'fa-times'} icon`;
        }

        // Update activities cell
        const activitiesCell = row.querySelector('.activities-cell');
        if (activitiesCell) {
            activitiesCell.innerHTML = '';
            if (contact.activities && contact.activities.length > 0) {
                // Calculate recent activities (within last 7 days)
                const recentActivities = contact.activities.filter(activity => {
                    const activityDate = new Date(activity.postWhen);
                    const now = new Date();
                    const diffTime = Math.abs(now - activityDate);
                    const diffDays = Math.ceil(diffTime / (1000 * 60 * 60 * 24));
                    return diffDays <= 7;
                });
                const hasRecentActivity = recentActivities.length > 0;

                const activitiesDiv = document.createElement('div');
                activitiesDiv.className = `activities-count${hasRecentActivity ? ' has-recent' : ''}`;
                activitiesDiv.title = `${contact.activities.length} activities${hasRecentActivity ? ` (${recentActivities.length} in last week)` : ''}`;
                activitiesDiv.innerHTML = `
                    <span class="count">${contact.activities.length}</span>
                    <svg class="icon" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M21.99 4c0-1.1-.89-2-1.99-2H4c-1.1 0-2 .9-2 2v12c0 1.1.9 2 2 2h14l4 4-.01-18zM18 14H6v-2h12v2zm0-3H6V9h12v2zm0-3H6V6h12v2z" fill="currentColor"/>
                    </svg>
                `;
                activitiesDiv.onclick = () => showActivitiesModal(contact.activities);
                activitiesCell.appendChild(activitiesDiv);
            } else {
                activitiesCell.textContent = '-';
            }
        }

        // Update suggested action based on refreshed data
        await updateSuggestedAction(row, contact);

        // Restore original icon if it was a visit_profile action
        if (statusCell && originalIcon && originalTitle) {
            statusCell.querySelector('.status-icon').innerHTML = originalIcon;
            statusCell.querySelector('.status-icon').title = originalTitle;
        }

        console.log('Successfully refreshed contact data for:', linkedinId);

    } catch (error) {
        console.error('Error in refreshContactOnTable:', error);
        
        // Restore original icon if it was a visit_profile action
        if (statusCell && originalIcon && originalTitle) {
            statusCell.querySelector('.status-icon').innerHTML = originalIcon;
            statusCell.querySelector('.status-icon').title = originalTitle;
        } else if (statusCell) {
            statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-times" style="color: #dc3545;"></i>';
            statusCell.querySelector('.status-icon').title = 'Error: ' + error.message;
        }
        
        throw error; // Re-throw the error to be handled by the caller
    }
}

async function handleVisitProfile(actionType, profileUrl, messagesCell, statusCell, email, authentication_token) {
    console.log('handleVisitProfile called with:', {
        actionType,
        profileUrl,
        messagesCell: !!messagesCell,
        statusCell: !!statusCell,
        email: !!email
    });

    let tab = null;
    // Get auto-close setting at the start, default to false if not set
    const result = await chrome.storage.local.get('autoCloseTabs');
    const autoCloseTabs = result.autoCloseTabs ?? false;
    console.log('Auto-close tabs setting:', autoCloseTabs);

    try {
        if (!profileUrl) {
            throw new Error('Profile URL is required');
        }

        // Update status to show processing with grey eye icon
        if (statusCell && statusCell.querySelector('.status-icon')) {
            statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-eye" style="color: #6b7280;"></i>';
            statusCell.querySelector('.status-icon').title = 'Visiting Profile';
        }

        // Create a promise that resolves on tab update
        const tabUpdatePromise = new Promise((resolve, reject) => {
            console.log('Creating new tab with URL:', profileUrl);
            chrome.tabs.create({ url: profileUrl, active: false }, async (createdTab) => {
                tab = createdTab;
                console.log('Tab created:', tab?.id);

                // Wait for tab to be ready
                const checkTab = () => {
                    chrome.tabs.get(tab.id, async (updatedTab) => {
                        if (chrome.runtime.lastError) {
                            reject(new Error('Tab was closed'));
                            return;
                        }

                        if (updatedTab.status === 'complete') {
                            resolve();
                        } else {
                            setTimeout(checkTab, 500);
                        }
                    });
                };
                checkTab();
            });
        });

        // Wait for tab to be ready
        await tabUpdatePromise;
        
        // Extract LinkedIn ID from profile URL
        const linkedinId = profileUrl?.match(/linkedin\.com\/in\/([^\/]+)/)?.[1];
        if (!linkedinId) {
            console.error('Invalid profile URL format:', profileUrl);
            throw new Error('Could not extract LinkedIn ID from profile URL');
        }

        // Send message to content script and wait for response
        const response = await chrome.tabs.sendMessage(tab.id, {
            action: 'processProfile',
            profileUrl: profileUrl
        });

        console.log('Profile processing response:', response);

        if (!response || !response.success) {
            throw new Error(response?.error || 'Failed to process profile');
        }

        // Close tab if auto-close is enabled
        if (autoCloseTabs && tab) {
            try {
                await chrome.tabs.remove(tab.id);
                console.log('Tab closed:', tab.id);
            } catch (error) {
                console.warn('Error closing tab:', error);
            }
        }

        // Update contact data in table
        if (statusCell) {
            const row = statusCell.closest('tr');
            if (row) {
                await refreshContactOnTable(row, linkedinId, email, authentication_token);
            } else {
                console.error('Could not find row element for refreshing contact data');
            }
        } else {
            console.error('Status cell not found for refreshing contact data');
        }

        // Update status to show success with green eye icon
        if (statusCell && statusCell.querySelector('.status-icon')) {
            statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-eye" style="color: #22c55e;"></i>';
            statusCell.querySelector('.status-icon').title = 'Profile Visited';
        }

    } catch (error) {
        console.error('Error in handleVisitProfile:', error);
        
        // Close tab if it exists and auto-close is enabled
        if (autoCloseTabs && tab) {
            try {
                await chrome.tabs.remove(tab.id);
                console.log('Tab closed after error:', tab.id);
            } catch (closeError) {
                console.warn('Error closing tab after error:', closeError);
            }
        }

        // Update status to show error
        if (statusCell && statusCell.querySelector('.status-icon')) {
            statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-times" style="color: #dc3545;"></i>';
            statusCell.querySelector('.status-icon').title = 'Error: ' + error.message;
        }

        // Re-throw the error to be handled by the caller
        throw error;
    }
}

async function generateNewMessages(actionType, messagesCell, statusCell, profileUrl, email, authentication_token) {
    const linkedinId = profileUrl.split('/in/')[1].split('/')[0];
    
    messagesCell.innerHTML = `
        <div class="spinner-container">
            <div class="loading-spinner"></div>
            <span class="spinner-timer">0</span>
        </div>
    `;

    const spinnerTimer = messagesCell.querySelector('.spinner-timer');
    let seconds = 0;
    const timerInterval = setInterval(() => {
        seconds++;
        if (spinnerTimer) {
            spinnerTimer.textContent = `${seconds}`;
        }
    }, 1000);

    try {
        const endpoint = actionType === 'promo_message' ? 'generatePromoMessages' : 'generateMessages';
        const response = await fetch(`https://www.goai-app.com/${endpoint}`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                email,
                authToken: authentication_token,
                contactId: linkedinId,
                aiPersona: 'sales',
                messageType: actionType === 'connection_message' ? 'connection-message' : 'direct-message'
            })
        });
        
        clearInterval(timerInterval);
        const data = await response.json();
        
        if (data.success === false) {
            messagesCell.innerHTML = `<span class="error-message">${data.error}</span>`;
            statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-times" style="color: #dc3545;"></i>';
            statusCell.querySelector('.status-icon').title = 'Failed to Generate Messages';
        } else {
            const messages = data.data?.messages || data.messages || [];
            const encodedMessages = encodeURIComponent(JSON.stringify(messages));
            const isConnectionRequest = actionType === 'connection_message';
            const isPromoMessage = actionType === 'promo_message';
            messagesCell.innerHTML = `
                <button class="messages-count-button ${isConnectionRequest ? 'connection-request' : ''} ${isPromoMessage ? 'promo-message' : ''}" 
                        data-messages="${encodedMessages}">
                    ${messages.length}
                </button>
            `;
            statusCell.querySelector('.status-icon').innerHTML = `<i class="fas fa-comment-dots" style="color: ${
                isConnectionRequest ? '#22c55e' : isPromoMessage ? '#f59e0b' : '#0a66c2'
            };"></i>`;
            statusCell.querySelector('.status-icon').title = 'Messages Ready';
        }
    } catch (error) {
        clearInterval(timerInterval);
        console.error('Error generating messages:', error);
        messagesCell.innerHTML = `<span class="error-message">Error</span>`;
        statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-times" style="color: #dc3545;"></i>';
        statusCell.querySelector('.status-icon').title = 'Failed to Generate Messages';
    }
}

async function processRow(row, email, authentication_token) {
    console.log('Starting processRow with:', { row, email });
    
    const actionType = row.querySelector('.action-select').value;
    console.log('Action type:', actionType);
    
    const actionSelect = row.querySelector('.action-select');
    const messagesCell = row.querySelector('.messages-cell');
    const statusCell = row.querySelector('.status-cell');
    let messageGenerationTimer;
    
    // Debug logging for DOM elements
    console.log('Found DOM elements:', {
        messagesCell: !!messagesCell,
        statusCell: !!statusCell,
        row: row.innerHTML // Show the actual HTML to verify structure
    });
    
    // Get profile URL from the data attribute
    const profilePictureElement = row.querySelector('.profile-picture');
    console.log('Profile picture element:', profilePictureElement);
    
    // Try to get profile URL from data attribute first, fallback to row data
    let profileUrl = profilePictureElement?.dataset.profileUrl;
    if (!profileUrl || profileUrl === 'undefined') {
        profileUrl = row.dataset.originalUrl;
    }
    
    console.log('Profile URL:', profileUrl);

    // Update status to show processing
    if (statusCell) {
        statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-spinner fa-spin" style="color: #0a66c2;"></i>';
        statusCell.querySelector('.status-icon').title = 'Processing';
    }

    try {
        await Promise.race([
            (async () => {
                switch (actionType) {
                    case 'skip':
                        try {
                            console.log('Skip action selected');
                            await handleSkipAction(actionType, statusCell);
                        } catch (error) {
                            console.error('Error in skip action:', error);
                            if (statusCell) {
                                statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-times" style="color: #dc3545;"></i>';
                                statusCell.querySelector('.status-icon').title = 'Error: ' + error.message;
                            }
                        }
                        await delay(200);
                        break;

                    case 'visit_profile':
                        try {
                            console.log('Visit profile action selected with URL:', profileUrl);
                            if (!profileUrl) {
                                console.error('Profile URL is missing. Row HTML:', row.outerHTML);
                                throw new Error('Profile URL is missing for visit_profile action');
                            }
                            await handleVisitProfile(actionType, profileUrl, messagesCell, statusCell, email, authentication_token);
                        } catch (error) {
                            console.error('Error in visit_profile action:', error);
                            if (statusCell) {
                                statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-times" style="color: #dc3545;"></i>';
                                statusCell.querySelector('.status-icon').title = 'Error: ' + error.message;
                            }
                        }
                        await delay(1000);
                        break;

                    case 'connection_message':
                        try {
                            // Generate new messages
                            statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-spinner fa-spin" style="color: #0a66c2;"></i>';
                            statusCell.querySelector('.status-icon').title = 'Generating Messages';
                            
                            const linkedinId = profileUrl.split('/in/')[1].split('/')[0];
                            
                            // Show generating status with timer
                            messagesCell.innerHTML = `
                                <div class="spinner-container">
                                    <div class="loading-spinner"></div>
                                    <span class="spinner-timer">0</span>
                                </div>
                            `;

                            // Start timer for generation
                            const spinnerTimer = messagesCell.querySelector('.spinner-timer');
                            let seconds = 0;
                            messageGenerationTimer = setInterval(() => {
                                seconds++;
                                if (spinnerTimer) {
                                    spinnerTimer.textContent = `${seconds}`;
                                }
                            }, 1000);

                            // Generate messages
                            const response = await fetch('https://www.goai-app.com/generateMessages', {
                                method: 'POST',
                                headers: { 'Content-Type': 'application/json' },
                                body: JSON.stringify({
                                    email,
                                    authToken: authentication_token,
                                    contactId: linkedinId,
                                    aiPersona: 'sales',
                                    messageType: 'connection-message'
                                })
                            });
                            
                            clearInterval(messageGenerationTimer);
                            const data = await response.json();
                            
                            if (!data.success) {
                                throw new Error(data.error || 'Failed to generate messages');
                            }

                            const messages = data.data?.messages || data.messages || [];
                            const encodedMessages = encodeURIComponent(JSON.stringify(messages));
                            
                            messagesCell.innerHTML = `
                                <button class="messages-count-button connection-request" 
                                        data-messages="${encodedMessages}">
                                    ${messages.length}
                                </button>
                            `;
                            statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-check" style="color: #22c55e;"></i>';
                            statusCell.querySelector('.status-icon').title = 'Messages Generated';

                        } catch (error) {
                            clearInterval(messageGenerationTimer);
                            console.error('Error generating messages:', error);
                            messagesCell.innerHTML = `<span class="error-message">Error</span>`;
                            statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-times" style="color: #dc3545;"></i>';
                            statusCell.querySelector('.status-icon').title = 'Failed to Generate Messages';
                        }
                        await delay(1000);
                        break;

                    case 'direct_message':
                        try {
                            // Generate new messages
                            statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-spinner fa-spin" style="color: #0a66c2;"></i>';
                            statusCell.querySelector('.status-icon').title = 'Generating Messages';
                            
                            const linkedinId = profileUrl.split('/in/')[1].split('/')[0];
                            
                            // Show generating status with timer
                            messagesCell.innerHTML = `
                                <div class="spinner-container">
                                    <div class="loading-spinner"></div>
                                    <span class="spinner-timer">0</span>
                                </div>
                            `;

                            // Start timer for generation
                            const spinnerTimer = messagesCell.querySelector('.spinner-timer');
                            let seconds = 0;
                            messageGenerationTimer = setInterval(() => {
                                seconds++;
                                if (spinnerTimer) {
                                    spinnerTimer.textContent = `${seconds}`;
                                }
                            }, 1000);

                            // Generate messages
                            const response = await fetch('https://www.goai-app.com/generateMessages', {
                                method: 'POST',
                                headers: { 'Content-Type': 'application/json' },
                                body: JSON.stringify({
                                    email,
                                    authToken: authentication_token,
                                    contactId: linkedinId,
                                    aiPersona: 'sales',
                                    messageType: 'direct-message'
                                })
                            });
                            
                            clearInterval(messageGenerationTimer);
                            const data = await response.json();
                            
                            if (!data.success) {
                                throw new Error(data.error || 'Failed to generate messages');
                            }

                            const messages = data.data?.messages || data.messages || [];
                            const encodedMessages = encodeURIComponent(JSON.stringify(messages));
                            
                            messagesCell.innerHTML = `
                                <button class="messages-count-button direct-message" 
                                        data-messages="${encodedMessages}">
                                    ${messages.length}
                                </button>
                            `;
                            statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-check" style="color: #22c55e;"></i>';
                            statusCell.querySelector('.status-icon').title = 'Messages Generated';

                        } catch (error) {
                            clearInterval(messageGenerationTimer);
                            console.error('Error generating messages:', error);
                            messagesCell.innerHTML = `<span class="error-message">Error</span>`;
                            statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-times" style="color: #dc3545;"></i>';
                            statusCell.querySelector('.status-icon').title = 'Failed to Generate Messages';
                        }
                        await delay(1000);
                        break;
                    case 'promo_message':
                        try {
                            // Generate new messages
                            statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-spinner fa-spin" style="color: #0a66c2;"></i>';
                            statusCell.querySelector('.status-icon').title = 'Generating Messages';
                            
                            const linkedinId = profileUrl.split('/in/')[1].split('/')[0];
                            
                            // Show generating status with timer
                            messagesCell.innerHTML = `
                                <div class="spinner-container">
                                    <div class="loading-spinner"></div>
                                    <span class="spinner-timer">0</span>
                                </div>
                            `;

                            // Start timer for generation
                            const spinnerTimer = messagesCell.querySelector('.spinner-timer');
                            let seconds = 0;
                            messageGenerationTimer = setInterval(() => {
                                seconds++;
                                if (spinnerTimer) {
                                    spinnerTimer.textContent = `${seconds}`;
                                }
                            }, 1000);

                            // Generate messages
                            const response = await fetch('https://www.goai-app.com/generatePromoMessages', {
                                method: 'POST',
                                headers: { 'Content-Type': 'application/json' },
                                body: JSON.stringify({
                                    email,
                                    authToken: authentication_token,
                                    contactId: linkedinId,
                                    aiPersona: 'sales'
                                })
                            });
                            
                            clearInterval(messageGenerationTimer);
                            const data = await response.json();
                            
                            if (!data.success) {
                                throw new Error(data.error || 'Failed to generate messages');
                            }

                            const messages = data.data?.messages || data.messages || [];
                            const encodedMessages = encodeURIComponent(JSON.stringify(messages));
                            
                            messagesCell.innerHTML = `
                                <button class="messages-count-button promo-message" 
                                        data-messages="${encodedMessages}">
                                    ${messages.length}
                                </button>
                            `;
                            statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-check" style="color: #22c55e;"></i>';
                            statusCell.querySelector('.status-icon').title = 'Messages Generated';

                        } catch (error) {
                            clearInterval(messageGenerationTimer);
                            console.error('Error generating messages:', error);
                            messagesCell.innerHTML = `<span class="error-message">Error</span>`;
                            statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-times" style="color: #dc3545;"></i>';
                            statusCell.querySelector('.status-icon').title = 'Failed to Generate Messages';
                        }
                        await delay(1000);
                        break;
                        
                    default:
                        console.error('Unknown action type:', actionType);
                        statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-times" style="color: #dc3545;"></i>';
                        statusCell.querySelector('.status-icon').title = 'Invalid Action';
                        await delay(1000);
                }
            })(),
            createTimeout(OPERATION_TIMEOUT)
        ]);
    } catch (error) {
        console.error(`Error processing ${actionType}:`, error);
        if (error.message === 'Operation timed out') {
            statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-clock" style="color: #dc3545;"></i>';
            statusCell.querySelector('.status-icon').title = 'Operation timed out';
        }
    }
}

async function showCountdown(delayCountdown) {
    delayCountdown.classList.add('active');
    for (let seconds = delaySeconds; seconds > 0; seconds--) {
        if (isAutomationStopped || isAutomationPaused) break;
        delayCountdown.textContent = `Next contact in: ${seconds}s`;
        await delay(1000);
    }
    delayCountdown.classList.remove('active');
    delayCountdown.textContent = '';
}

function resetUIState(generateButton, clearListButton, progressStatus, delayCountdown) {
    stopTimer();
    generateButton.style.display = 'block';
    clearListButton.style.display = 'block';
    resetControlButtons();
    
    if (isAutomationStopped) {
        progressStatus.textContent = 'Generation Stopped';
    } else {
        progressStatus.textContent = 'ALL Actions Complete';
        progressStatus.classList.remove('in-progress');
        progressStatus.classList.add('completed');
    }
    
    delayCountdown.classList.remove('active');
    delayCountdown.textContent = '';
}

async function generateMessages(email, authentication_token) {
    isAutomationStopped = false;
    isAutomationPaused = false;
    
    const tbody = document.querySelector('#selectedContactsTable tbody');
    const rows = tbody.getElementsByTagName('tr');
    const generateButton = document.getElementById('generateMessagesButton');
    const progressStatus = document.querySelector('.progress-status');
    const delayCountdown = document.getElementById('delayCountdown');
    
    if (!email || !authentication_token) {
        alert('Authentication credentials not found. Please log in again.');
        return;
    }

    // Get delay value from input
    const delayInput = document.getElementById('delaySeconds');
    delaySeconds = delayInput.value === '' ? 60 : parseInt(delayInput.value);

    generateButton.style.display = 'none';
    progressStatus.textContent = 'Running Actions...';
    progressStatus.classList.add('in-progress');
    createControlButtons();
    startTimer();

    const clearListButton = document.getElementById('clearListButton');
    clearListButton.style.display = 'none';

    try {
        for (let i = 0; i < rows.length; i++) {
            if (isAutomationStopped) break;
            
            while (isAutomationPaused) {
                await delay(100);
                if (isAutomationStopped) break;
            }

            console.log(`Processing contact ${i + 1} of ${rows.length}`);
            const row = rows[i];
            await processRow(row, email, authentication_token);

            // Add delay between contacts if not the last contact
            if (i < rows.length - 1 && !isAutomationStopped && delaySeconds > 0) {
                progressStatus.textContent = 'Waiting...';
                await showCountdown(delayCountdown);
                progressStatus.textContent = 'Processing...';
            }
        }
    } finally {
        resetUIState(generateButton, clearListButton, progressStatus, delayCountdown);
    }
}

export {
    generateMessages,
    isAutomationPaused,
    isAutomationStopped
}; 

// Add message listener for automation status updates
chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
    if (request.type === 'CONNECTION_REQUEST_STATUS' || 
        request.type === 'DIRECT_MESSAGE_STATUS' || 
        request.type === 'PROFILE_VISIT_STATUS') {
        // Find the row that's currently being processed
        const tbody = document.querySelector('#selectedContactsTable tbody');
        const rows = tbody.getElementsByTagName('tr');
        for (const row of rows) {
            const statusCell = row.querySelector('.status-cell');
            const statusIcon = statusCell?.querySelector('.status-icon');
            if (statusIcon?.querySelector('.fa-spinner')) {
                // This is the row being processed
                if (request.success) {
                    let color, title;
                    switch (request.type) {
                        case 'CONNECTION_REQUEST_STATUS':
                            color = '#22c55e';
                            title = 'Connection Request Sent';
                            break;
                        case 'DIRECT_MESSAGE_STATUS':
                            color = '#0a66c2';
                            title = 'Direct Message Sent';
                            break;
                        case 'PROFILE_VISIT_STATUS':
                            color = '#6366f1';
                            title = 'Profile Visited';
                            break;
                    }
                    statusIcon.innerHTML = `<i class="fas fa-check" style="color: ${color};"></i>`;
                    statusIcon.title = title;
                } else {
                    statusIcon.innerHTML = '<i class="fas fa-times" style="color: #dc3545;"></i>';
                    let actionType;
                    switch (request.type) {
                        case 'CONNECTION_REQUEST_STATUS':
                            actionType = 'Connection Request';
                            break;
                        case 'DIRECT_MESSAGE_STATUS':
                            actionType = 'Direct Message';
                            break;
                        case 'PROFILE_VISIT_STATUS':
                            actionType = 'Profile Visit';
                            break;
                    }
                    statusIcon.title = request.error || `Failed to Complete ${actionType}`;
                }
                break;
            }
        }
    }
    return true;
});