import { updateStats, updatePendingFromQueue } from './statsManager.js';

async function fetchGeneratedMessages(email, authToken, contactId, messageType) {
    try {
        const response = await fetch(
            `https://www.goai-app.com/getGeneratedMessages?email=${encodeURIComponent(email)}&authToken=${encodeURIComponent(authToken)}&contactId=${encodeURIComponent(contactId)}&messageType=${messageType === 'connection_message' ? 'connection-message' : 'direct-message'}`
        );
        
        if (response.status === 404) return null;
        if (!response.ok) throw new Error('Failed to fetch messages');
        
        const data = await response.json();
        return data.success ? data.data?.messages : null;
    } catch (error) {
        if (error.message !== 'Failed to fetch messages') {
            console.error('Error fetching messages:', error);
        }
        return null;
    }
}

function showMessagesModal(messages) {
    // Create modal if it doesn't exist
    let modal = document.getElementById('messagesModal');
    if (!modal) {
        modal = document.createElement('div');
        modal.id = 'messagesModal';
        modal.className = 'modal';
        modal.innerHTML = `
            <div class="modal-content">
                <div class="modal-header">
                    <h2>Generated Messages</h2>
                    <button class="close-button">&times;</button>
                </div>
                <div class="messagesDiv"></div>
            </div>
        `;
        document.body.appendChild(modal);

        // Add close button event listener
        modal.querySelector('.close-button').addEventListener('click', () => {
            modal.style.display = 'none';
        });

        // Close modal when clicking outside
        window.addEventListener('click', (event) => {
            if (event.target === modal) {
                modal.style.display = 'none';
            }
        });
    }

    // Update messages content
    const messagesDiv = modal.querySelector('.messagesDiv');
    messagesDiv.innerHTML = messages.map((msg, index) => `
        <div class="messageDiv">
            <div class="message-content">
                <div class="message-header">Option ${index + 1}</div>
                <div class="message-body">${msg.messageBody}</div>
                ${msg.reasons ? `
                    <div class="message-reasons">
                        ${msg.reasons.map(reason => `
                            <div class="reason">${reason}</div>
                        `).join('')}
                    </div>
                ` : ''}
            </div>
            <div class="message-actions">
                <button class="message-action-button copy-button" 
                        data-message="${encodeURIComponent(msg.messageBody)}"
                        data-tooltip="Copy">
                    <span>Copy</span>
                </button>
                <button class="message-action-button edit-button" 
                        data-message="${encodeURIComponent(msg.messageBody)}"
                        data-tooltip="Edit">
                    <span>Edit</span>
                </button>
                <button class="message-action-button change-button" 
                        data-message="${encodeURIComponent(msg.messageBody)}"
                        data-tooltip="Change!">
                    <span>Change</span>
                </button>
                <button class="message-action-button use-button" 
                        data-message="${encodeURIComponent(msg.messageBody)}"
                        data-tooltip="Send Message">
                    <span>Send</span>
                </button>                    
            </div>
        </div>
    `).join('');

    setupModalEventListeners(modal);
    modal.style.display = 'block';
}

function setupModalEventListeners(modal) {
    // Copy button handlers
    modal.querySelectorAll('.copy-button').forEach(button => {
        button.addEventListener('click', async (e) => {
            const message = decodeURIComponent(e.currentTarget.dataset.message);
            try {
                await navigator.clipboard.writeText(message);
                const originalText = button.innerHTML;
                button.innerHTML = '✓ Copied!';
                setTimeout(() => {
                    button.innerHTML = originalText;
                }, 2000);
            } catch (err) {
                console.error('Failed to copy:', err);
            }
        });
    });

    // Edit button handlers
    modal.querySelectorAll('.edit-button').forEach(button => {
        button.addEventListener('click', (e) => {
            const message = decodeURIComponent(e.currentTarget.dataset.message);
            const messageDiv = e.currentTarget.closest('.messageDiv');
            const messageBody = messageDiv.querySelector('.message-body');
            
            messageBody.innerHTML = `
                <textarea class="edit-textarea">${message}</textarea>
                <div class="edit-actions">
                    <button class="save-send-edit">Save & Send</button>
                    <button class="save-edit">Save</button>
                    <button class="cancel-edit">Cancel</button>
                </div>
            `;
            
            const textarea = messageBody.querySelector('.edit-textarea');
            textarea.focus();
            
            setupEditHandlers(messageBody, messageDiv, message);
        });
    });

    // Use button handlers
    modal.querySelectorAll('.use-button').forEach(button => {
        button.addEventListener('click', (e) => {
            const message = decodeURIComponent(e.currentTarget.dataset.message);
            addToSendList(message, e.currentTarget.closest('.messageDiv'));
            modal.style.display = 'none';
        });
    });

    // Add click handler for message bodies
    modal.querySelectorAll('.message-body').forEach(messageBody => {
        messageBody.addEventListener('click', (e) => {
            // Only handle direct clicks on the message body, not its children
            if (e.target === messageBody && !messageBody.querySelector('.edit-textarea')) {
                const messageDiv = messageBody.closest('.messageDiv');
                const originalMessage = messageBody.textContent;
                
                messageBody.innerHTML = `
                    <textarea class="edit-textarea">${originalMessage}</textarea>
                    <div class="edit-actions">
                        <button class="save-send-edit">Save & Send</button>
                        <button class="save-edit">Save</button>
                        <button class="cancel-edit">Cancel</button>
                    </div>
                `;
                
                const textarea = messageBody.querySelector('.edit-textarea');
                textarea.focus();
                
                setupEditHandlers(messageBody, messageDiv, originalMessage);
            }
        });
    });

    // Update change button handlers
    modal.querySelectorAll('.change-button').forEach(button => {
        button.addEventListener('click', async (e) => {
            const messageDiv = e.target.closest('.messageDiv');
            const originalMessageBody = messageDiv.querySelector('.message-body').textContent;
            
            // Store the original message and its button for later use
            const messagesCountButtons = document.querySelectorAll('#selectedContactsTable .messages-count-button');
            let originalButton;
            let originalMessages;
            let contactProfileUrl;

            // Find the matching button and store its data
            for (const btn of messagesCountButtons) {
                try {
                    const messages = JSON.parse(decodeURIComponent(btn.dataset.messages));
                    if (messages.some(msg => msg.messageBody === originalMessageBody)) {
                        originalButton = btn;
                        originalMessages = messages;
                        contactProfileUrl = btn.closest('.actionListContacts-row').querySelector('.profile-picture').dataset.profileUrl;
                        break;
                    }
                } catch (error) {
                    console.error('Error parsing messages for button:', error);
                }
            }

            if (!originalButton || !contactProfileUrl) {
                console.error('Could not find original contact for message:', originalMessageBody);
                return;
            }

            // Hide all other messageDivs
            const allMessageDivs = modal.querySelectorAll('.messageDiv');
            allMessageDivs.forEach(div => {
                if (div !== messageDiv) {
                    div.style.display = 'none';
                }
            });

            // Create and insert modifier buttons div
            const modifierButtonsDiv = document.createElement('div');
            modifierButtonsDiv.className = 'modifier-buttons-container';
            modifierButtonsDiv.innerHTML = `
                <button class="modifier-btn shorter">✂️ Shorter</button>
                <button class="modifier-btn funnier">😄 Funnier</button>
                <button class="modifier-btn less-direct">🎭 Less Direct</button>
                <button class="modifier-btn more-direct">🎯 More Direct</button>
                <button class="modifier-btn professional">🤝 Professional</button>
                <button class="modifier-btn creative">🎨 Creative</button>
                <button class="modifier-btn enthusiastic">🌟 Enthusiastic</button>
                <button class="modifier-btn casual">🗣️ Casual</button>
                <button class="modifier-btn rephrase">🔄 Rephrase</button>
                <button class="modifier-btn technical">🤓 Technical</button>
                <button class="modifier-btn playful">🎪 Playful</button>
                <button class="modifier-btn goal-oriented">🎯 Goal-oriented</button>
                <button class="modifier-btn data-driven">📊 Data-driven</button>
                <button class="modifier-btn custom" disabled>✨ Custom</button>
            `;

            // Add click listeners to modifier buttons
            modifierButtonsDiv.querySelectorAll('.modifier-btn').forEach(modifierBtn => {
                modifierBtn.addEventListener('click', async (e) => {
                    const buttonText = e.target.innerText;
                    const revisionType = buttonText.split(' ')[1]; // Get the revision type

                    // Show loading state
                    messageDiv.querySelector('.message-body').innerHTML = `
                        <div class="spinner-container">
                            <div class="loading-spinner"></div>
                            <span>Modifying message...</span>
                        </div>
                    `;

                    try {
                        const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);

                        // Use the reviseMessage endpoint
                        const response = await fetch('https://www.goai-app.com/reviseMessage', {
                            method: 'POST',
                            headers: { 'Content-Type': 'application/json' },
                            body: JSON.stringify({
                                email,
                                authToken: authentication_token,
                                contactId: contactProfileUrl.split('/in/')[1].split('/')[0],
                                currentMessage: originalMessageBody,
                                revision: revisionType
                            })
                        });

                        const data = await response.json();
                        if (!data.success) {
                            throw new Error(data.error || 'Failed to revise message');
                        }

                        // Get the messages from the response
                        const newMessages = data.messages;

                        // Update the original messages array in the messages count button
                        const messageIndex = originalMessages.findIndex(msg => msg.messageBody === originalMessageBody);
                        if (messageIndex !== -1) {
                            // Replace the old messages with the new ones
                            originalButton.dataset.messages = encodeURIComponent(JSON.stringify(newMessages));
                        }

                        // Close the current modal
                        const currentModal = document.getElementById('messagesModal');
                        if (currentModal) {
                            currentModal.remove();
                        }

                        // Show new modal with revised messages
                        showMessagesModal(newMessages);

                    } catch (error) {
                        console.error('Error revising message:', error);
                        messageDiv.querySelector('.message-body').innerHTML = `
                            <div class="error-message">Failed to revise message: ${error.message}</div>
                        `;
                    }
                });
            });

            // Insert modifier buttons
            messageDiv.insertAdjacentElement('afterend', modifierButtonsDiv);
        });
    });
}

function setupEditHandlers(messageBody, messageDiv, originalMessage) {
    messageBody.querySelector('.save-send-edit').addEventListener('click', async () => {
        const textarea = messageBody.querySelector('.edit-textarea');
        const newMessage = textarea.value;
        messageBody.innerHTML = newMessage;
        
        // Find the original messages count button and update its dataset
        const messagesCountButtons = document.querySelectorAll('#selectedContactsTable .messages-count-button');
        for (const btn of messagesCountButtons) {
            try {
                const messages = JSON.parse(decodeURIComponent(btn.dataset.messages));
                const messageIndex = messages.findIndex(msg => msg.messageBody === originalMessage);
                if (messageIndex !== -1) {
                    // Update the message in the array
                    messages[messageIndex].messageBody = newMessage;
                    // Update the dataset with the modified messages array
                    btn.dataset.messages = encodeURIComponent(JSON.stringify(messages));
                    break;
                }
            } catch (error) {
                console.error('Error updating messages dataset:', error);
            }
        }
        
        // Update all action buttons in this messageDiv with new message
        messageDiv.querySelectorAll('.message-action-button').forEach(btn => {
            btn.dataset.message = encodeURIComponent(newMessage);
        });
        
        // Add to send list and close modal
        addToSendList(newMessage, messageDiv);
        const modal = messageBody.closest('#messagesModal');
        if (modal) {
            modal.style.display = 'none';
        }
    });
    
    messageBody.querySelector('.save-edit').addEventListener('click', () => {
        const textarea = messageBody.querySelector('.edit-textarea');
        const newMessage = textarea.value;
        messageBody.innerHTML = newMessage;
        
        // Find the original messages count button and update its dataset
        const messagesCountButtons = document.querySelectorAll('#selectedContactsTable .messages-count-button');
        for (const btn of messagesCountButtons) {
            try {
                const messages = JSON.parse(decodeURIComponent(btn.dataset.messages));
                const messageIndex = messages.findIndex(msg => msg.messageBody === originalMessage);
                if (messageIndex !== -1) {
                    // Update the message in the array
                    messages[messageIndex].messageBody = newMessage;
                    // Update the dataset with the modified messages array
                    btn.dataset.messages = encodeURIComponent(JSON.stringify(messages));
                    break;
                }
            } catch (error) {
                console.error('Error updating messages dataset:', error);
            }
        }
        
        // Update all action buttons in this messageDiv with new message
        messageDiv.querySelectorAll('.message-action-button').forEach(btn => {
            btn.dataset.message = encodeURIComponent(newMessage);
        });
    });
    
    messageBody.querySelector('.cancel-edit').addEventListener('click', () => {
        messageBody.innerHTML = originalMessage;
    });
}

async function addToSendList(message, messageDiv) {
    console.log('Adding message to send list:', { message: message.substring(0, 50) + '...' });
    
    // First find the modal that contains this message
    const modal = messageDiv.closest('#messagesModal');
    if (!modal) {
        console.error('Could not find messages modal');
        return;
    }

    // Get all messages count buttons from the contacts table
    const messagesCountButtons = document.querySelectorAll('#selectedContactsTable .messages-count-button');
    console.log('Found message count buttons:', messagesCountButtons.length);

    let contactRow;
    let matchingMessagesButton;

    // Find the matching messages count button by comparing all messages in its dataset
    for (const btn of messagesCountButtons) {
        try {
            const messages = JSON.parse(decodeURIComponent(btn.dataset.messages));
            console.log('Checking messages in button:', messages.length);
            
            // Check if any message in the button's dataset matches our message
            if (messages.some(msg => msg.messageBody === message)) {
                contactRow = btn.closest('tr');
                matchingMessagesButton = btn;
                break;
            }

            // If no exact match found, try to find by comparing without whitespace
            const normalizedMessage = message.replace(/\s+/g, '');
            if (messages.some(msg => msg.messageBody.replace(/\s+/g, '') === normalizedMessage)) {
                contactRow = btn.closest('tr');
                matchingMessagesButton = btn;
                console.log('Found matching message in button');
                break;
            }
        } catch (error) {
            console.error('Error parsing messages for button:', error);
            continue;
        }
    }

    if (!contactRow || !matchingMessagesButton) {
        console.error('Could not find matching contact row for message');
        return;
    }

    console.log('Found matching contact row:', contactRow);

    // Disable the messages count button and update its appearance
    matchingMessagesButton.disabled = true;
    matchingMessagesButton.style.opacity = '0.5';
    matchingMessagesButton.style.cursor = 'not-allowed';
    matchingMessagesButton.title = 'Message already added to Send List';
    console.log('Found matching contact row');

    // Get all contact info from the row
    const contactInfo = contactRow.querySelector('.contact-info');
    const contactName = contactInfo.querySelector('.contact-name').textContent;
    const contactDetails = contactInfo.querySelector('.contact-details').innerHTML;
    const action = contactRow.querySelector('.action-select').value;
    const profilePicture = contactRow.querySelector('.profile-picture');
    const connectionStatus = contactRow.querySelector('.connection-status-overlay').outerHTML;
    const linkedinId = profilePicture.getAttribute('data-linkedin-id');
    const contactId = profilePicture.getAttribute('data-contact-id');
    const profileUrl = profilePicture.getAttribute('data-profile-url');
    const statusCell = contactRow.querySelector('.status-cell');
    
    console.log('[DEBUG] addToSendList - Contact details:', { 
        contactName, 
        action, 
        contactId,
        linkedinId,
        profileUrl,
        profilePicture: {
            dataset: profilePicture.dataset,
            attributes: Array.from(profilePicture.attributes).map(attr => ({ name: attr.name, value: attr.value }))
        }
    });

    if (!contactId && !linkedinId) {
        console.error('[DEBUG] addToSendList - No ContactID or LinkedinID found:', {
            profilePictureDataset: profilePicture.dataset,
            profilePictureAttributes: Array.from(profilePicture.attributes).map(attr => ({ name: attr.name, value: attr.value }))
        });
        return;
    }

    // Update the status in the original row
    statusCell.querySelector('.status-icon').innerHTML = '<i class="fas fa-envelope" style="color: #6b7280;"></i>';
    statusCell.querySelector('.status-icon').title = 'Message Added to Queue';
    contactRow.style.backgroundColor = '#e3e1e1';

    // Create message data object for storage
    const messageData = {
        queueNumber: document.querySelector('#sendListTable tbody').children.length + 1,
        status: 'pending',
        contactName,
        contactDetails,
        action,
        friendlyAction: {
            'direct_message': 'Direct Message',
            'post_reply': 'Post Reply',
            'connection_message': 'Connection Message',
            'visit_profile': 'Visit Profile',
            'skip': 'Skip'
        }[action] || action,
        message,
        profileUrl,
        contactId: contactId || undefined,  // Store as undefined instead of null
        linkedinId: linkedinId || undefined,  // Store as undefined instead of null
        connectionStatus
    };

    console.log('[DEBUG] addToSendList - Created message data:', { 
        queueNumber: messageData.queueNumber,
        status: messageData.status,
        action: messageData.action,
        contactId: messageData.contactId,
        linkedinId: messageData.linkedinId,
        messageData
    });

    // Add to Chrome storage
    try {
        const { sendListMessages = [] } = await chrome.storage.local.get('sendListMessages');
        
        // Ensure IDs are properly set
        const messageToStore = {
            ...messageData,
            contactId: contactId || undefined,  // Don't store null or 'undefined' string
            linkedinId: linkedinId || undefined  // Don't store null or 'undefined' string
        };
        
        console.log('[DEBUG] Storing message with IDs:', {
            contactId: messageToStore.contactId,
            linkedinId: messageToStore.linkedinId
        });
        
        sendListMessages.push(messageToStore);
        await chrome.storage.local.set({ sendListMessages });
        console.log('[DEBUG] Added message to storage, total messages:', sendListMessages.length);
    } catch (error) {
        console.error('[DEBUG] Error storing message in Chrome storage:', error);
    }

    // Add to Send List table
    const sendListTbody = document.querySelector('#sendListTable tbody');
    if (!sendListTbody) {
        console.error('Could not find send list table body');
        return;
    }

    const newRow = document.createElement('tr');
    newRow.classList.add('actionListContacts-row');
    const queueNumber = sendListTbody.children.length + 1;
    
    console.log('[DEBUG] Adding new row to table with queue number:', queueNumber);
    
    newRow.innerHTML = `
        <td class="send-table-cell send-queue-cell">${queueNumber}</td>
        <td class="send-table-cell send-status-cell">
            <i class="fas fa-clock" style="color: #f59e0b;"></i> Pending
        </td>
        <td class="send-table-cell send-contact-cell">
            <div id="sendListPictureDiv">
                <div class="profile-picture-cell">
                    <img src="${chrome.runtime.getURL('icons/default.jpg')}" 
                        class="profile-picture clickable" 
                        alt="Profile"
                        data-profile-url="${profileUrl}"
                        ${contactId ? `data-contact-id="${contactId}"` : ''}
                        ${linkedinId ? `data-linkedin-id="${linkedinId}"` : ''}
                        title="Click to open profile"
                        style="width: 40px; height: 40px; border-radius: 50%; object-fit: cover; cursor: pointer;">
                        ${connectionStatus}
                </div>
                <div>
                    <div class="contact-name">${contactName}</div>
                    <div class="contact-details">
                        ${contactDetails}
                    </div>
                </div>
            </div>
        </td>
        <td class="send-table-cell send-action-cell">${messageData.friendlyAction}</td>
        <td class="send-table-cell send-message-cell">
            <div class="messageContainerDiv">${message}</div>
        </td>
    `;
    
    // Add click handler for queue cell
    const queueCell = newRow.querySelector('.send-queue-cell');
    if (queueCell) {
        queueCell.clickHandler = (e) => showContextMenu(e, queueNumber);
        queueCell.addEventListener('click', queueCell.clickHandler);
    }

    // Add the row to the table
    sendListTbody.appendChild(newRow);

    // Then fetch and update the profile picture
    try {
        const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);

        const response = await fetch(
            `https://www.goai-app.com/getProfilePicture?email=${encodeURIComponent(email)}&authToken=${encodeURIComponent(authentication_token)}&contactId=${encodeURIComponent(contactId)}`
        );
        
        if (response.ok) {
            const blob = await response.blob();
            const url = URL.createObjectURL(blob);
            const newProfilePicture = newRow.querySelector('.profile-picture');
            newProfilePicture.src = url;
            newProfilePicture.onload = () => URL.revokeObjectURL(url);
        }
    } catch (error) {
        console.error('Error loading profile picture:', error);
        // Keep default image if fetch fails
    }
    
    // Add click event listener for the profile picture
    const newProfilePicture = newRow.querySelector('.profile-picture');
    newProfilePicture.addEventListener('click', () => {
        const profileUrl = newProfilePicture.dataset.profileUrl;
        if (profileUrl) {
            chrome.tabs.create({ 
                url: profileUrl,
                active: false
            });
        }
    });
    
    // Update the Send List count in the tab
    const tabSendCount = document.getElementById('tabSendCount');
    if (tabSendCount) {
        tabSendCount.textContent = queueNumber;
    }
    
    // Show success notification
    const successMessage = document.createElement('div');
    successMessage.className = 'success-notification';
    successMessage.textContent = 'Added to Send List';
    document.body.appendChild(successMessage);
    
    // Remove notification after 2 seconds
    setTimeout(() => {
        successMessage.remove();
    }, 2000);

    // After adding to send list, update stats
    const messageType = action; // 'connection_message' or 'direct_message'
    await updateStats(messageType, 'pending');
}

// Add new function to load messages from storage
async function loadSendListFromStorage() {
    try {
        const { sendListMessages = [] } = await chrome.storage.local.get('sendListMessages');
        const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
        const sendListTbody = document.querySelector('#sendListTable tbody');
        
        if (sendListTbody) {
            // Clear existing rows
            sendListTbody.innerHTML = '';
            
            // Add each message from storage
            for (const messageData of sendListMessages) {
                const newRow = document.createElement('tr');
                newRow.classList.add('actionListContacts-row');
                // Add data-queue-number attribute to the row
                newRow.setAttribute('data-queue-number', messageData.queueNumber);
                newRow.innerHTML = `
                    <td class="send-table-cell send-queue-cell">${messageData.queueNumber}</td>
                    <td class="send-table-cell send-status-cell">
                        ${(() => {
                            switch (messageData.status) {
                                case 'connection_sent':
                                    return '<i class="fas fa-envelope-open-text" style="color: #22c55e;"></i> Sent';
                                case 'message_sent':
                                    return '<i class="fas fa-envelope-open-text" style="color: #0a66c2;"></i> Sent';
                                case 'error':
                                    return '<i class="fas fa-times" style="color: #dc3545;"></i> Failed';
                                default:
                                    return '<i class="fas fa-clock" style="color: #f59e0b;"></i> Pending';
                            }
                        })()}
                    </td>
                    <td class="send-table-cell send-contact-cell">
                        <div style="display: flex; align-items: center; gap: 12px;">
                            <div class="profile-picture-cell">
                                <img src="${chrome.runtime.getURL('icons/default.jpg')}" 
                                    class="profile-picture clickable" 
                                    alt="Profile"
                                    data-profile-url="${messageData.profileUrl}"
                                    ${messageData.contactId ? `data-contact-id="${messageData.contactId}"` : ''}
                                    ${messageData.linkedinId ? `data-linkedin-id="${messageData.linkedinId}"` : ''}
                                    title="Click to open profile"
                                    style="width: 40px; height: 40px; border-radius: 50%; object-fit: cover; cursor: pointer;">
                                    ${messageData.connectionStatus}
                            </div>
                            <div>
                                <div class="contact-name">${messageData.contactName}</div>
                                <div class="contact-details">
                                    ${messageData.contactDetails}
                                </div>
                            </div>
                        </div>
                    </td>
                    <td class="send-table-cell send-action-cell">${messageData.friendlyAction}</td>
                    <td class="send-table-cell send-message-cell">
                        <div class="messageContainerDiv">${messageData.message}</div>
                    </td>
                `;
                
                // Add click handler for queue cell
                const queueCell = newRow.querySelector('.send-queue-cell');
                if (queueCell) {
                    queueCell.clickHandler = (e) => showContextMenu(e, messageData.queueNumber);
                    queueCell.addEventListener('click', queueCell.clickHandler);
                }
                
                // Add click event listener for the profile picture
                const profilePicture = newRow.querySelector('.profile-picture');
                if (profilePicture) {
                    profilePicture.addEventListener('click', () => {
                        const profileUrl = profilePicture.dataset.profileUrl;
                        if (profileUrl) {
                            chrome.tabs.create({ 
                                url: profileUrl,
                                active: false
                            });
                        }
                    });

                    // Fetch and update profile picture
                    (async () => {
                        try {
                            const contactId = messageData.contactId;
                            const linkedinId = messageData.linkedinId;
                            
                            console.log('[DEBUG] Attempting to load profile picture:', {
                                contactId,
                                linkedinId,
                                email,
                                contactName: messageData.contactName,
                                messageData,
                                dataset: profilePicture.dataset
                            });

                            // Skip if both IDs are invalid
                            if ((!contactId || contactId === 'undefined') && (!linkedinId || linkedinId === 'undefined')) {
                                console.error('[DEBUG] No valid ContactID or LinkedinID available for profile picture');
                                return;
                            }

                            // Use contactId if available, otherwise use linkedinId
                            const idToUse = (contactId && contactId !== 'undefined') ? contactId : linkedinId;

                            const response = await fetch(
                                `https://www.goai-app.com/getProfilePicture?email=${encodeURIComponent(email)}&authToken=${encodeURIComponent(authentication_token)}&contactId=${encodeURIComponent(idToUse)}`
                            );
                            
                            if (response.ok) {
                                const blob = await response.blob();
                                const url = URL.createObjectURL(blob);
                                profilePicture.src = url;
                                profilePicture.onload = () => URL.revokeObjectURL(url);
                                console.log('[DEBUG] Successfully loaded profile picture:', {
                                    idUsed: idToUse,
                                    success: true
                                });
                            } else {
                                console.error('[DEBUG] Failed to load profile picture:', {
                                    status: response.status,
                                    statusText: response.statusText,
                                    idUsed: idToUse
                                });
                            }
                        } catch (error) {
                            console.error('[DEBUG] Error loading profile picture:', error, {
                                contactId: messageData.contactId,
                                linkedinId: messageData.linkedinId,
                                dataset: profilePicture.dataset
                            });
                            // Keep default image if fetch fails
                        }
                    })();
                }
                
                sendListTbody.appendChild(newRow);
            }
            
            // Update counters after loading all messages
            updateSendListCounters();
        }
    } catch (error) {
        console.error('Error loading messages from Chrome storage:', error);
    }
}

// Add function to remove message from storage and UI
async function removeFromSendList(queueNumber) {
    try {
        // Get current messages from storage
        const { sendListMessages = [] } = await chrome.storage.local.get('sendListMessages');
        
        // Find and remove the message with matching queue number
        const updatedMessages = sendListMessages.filter(msg => msg.queueNumber !== queueNumber);
        
        // Update queue numbers in storage
        updatedMessages.forEach((msg, index) => {
            msg.queueNumber = index + 1;
        });
        
        // Update storage with reordered messages
        await chrome.storage.local.set({ sendListMessages: updatedMessages });
        
        // Update UI
        const sendListTbody = document.querySelector('#sendListTable tbody');
        const tabSendCount = document.getElementById('tabSendCount');
        
        if (sendListTbody) {
            // Find and remove the row
            const rowToRemove = Array.from(sendListTbody.children).find(row => {
                const queueCell = row.querySelector('.send-queue-cell');
                return queueCell && queueCell.textContent.trim() === queueNumber.toString();
            });
            
            if (rowToRemove) {
                rowToRemove.remove();
                
                // Update queue numbers for remaining rows
                Array.from(sendListTbody.children).forEach((row, index) => {
                    const queueCell = row.querySelector('.send-queue-cell');
                    if (queueCell) {
                        queueCell.textContent = (index + 1).toString();
                        
                        // Update click handler with new queue number
                        queueCell.removeEventListener('click', queueCell.clickHandler);
                        queueCell.clickHandler = (e) => showContextMenu(e, index + 1);
                        queueCell.addEventListener('click', queueCell.clickHandler);
                    }
                });
                
                // Update tab count and counters
                updateSendListCounters();
            }
        }
    } catch (error) {
        console.error('Error removing message from send list:', error);
    }
}

// Add function to handle context menu
function showContextMenu(event, queueNumber) {
    event.preventDefault();
    console.log('Context menu opened for queue number:', queueNumber);
    
    // Remove any existing context menus
    const existingMenu = document.querySelector('.context-menu');
    if (existingMenu) {
        existingMenu.remove();
    }
    
    // Create context menu
    const contextMenu = document.createElement('div');
    contextMenu.className = 'context-menu';
    contextMenu.innerHTML = `
        <div class="context-menu-item" data-action="retry">
            <i class="fas fa-redo"></i>
            Retry
        </div>
        <div class="context-menu-item" data-action="delete">
            <i class="fas fa-trash-alt"></i>
            Delete
        </div>
    `;
    
    // Position the menu
    const rect = event.target.getBoundingClientRect();
    contextMenu.style.top = `${rect.bottom + window.scrollY}px`;
    contextMenu.style.left = `${rect.left + window.scrollX}px`;
    
    // Add click handler for menu items
    contextMenu.addEventListener('click', async (e) => {
        const menuItem = e.target.closest('.context-menu-item');
        if (menuItem) {
            const action = menuItem.dataset.action;
            console.log('Context menu action clicked:', action);
            
            if (action === 'delete') {
                await removeFromSendList(queueNumber);
            } else if (action === 'retry') {
                console.log('Processing retry for queue number:', queueNumber);
                
                // First try by data-queue-number
                let row = document.querySelector(`tr[data-queue-number="${queueNumber}"]`);
                
                // If not found, try finding by queue cell content
                if (!row) {
                    console.log('Row not found by data-queue-number, trying queue cell content');
                    const rows = document.querySelectorAll('#sendListTable tr');
                    row = Array.from(rows).find(r => {
                        const queueCell = r.querySelector('.send-queue-cell');
                        return queueCell && queueCell.textContent.trim() === queueNumber.toString();
                    });
                }
                
                console.log('Found row:', row ? 'yes' : 'no');
                
                if (row) {
                    const statusCell = row.querySelector('.send-status-cell');
                    console.log('Found status cell:', statusCell ? 'yes' : 'no');
                    console.log('Current status cell HTML:', statusCell?.innerHTML);
                    
                    if (statusCell) {
                        console.log('Updating UI status to Pending');
                        // Update UI
                        statusCell.innerHTML = '<i class="fas fa-clock" style="color: #0a66c2;"></i> Pending';
                        
                        // Update storage
                        try {
                            console.log('Fetching current messages from storage');
                            const { sendListMessages = [] } = await chrome.storage.local.get('sendListMessages');
                            console.log('Current messages in storage:', sendListMessages);
                            
                            const updatedMessages = sendListMessages.map(msg => {
                                if (msg.queueNumber === parseInt(queueNumber)) {
                                    console.log('Found matching message in storage:', msg);
                                    return { ...msg, status: 'pending' };
                                }
                                return msg;
                            });
                            
                            console.log('Saving updated messages to storage');
                            await chrome.storage.local.set({ sendListMessages: updatedMessages });
                            console.log('Storage update complete');
                        } catch (error) {
                            console.error('Error updating message status in storage:', error);
                        }
                        
                        console.log('Updating queue counters');
                        await updatePendingFromQueue();
                        console.log('Queue counters updated');
                    }
                }
            }
            contextMenu.remove();
        }
    });
    
    // Add click handler to close menu when clicking outside
    document.addEventListener('click', function closeMenu(e) {
        if (!contextMenu.contains(e.target) && !e.target.classList.contains('send-queue-cell')) {
            contextMenu.remove();
            document.removeEventListener('click', closeMenu);
        }
    });
    
    document.body.appendChild(contextMenu);
}

// Function to update the send list counters in the UI
function updateSendListCounters() {
    console.log('messageManager: Triggering counter update...');
    updatePendingFromQueue().catch(err => 
        console.error('messageManager: Error updating counters:', err)
    );
}

// Function to clear messages based on status
async function clearMessagesByStatus(status = null) {
    // Get confirmation message based on status
    let confirmMessage;
    switch(status) {
        case null:
            confirmMessage = 'Are you sure you want to clear ALL messages from the send list?';
            break;
        case 'sent':
            confirmMessage = 'Are you sure you want to clear all SENT messages from the send list?';
            break;
        case 'pending':
            confirmMessage = 'Are you sure you want to clear all PENDING messages from the send list?';
            break;
        case 'error':
            confirmMessage = 'Are you sure you want to clear all FAILED messages from the send list?';
            break;
    }

    // Ask for confirmation
    if (!confirm(confirmMessage)) {
        console.log('Clear operation cancelled by user');
        return;
    }

    try {
        console.log('Clearing messages with status:', status);
        const { sendListMessages = [] } = await chrome.storage.local.get('sendListMessages');
        
        // Filter messages based on status
        const updatedMessages = status === null ? [] : sendListMessages.filter(msg => {
            switch(status) {
                case 'sent':
                    return !['connection_sent', 'message_sent'].includes(msg.status);
                case 'pending':
                    return msg.status !== 'pending';
                case 'error':
                    return msg.status !== 'error';
                default:
                    return true;
            }
        });

        // Update queue numbers
        updatedMessages.forEach((msg, index) => {
            msg.queueNumber = index + 1;
        });

        // Save to storage
        await chrome.storage.local.set({ sendListMessages: updatedMessages });
        
        // Reload the table
        await loadSendListFromStorage();
        
        // Update counters
        await updatePendingFromQueue();
        
        console.log('Clear operation completed');
    } catch (error) {
        console.error('Error clearing messages:', error);
    }
}

// Set up mutation observer to watch for table changes
const observer = new MutationObserver((mutations) => {
    for (const mutation of mutations) {
        if (mutation.type === 'childList' || mutation.type === 'characterData') {
            console.log('messageManager: Table mutation detected:', mutation.type);
            updateSendListCounters();
            break;
        }
    }
});

// Start observing the table for changes
function startObservingTable() {
    console.log('messageManager: Starting table observation...');
    const tbody = document.querySelector('#sendListTable tbody');
    if (tbody) {
        console.log('messageManager: Found table body, setting up observer');
        observer.observe(tbody, { 
            childList: true, 
            subtree: true,
            characterData: true
        });
        
        // Set up clear buttons
        document.getElementById('clearAllBtn')?.addEventListener('click', () => clearMessagesByStatus(null));
        document.getElementById('clearSentBtn')?.addEventListener('click', () => clearMessagesByStatus('sent'));
        document.getElementById('clearPendingBtn')?.addEventListener('click', () => clearMessagesByStatus('pending'));
        document.getElementById('clearErrorBtn')?.addEventListener('click', () => clearMessagesByStatus('error'));
        
        // Initial count update
        updateSendListCounters();
    } else {
        console.error('messageManager: Could not find table body');
    }
}

// Call this when the document is ready
document.addEventListener('DOMContentLoaded', startObservingTable);

export {
    fetchGeneratedMessages,
    showMessagesModal,
    loadSendListFromStorage,
    removeFromSendList,
    showContextMenu,
    updateSendListCounters,
    clearMessagesByStatus
};