import { updateStatus } from './uiManager.js';
import { delay } from './utils.js';
import { updateStats } from './statsManager.js';

// Queue state management
let isQueueProcessing = false;

function extractRowData(row) {
    const statusCell = row.querySelector('.send-status-cell');
    const contactCell = row.querySelector('.send-contact-cell');
    const actionCell = row.querySelector('.send-action-cell');
    const messageCell = row.querySelector('.send-message-cell');
    const messageContainer = messageCell?.querySelector('.messageContainerDiv');
    const profileUrl = contactCell?.querySelector('.profile-picture')?.dataset?.profileUrl;

    return {
        statusCell,
        contactCell,
        action: actionCell?.textContent?.trim(),
        message: messageContainer?.textContent?.trim(),
        profileUrl
    };
}

function validateRowData(statusCell, contactCell, action, message, profileUrl) {
    if (!statusCell || !contactCell || !action || !message || !profileUrl) {
        console.error('Missing required data in row:', {
            hasStatusCell: !!statusCell,
            hasContactCell: !!contactCell,
            hasAction: !!action,
            hasMessage: !!message,
            hasProfileUrl: !!profileUrl
        });
        return false;
    }
    return true;
}

function logContactDetails(row, contactCell, action, message, profileUrl, statusCell) {
    console.log('Send Queue - Processing contact:', {
        queuePosition: 1,
        contactName: contactCell.querySelector('.contact-name')?.textContent || 'Unknown',
        contactDetails: contactCell.querySelector('.contact-details')?.innerHTML || 'No details',
        action: action,
        message: message,
        profileUrl: profileUrl,
        currentStatus: statusCell.innerHTML
    });
}

async function createProfileTab(profileUrl) {
    // Create tab in foreground
    const newTab = await chrome.tabs.create({ 
        url: profileUrl,
        active: true
    });

    // Get the current window to find the contactAutomator tab
    const currentWindow = await chrome.windows.getCurrent();
    const tabs = await chrome.tabs.query({ windowId: currentWindow.id });
    
    // Find the contactAutomator.html tab
    const automatorTab = tabs.find(tab => tab.url.endsWith('contactAutomator.html'));
    
    if (automatorTab) {
        // Keep the new tab in focus for 5 seconds
        await delay(5000);
        // Switch focus back to the contactAutomator tab
        await chrome.tabs.update(automatorTab.id, { active: true });
    }

    return newTab;
}

async function processTab(tab, action, message) {
    await delay(3000); // Wait for page load

    const response = await chrome.tabs.sendMessage(tab.id, {
        action: 'sendMessage',
        messageType: action.toLowerCase().replace(' ', '-'),
        message: message
    });

    if (!response.success) {
        throw new Error(response.error || 'Failed to send message');
    }

    return response;
}

async function handleAutoClose(tab) {
    const { autoCloseTabs = true } = await chrome.storage.local.get('autoCloseTabs');
    if (autoCloseTabs) {
        await chrome.tabs.remove(tab.id);
    }
}

async function moveToNextContact() {
    if (isQueueProcessing) {
        processSendQueue();
    }
}

function setupQueueToggleListener(enableQueueToggle, sendListTab) {
    console.log('Found enableQueueToggle element, current state:', enableQueueToggle.checked);

    enableQueueToggle.addEventListener('change', (e) => {
        isQueueProcessing = e.target.checked;
        console.log('Queue toggle changed:', {
            isQueueProcessing,
            checked: e.target.checked
        });
        
        if (isQueueProcessing) {
            console.log('Queue processing enabled - starting queue...');
            sendListTab?.classList?.add('queue-active');
            processSendQueue();
        } else {
            console.log('Queue processing disabled');
            sendListTab?.classList?.remove('queue-active');
        }
    });
}

async function processSendQueue() {
    console.log('processSendQueue called - Queue State:', {
        isQueueProcessing
    });

    if (!isQueueProcessing) {
        console.log('Queue processing is disabled, returning early');
        return;
    }

    const tbody = document.querySelector('#sendListTable tbody');
    if (!tbody) {
        console.error('Could not find sendListTable tbody');
        return;
    }

    // Get all pending rows
    const pendingRows = Array.from(tbody.getElementsByTagName('tr')).filter(row => {
        const statusCell = row.querySelector('.send-status-cell');
        return statusCell?.innerHTML.includes('Pending') || 
               statusCell?.innerHTML.includes('fa-clock');
    });

    if (pendingRows.length === 0) {
        console.log('No pending contacts found, waiting for new items...');
        setTimeout(processSendQueue, 5000);
        return;
    }

    // Find the row with the lowest queue number
    const nextRow = pendingRows.reduce((lowest, current) => {
        const lowestQueueNum = parseInt(lowest.querySelector('.send-queue-cell')?.textContent) || Infinity;
        const currentQueueNum = parseInt(current.querySelector('.send-queue-cell')?.textContent) || Infinity;
        return currentQueueNum < lowestQueueNum ? current : lowest;
    });

    console.log('Processing contact with Queue #:', nextRow.querySelector('.send-queue-cell')?.textContent);
    
    await processCurrentRow(nextRow);
}

async function processCurrentRow(currentRow) {
    if (!currentRow.classList.contains('actionListContacts-row')) {
        console.error('Invalid row type, skipping');
        moveToNextContact();
        return;
    }

    const { statusCell, contactCell, action, message, profileUrl } = extractRowData(currentRow);
    
    if (!validateRowData(statusCell, contactCell, action, message, profileUrl)) {
        console.error('Invalid row data, skipping');
        moveToNextContact();
        return;
    }

    logContactDetails(currentRow, contactCell, action, message, profileUrl, statusCell);

    try {
        // Update status to processing
        statusCell.innerHTML = '<i class="fas fa-spinner fa-spin" style="color: #0a66c2;"></i> Processing';

        // Create and process tab
        const tab = await createProfileTab(profileUrl);
        const success = await processTab(tab, action, message);
        
        // Handle tab closure
        await handleAutoClose(tab);

        // Update status based on success and message type
        const isConnectionMessage = action.toLowerCase().includes('connection');
        let newStatus = '';
        if (success) {
            newStatus = isConnectionMessage ? 'connection_sent' : 'message_sent';
            statusCell.innerHTML = `<i class="fas fa-envelope-open-text" style="color: ${isConnectionMessage ? '#22c55e' : '#0a66c2'};"></i> Sent`;
        } else {
            newStatus = 'failed';
            statusCell.innerHTML = '<i class="fas fa-times" style="color: #dc3545;"></i> Failed';
        }

        // Update Chrome storage with new status
        try {
            const { sendListMessages = [] } = await chrome.storage.local.get('sendListMessages');
            const updatedMessages = sendListMessages.map(msg => {
                if (msg.message === message && msg.profileUrl === profileUrl) {
                    return { ...msg, status: newStatus };
                }
                return msg;
            });
            await chrome.storage.local.set({ sendListMessages: updatedMessages });
        } catch (error) {
            console.error('Error updating Chrome storage after processing message:', error);
        }

        // Update stats
        await updateStats(action, success ? 'success' : 'error');

        // Add delay before moving to next contact
        const sendDelaySeconds = parseInt(document.getElementById('sendDelaySeconds')?.value) || 5;
        console.log(`Waiting ${sendDelaySeconds} seconds before processing next contact`);
        await delay(sendDelaySeconds * 1000);
    } catch (error) {
        console.error('Error processing row:', error);
        statusCell.innerHTML = '<i class="fas fa-times" style="color: #dc3545;"></i> Error';
        
        // Update Chrome storage with error status
        try {
            const { sendListMessages = [] } = await chrome.storage.local.get('sendListMessages');
            const updatedMessages = sendListMessages.map(msg => {
                if (msg.message === message && msg.profileUrl === profileUrl) {
                    return { ...msg, status: 'error' };
                }
                return msg;
            });
            await chrome.storage.local.set({ sendListMessages: updatedMessages });
        } catch (storageError) {
            console.error('Error updating Chrome storage with error status:', storageError);
        }
        
        await updateStats(action, 'error');
    }

    moveToNextContact();
}

async function initializeSendQueue() {
    // Load messages from storage first
    const { sendListMessages = [] } = await chrome.storage.local.get('sendListMessages');
    const sendListTbody = document.querySelector('#sendListTable tbody');
    const tabSendCount = document.getElementById('tabSendCount');
    
    if (sendListTbody && sendListMessages.length > 0) {
        // Update tab count
        if (tabSendCount) {
            tabSendCount.textContent = sendListMessages.length;
        }
    }

    // Set up queue toggle listener
    const enableQueueToggle = document.getElementById('enableQueueToggle');
    const sendListTab = document.getElementById('sendListTab');

    if (enableQueueToggle) {
        // Always start with queue disabled
        enableQueueToggle.checked = false;
        isQueueProcessing = false;
        sendListTab?.classList.remove('queue-active');
    }

    setupQueueToggleListener(enableQueueToggle, sendListTab);
}

export {
    processSendQueue,
    initializeSendQueue,
    isQueueProcessing
}; 