// Initialize stats in chrome.storage.local if they don't exist
async function initializeStats() {
    //console.log('Initializing stats...');
    
    const allTimeConnection = await getAllTimeStats('connection');
    const allTimeDirect = await getAllTimeStats('direct');
    
    //console.log('All-time stats retrieved:', { connection: allTimeConnection, direct: allTimeDirect });
    
    // Reset session stats on page load
    await chrome.storage.local.set({
        messageStats: {
            connection: {
                success: 0,
                error: 0,
                pending: 0,
                allTime: allTimeConnection
            },
            direct: {
                success: 0,
                error: 0,
                pending: 0,
                allTime: allTimeDirect
            }
        }
    });
    
    //console.log('Stats initialized in storage');
    
    // Update display with zeroed stats
    updateStatsDisplay();
    
    // Then count pending items from queue
    await updatePendingFromQueue();
}

// Get all-time stats without affecting current session
async function getAllTimeStats(type) {
    const stats = await chrome.storage.local.get('messageStats');
    return stats.messageStats?.[type]?.allTime || 0;
}

// Count and update pending items from the send list table
async function updatePendingFromQueue() {
    //console.log('=== Starting updatePendingFromQueue ===');
    
    const sendListTbody = document.querySelector('#sendListTable tbody');
    if (!sendListTbody) {
        console.error('Could not find sendListTable tbody');
        return;
    }

    const { messageStats } = await chrome.storage.local.get('messageStats');
    //console.log('Current stats before update:', JSON.stringify(messageStats, null, 2));
    
    // Store current allTime counts
    const connectionAllTime = messageStats.connection.allTime;
    const directAllTime = messageStats.direct.allTime;
    
    // Reset counts but preserve allTime
    messageStats.connection = {
        allTime: connectionAllTime,
        error: 0,
        pending: 0,
        success: 0
    };
    messageStats.direct = {
        allTime: directAllTime,
        error: 0,
        pending: 0,
        success: 0
    };

    // Count items in queue
    const rows = Array.from(sendListTbody.getElementsByTagName('tr'));
    //console.log(`Found ${rows.length} rows in send list table`);

    for (const row of rows) {
        const statusCell = row.querySelector('.send-status-cell');
        const actionCell = row.querySelector('.send-action-cell');
        const status = statusCell?.innerHTML || '';
        const action = actionCell?.textContent?.trim() || '';
        
        // Check for success indicators
        const isSent = status.includes('fa-envelope-open-text') || 
                      status.includes('Sent') ||
                      status.includes('fa-check');
                      
        // Check for pending indicators
        const isPending = status.includes('Pending') || 
                         status.includes('fa-clock');
                         
        // Check for error indicators
        const isError = status.includes('fa-times') ||
                       status.includes('Failed');
        

        const type = action.toLowerCase().includes('connection') ? 'connection' : 'direct';
        //console.log(`Processing row as ${type} message`);
        
        if (isPending) {
            messageStats[type].pending++;
            //console.log(`Incremented ${type} pending to ${messageStats[type].pending}`);
        } else if (isSent) {
            messageStats[type].success++;
            //console.log(`Incremented ${type} success to ${messageStats[type].success}`);
        } else if (isError) {
            messageStats[type].error++;
            //console.log(`Incremented ${type} error to ${messageStats[type].error}`);
        }
    }

    //console.log('Final stats:', JSON.stringify(messageStats, null, 2));
    await chrome.storage.local.set({ messageStats });
    updateStatsDisplay();
    //console.log('=== Finished updatePendingFromQueue ===');
}

// Update stats in storage and display
async function updateStats(messageType, status) {
    //console.log('=== Starting updateStats ===', { messageType, status });
    const { messageStats } = await chrome.storage.local.get('messageStats');
    const type = messageType.toLowerCase().includes('connection') ? 'connection' : 'direct';
    
    //console.log('Current stats:', JSON.stringify(messageStats, null, 2));
    
    // Only update allTime on success
    if (status === 'success') {
        messageStats[type].allTime++;
        //console.log(`Incremented ${type} allTime to ${messageStats[type].allTime}`);
    }

    //console.log('Updated stats:', JSON.stringify(messageStats, null, 2));
    await chrome.storage.local.set({ messageStats });
    updateStatsDisplay();
    //console.log('=== Finished updateStats ===');
}

// Update the stats table display
function updateStatsDisplay() {
    //console.log('Updating stats display...');
    chrome.storage.local.get('messageStats', ({ messageStats }) => {
        if (!messageStats) {
            console.error('No messageStats found in storage');
            return;
        }

        //console.log('Stats for display:', messageStats);

        // Update Connection Messages row
        document.getElementById('connection-success').textContent = messageStats.connection.success;
        document.getElementById('connection-error').textContent = messageStats.connection.error;
        document.getElementById('connection-pending').textContent = messageStats.connection.pending;
        document.getElementById('connection-alltime').textContent = messageStats.connection.allTime;

        // Update Direct Messages row
        document.getElementById('direct-success').textContent = messageStats.direct.success;
        document.getElementById('direct-error').textContent = messageStats.direct.error;
        document.getElementById('direct-pending').textContent = messageStats.direct.pending;
        document.getElementById('direct-alltime').textContent = messageStats.direct.allTime;

        // Calculate totals
        const totals = {
            pending: messageStats.connection.pending + messageStats.direct.pending,
            success: messageStats.connection.success + messageStats.direct.success,
            error: messageStats.connection.error + messageStats.direct.error,
            allTime: messageStats.connection.allTime + messageStats.direct.allTime
        };

        //console.log('Calculated totals:', totals);

        // Update Totals row
        document.getElementById('total-pending').textContent = totals.pending;
        document.getElementById('total-success').textContent = totals.success;
        document.getElementById('total-error').textContent = totals.error;
        document.getElementById('total-alltime').textContent = totals.allTime;

        // Update the Send List tab count with pending:success format
        const tabSendCount = document.getElementById('tabSendCount');
        if (tabSendCount) {
            tabSendCount.textContent = `${totals.pending} : ${totals.success}`;
            //console.log('Updated tabSendCount:', tabSendCount.textContent);
        } else {
            console.error('Could not find tabSendCount element');
        }
    });
}

// Reset current session stats (keeps all-time)
async function resetSessionStats() {
    //console.log('Resetting session stats...');
    const stats = await chrome.storage.local.get('messageStats');
    const allTimeConnection = stats.messageStats.connection.allTime;
    const allTimeDirect = stats.messageStats.direct.allTime;

    stats.messageStats = {
        connection: {
            success: 0,
            error: 0,
            pending: 0,
            allTime: allTimeConnection
        },
        direct: {
            success: 0,
            error: 0,
            pending: 0,
            allTime: allTimeDirect
        }
    };

    //console.log('Session stats reset:', stats.messageStats);
    await chrome.storage.local.set({ messageStats: stats.messageStats });
    updateStatsDisplay();
    
    // Update pending counts from queue
    await updatePendingFromQueue();
}

export {
    initializeStats,
    updateStats,
    updateStatsDisplay,
    resetSessionStats,
    updatePendingFromQueue
}; 