import { fetchGeneratedMessages } from './messageManager.js';

function updateAllActions(action) {
    console.log(`Updating all actions to: ${action}`);
    const tbody = document.querySelector('#selectedContactsTable tbody');
    if (!tbody) {
        console.error('Could not find selectedContactsTable tbody');
        return;
    }

    const selects = tbody.querySelectorAll('.action-select');
    //console.log(`Found ${selects.length} action selects`);
    
    selects.forEach((select, index) => {
        const option = select.querySelector(`option[value="${action}"]`);
        if (option && !option.disabled) {
            select.value = action;
            //console.log(`Updated select ${index} to ${action}`);
            
            const event = new Event('change', { bubbles: true });
            select.dispatchEvent(event);
        } else {
            //console.log(`Could not update select ${index} - option not found or disabled`);
        }
    });
}

async function renderTags(tagsString) {
    if (!tagsString) return '';
    const { tagColors = {} } = await chrome.storage.local.get('tagColors');
    const tags = tagsString.split(',').map(t => t.trim()).filter(t => t);
    return tags.map(tag => 
        `<span class="tag" style="background-color: ${tagColors[tag] || '#e1e1e1'}">${tag}</span>`
    ).join('');
}

async function populateContactsTable(urls, email, authentication_token) {
    const tbody = document.querySelector('#selectedContactsTable tbody');
    if (!tbody) {
        console.error('Could not find selectedContactsTable tbody');
        return;
    }

    tbody.innerHTML = ''; // Clear existing rows

    // Get the profile picture toggle state
    const { showProfilePictures } = await chrome.storage.local.get(['showProfilePictures']);
    const defaultProfileSvg = `data:image/svg+xml,${encodeURIComponent('<svg width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg"><path d="M12 12C14.21 12 16 10.21 16 8C16 5.79 14.21 4 12 4C9.79 4 8 5.79 8 8C8 10.21 9.79 12 12 12ZM12 14C9.33 14 4 15.34 4 18V20H20V18C20 15.34 14.67 14 12 14Z" fill="currentColor"/></svg>')}`;

    // Fetch contact details for each URL
    const contacts = await Promise.all(urls.map(async (url) => {
        const linkedinId = url.split('/in/')[1]?.split('/')[0];
        if (!linkedinId) return null;

        try {
            const response = await fetch(
                `https://www.goai-app.com/getContact?email=${encodeURIComponent(email)}&authToken=${encodeURIComponent(authentication_token)}&contactId=${encodeURIComponent(linkedinId)}`
            );
            
            if (!response.ok) {
                console.error('[DEBUG] Failed to fetch contact:', {
                    linkedinId,
                    status: response.status,
                    statusText: response.statusText
                });
                return null;
            }

            const data = await response.json();

            // Check if we have a valid ContactID
            if (!data.contact?.ContactID) {
                console.error('[DEBUG] No ContactID in API response:', {
                    linkedinId,
                    contact: data.contact
                });
                return null;
            }

            return {
                ...data.contact,
                originalUrl: url,
                linkedinId
            };
        } catch (error) {
            console.error(`Error fetching contact ${linkedinId}:`, error);
            return null;
        }
    }));

    // Filter out failed fetches
    const validContacts = contacts.filter(contact => contact !== null);

    // Populate the table
    for (const [index, contact] of validContacts.entries()) {
        const renderedTags = await renderTags(contact.ContactTags);
        const isConnected = contact.existingContact === '1';
        const isPending = contact.pendingConnection === 1;
        let defaultAction = isConnected ? 'direct_message' : 'connection_message';
        
        // Get the original URL that was passed in
        const originalUrl = urls[index];
        
        const tr = document.createElement('tr');
        tr.className = 'actionListContacts-row';
        tr.dataset.originalUrl = originalUrl;  // Store the original URL in the row
        
        // Check if data is older than 24 hours
        let needsRefresh = false;
        if (contact.RefreshDatetime) {
            const refreshDate = new Date(contact.RefreshDatetime);
            const now = new Date();
            const diffHours = Math.abs(now - refreshDate) / 36e5; // Convert to hours
            if (diffHours > 24) {
                defaultAction = 'visit_profile';
                needsRefresh = true;
            }
        }
        
        // Fetch generated messages for the default action
        const linkedinId = urls[index].split('/in/')[1]?.split('/')[0];  // Use the original URL instead of contact.ContactProfileURL
        const messages = await fetchGeneratedMessages(email, authentication_token, linkedinId, defaultAction);
        
        // Calculate recent activities (within last 7 days)
        let hasRecentActivity = false;
        let recentActivities = [];
        if (contact.activities && contact.activities.length > 0) {
            recentActivities = contact.activities.filter(activity => {
                const activityDate = new Date(activity.postWhen);
                const now = new Date();
                const diffTime = Math.abs(now - activityDate);
                const diffDays = Math.ceil(diffTime / (1000 * 60 * 60 * 24));
                return diffDays <= 7;
            });
            hasRecentActivity = recentActivities.length > 0;
        }
        
        const jobTitle = contact.ContactJobTitle || 
                        contact.title || 
                        contact.currentRole || 
                        '<span class="text-muted">Not available</span>';
        
        tr.innerHTML = `
            <td>${index + 1}</td>
            <td>
                <div class="action-container">
                    <select class="action-select" data-contact-id="${index}">
                        ${isConnected ? `
                            <option value="direct_message" ${defaultAction === 'direct_message' ? 'selected' : ''}>Direct Message</option>
                            <option value="promo_message" ${defaultAction === 'promo_message' ? 'selected' : ''}>Promo Message</option>
                            <option value="visit_profile" ${defaultAction === 'visit_profile' ? 'selected' : ''}>Visit Profile</option>
                            <option value="skip">Skip</option>
                        ` : isPending ? `
                            <option value="visit_profile" ${defaultAction === 'visit_profile' ? 'selected' : ''}>Visit Profile</option>
                            <option value="skip">Skip</option>
                        ` : `
                            <option value="connection_message" ${defaultAction === 'connection_message' ? 'selected' : ''}>Connection Message</option>
                            <option value="visit_profile" ${defaultAction === 'visit_profile' ? 'selected' : ''}>Visit Profile</option>
                            <option value="skip">Skip</option>
                        `}
                    </select>
                    ${needsRefresh ? `
                        <div class="suggestedActionInfo">
                            <i class="fas fa-exclamation-triangle"></i>
                            >24 hours data
                        </div>
                    ` : ''}
                </div>
            </td>
            <td class="status-cell">
                <div class="status-icon" title="${contact.status || ''}">${contact.status ? getStatusIcon(contact.status) : ''}</div>
            </td>
            <td class="messages-cell">
                ${messages ? `
                    <button class="messages-count-button ${defaultAction === 'connection_message' ? 'connection-request' : ''}" 
                            data-messages="${encodeURIComponent(JSON.stringify(messages))}">
                        ${messages.length}
                    </button>
                ` : ''}
            </td>
            <td class="activities-cell">
                ${contact.activities && contact.activities.length > 0 ? `
                    <div class="activities-count${hasRecentActivity ? ' has-recent' : ''}" 
                         title="${contact.activities.length} activities${hasRecentActivity ? ` (${recentActivities.length} in last week)` : ''}" 
                         data-activities='${JSON.stringify(contact.activities).replace(/'/g, "&apos;")}'>
                        <span class="count">${contact.activities.length}</span>
                        <svg class="icon" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                            <path d="M21.99 4c0-1.1-.89-2-1.99-2H4c-1.1 0-2 .9-2 2v12c0 1.1.9 2 2 2h14l4 4-.01-18zM18 14H6v-2h12v2zm0-3H6V9h12v2zm0-3H6V6h12v2z" fill="currentColor"/>
                        </svg>
                    </div>
                ` : '-'}
            </td>
            <td class="profile-picture-cell">
                <div class="profile-picture-container">
                    <img src="${defaultProfileSvg}" 
                         class="profile-picture clickable" 
                         alt="Profile Picture"
                         data-profile-url="${originalUrl}"
                         data-contact-id="${contact.ContactID}"
                         data-linkedin-id="${linkedinId}"
                         title="Click to open profile"
                         ${showProfilePictures === true ? `data-original-src=""` : ''}
                         style="width: 40px; height: 40px; border-radius: 50%; object-fit: cover;">
                    <div class="connection-status-overlay ${isConnected ? 'connected' : isPending ? 'pending' : 'not-connected'}">
                        <i class="fas ${isConnected ? 'fa-check' : isPending ? 'fa-hourglass' : 'fa-times'} icon"></i>
                    </div>
                </div>
            </td>
            <td class="contact-info">
                <div class="contact-name">${contact.ContactFirstname || ''} ${contact.ContactLastname || ''}</div>
                <div class="contact-details">
                    <span class="company-badge">${contact.ContactCurrentCompany || ''}</span>
                    ${jobTitle ? ` - ${jobTitle}` : ''}
                </div>
            </td>
            <td>${renderedTags}</td>
        `;

        tbody.appendChild(tr);

        // Only load profile picture if the toggle is ON
        if (showProfilePictures === true) {
            loadProfilePicture(tr.querySelector('.profile-picture'), urls[index].split('/in/')[1]?.split('/')[0], email, authentication_token);
        }
        
        // Add click handler for profile picture
        setupProfilePictureHandler(tr.querySelector('.profile-picture'));
    }

    addActivitiesClickHandlers();
}

async function loadProfilePicture(imgElement, linkedinId, email, authToken) {
    try {
        // Get ContactID and LinkedinID from the element's data attributes
        const contactId = imgElement.dataset.contactId || null;
        const elementLinkedinId = imgElement.dataset.linkedinId || linkedinId || null;
        
        // If we don't have either ID, we can't proceed
        if (!contactId && !elementLinkedinId) {
            console.error('[DEBUG] No ContactID or LinkedinID available for profile picture:', {
                elementDataset: imgElement.dataset,
                elementAttributes: Array.from(imgElement.attributes).map(attr => ({ name: attr.name, value: attr.value }))
            });
            return;
        }
        
        // Use ContactID if available, otherwise use LinkedinID
        const idToUse = contactId || elementLinkedinId;
        
        const response = await fetch(
            `https://www.goai-app.com/getProfilePicture?email=${encodeURIComponent(email)}&authToken=${encodeURIComponent(authToken)}&contactId=${encodeURIComponent(idToUse)}`
        );
        
        if (response.ok) {
            const blob = await response.blob();
            const url = URL.createObjectURL(blob);
            imgElement.src = url;
            imgElement.onload = () => URL.revokeObjectURL(url);

        } else {
            console.error('[DEBUG] Failed to load profile picture:', {
                status: response.status,
                statusText: response.statusText,
                idUsed: idToUse
            });
        }
    } catch (error) {
        console.error('[DEBUG] Error in loadProfilePicture:', error, {
            contactId: imgElement.dataset.contactId,
            linkedinId: imgElement.dataset.linkedinId || linkedinId,
            elementDataset: imgElement.dataset,
            elementAttributes: Array.from(imgElement.attributes).map(attr => ({ name: attr.name, value: attr.value }))
        });
    }
}

function setupProfilePictureHandler(imgElement) {
    imgElement.addEventListener('click', () => {
        const profileUrl = imgElement.dataset.profileUrl;
        if (profileUrl) {
            chrome.tabs.create({ 
                url: profileUrl,
                active: false
            });
        }
    });
}

function showActivitiesModal(activities) {
    const modal = document.getElementById('activitiesModal');
    const activitiesList = document.getElementById('activitiesList');
    const closeButton = modal.querySelector('.close-button');

    // Clear previous content
    activitiesList.innerHTML = '';

    // Add each activity
    activities.forEach(activity => {
        const [who, when] = activity.whoPostedWhen.split(' • ');
        const activityElement = document.createElement('div');
        activityElement.className = 'activity-item';
        
        // Create context section if it exists
        const contextSection = activity.postContext ? `
            <div class="activity-context">
                <svg class="icon" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                    <path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm1 15h-2v-6h2v6zm0-8h-2V7h2v2zm0-3H6V3h6v2z" fill="currentColor"/>
                </svg>
                <span>${activity.postContext}</span>
            </div>
        ` : '';
        
        activityElement.innerHTML = `
            <div class="activity-header">
                <span class="activity-poster">${who}</span>
                <span class="activity-time">${when}</span>
            </div>
            ${contextSection}
            <div class="activity-content">${activity.postContent}</div>
            <div class="activity-footer">
                <div class="activity-stats">
                    <div class="activity-reactions" title="Reactions">
                        <svg class="icon" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                            <path d="M12 21.35l-1.45-1.32C5.4 15.36 2 12.28 2 8.5 2 5.42 4.42 3 7.5 3c1.74 0 3.41.81 4.5 2.09C13.09 3.81 14.76 3 16.5 3 19.58 3 22 5.42 22 8.5c0 3.78-3.4 6.86-8.55 11.54L12 21.35z" fill="currentColor"/>
                        </svg>
                        <span>${activity.postSocialReactions}</span>
                    </div>
                    ${activity.postComments ? `
                        <div class="activity-comments" title="Comments">
                            <svg class="icon" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                                <path d="M20 2H4c-1.1 0-2 .9-2 2v18l4-4h14c1.1 0 2-.9 2-2V4c0-1.1-.9-2-2-2z" fill="currentColor"/>
                            </svg>
                            <span>${activity.postComments}</span>
                        </div>
                    ` : ''}
                </div>
                <a href="${activity.postDirectLink}" target="_blank" class="activity-link">
                    View on LinkedIn
                    <svg class="icon" width="12" height="12" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M19 19H5V5h7V3H5a2 2 0 0 0-2 2v14a2 2 0 0 0 2 2h14a2 2 0 0 0 2-2v-7h-2v7zM14 3v2h3.59l-9.83 9.83 1.41 1.41L19 6.41V10h2V3h-7z" fill="currentColor"/>
                    </svg>
                </a>
            </div>
        `;
        activitiesList.appendChild(activityElement);
    });

    // Show modal
    modal.style.display = 'block';

    // Close modal when clicking close button
    closeButton.onclick = () => {
        modal.style.display = 'none';
    };

    // Close modal when clicking outside
    window.onclick = (event) => {
        if (event.target === modal) {
            modal.style.display = 'none';
        }
    };
}

function addActivitiesClickHandlers() {
    document.querySelectorAll('.activities-count').forEach(element => {
        element.addEventListener('click', (e) => {
            e.stopPropagation();
            showActivitiesModal(JSON.parse(element.dataset.activities));
        });
    });
}

function getStatusIcon(status) {
    if (!status) return '';
    
    switch (status) {
        case 'Pending':
            return `<svg class="icon pending" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z" fill="currentColor"/>
                        <path d="M12 6v6l5.2 3.1.8-1.3-4.5-2.7V6z" fill="currentColor"/>
                    </svg>`;
        case 'In Progress':
            return `<svg class="icon in-progress" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm-1 14H9V8h2v8zm4 0h-2V8h2v8z" fill="currentColor"/>
                    </svg>`;
        case 'Completed':
            return `<svg class="icon completed" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm-2 15l-5-5 1.41-1.41L10 14.17l7.59-7.59L19 8l-9 9z" fill="currentColor"/>
                    </svg>`;
        default:
            return '';
    }
}

export {
    updateAllActions,
    renderTags,
    populateContactsTable,
    showActivitiesModal
};