import { delay } from './utils.js';

async function handleAutoCloseToggle(e) {
    const autoClose = e.target.checked;
    await chrome.storage.local.set({ autoCloseTabs: autoClose });
    
    // If toggled ON, close any existing LinkedIn tabs
    if (autoClose) {
        try {
            const tabs = await chrome.tabs.query({ url: "*://*.linkedin.com/*" });
            if (tabs.length > 0) {
                const tabIds = tabs.map(tab => tab.id);
                await chrome.tabs.remove(tabIds);
            }
        } catch (error) {
            console.error('Error closing existing LinkedIn tabs:', error);
        }
    }
}

function initializeTabControls() {
    document.querySelectorAll('.tab-button').forEach(button => {
        button.addEventListener('click', () => {
            document.querySelectorAll('.tab-button').forEach(btn => btn.classList.remove('active'));
            document.querySelectorAll('.tab-pane').forEach(pane => pane.classList.remove('active'));
            
            button.classList.add('active');
            const tabId = button.dataset.tab;
            document.getElementById(tabId).classList.add('active');
        });
    });
}

async function initializeAutoCloseToggle() {
    // Load saved preference
    const { autoCloseTabs } = await chrome.storage.local.get('autoCloseTabs');
    const autoCloseToggle = document.getElementById('autoCloseToggle');
    
    if (autoCloseToggle) {
        // Set the toggle state based on saved preference (defaults to false if not set)
        autoCloseToggle.checked = Boolean(autoCloseTabs);
        autoCloseToggle.addEventListener('change', handleAutoCloseToggle);
    }
}

function updateStatus(statusCell, status) {
    const statusMap = {
        'processing': 'In Progress',
        'error': 'Failed',
        'sent': 'Completed',
        'Messages Ready': 'Pending'
    };

    const mappedStatus = statusMap[status] || status;
    statusCell.innerHTML = `
        <div class="status-icon" title="${mappedStatus}">
            ${getStatusIcon(mappedStatus)}
        </div>
    `;
}

// Helper function to get status icon based on status
function getStatusIcon(status) {
    switch (status) {
        case 'Pending':
            return `<svg class="icon pending" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z" fill="currentColor"/>
                        <path d="M12 6v6l5.2 3.1.8-1.3-4.5-2.7V6z" fill="currentColor"/>
                    </svg>`;
        case 'In Progress':
            return `<svg class="icon in-progress" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm-1 14H9V8h2v8zm4 0h-2V8h2v8z" fill="currentColor"/>
                    </svg>`;
        case 'Completed':
            return `<svg class="icon completed" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm-2 15l-5-5 1.41-1.41L10 14.17l7.59-7.59L19 8l-9 9z" fill="currentColor"/>
                    </svg>`;
        case 'Failed':
            return `<svg class="icon failed" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M12 2C6.47 2 2 6.47 2 12s4.47 10 10 10 10-4.47 10-10S17.53 2 12 2zm5 13.59L15.59 17 12 13.41 8.41 17 7 15.59 10.59 12 7 8.41 8.41 7 12 10.59 15.59 7 17 8.41 13.41 12 17 15.59z" fill="currentColor"/>
                    </svg>`;
        default:
            return `<svg class="icon unknown" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm1 17h-2v-2h2v2zm2.07-7.75l-.9.92C13.45 12.9 13 13.5 13 15h-2v-.5c0-1.1.45-2.1 1.17-2.83l1.24-1.26c.37-.36.59-.86.59-1.41 0-1.1-.9-2-2-2s-2 .9-2 2H8c0-2.21 1.79-4 4-4s4 1.79 4 4c0 .88-.36 1.68-.93 2.25z" fill="currentColor"/>
                    </svg>`;
    }
}

export {
    initializeTabControls,
    initializeAutoCloseToggle,
    updateStatus,
    handleAutoCloseToggle
};