// Import the new modules
import { setupEventListeners } from './eventManager.js';
import { loadContacts } from './tableManager.js';
import { initializeTagManager, updateMultiSelectTagOptions } from './tagManager.js';
import { showError } from '../utils/errorUtils.js';

// Keep the utility functions at the top
const debounce = (func, wait) => {
    let timeout;
    return function executedFunction(...args) {
        const later = () => {
            clearTimeout(timeout);
            func(...args);
        };
        clearTimeout(timeout);
        timeout = setTimeout(later, wait);
    };
};

// Simple markdown parser
function parseMarkdown(text) {
    // Replace headings
    text = text.replace(/^# (.*$)/gm, '<h1>$1</h1>');
    text = text.replace(/^## (.*$)/gm, '<h2>$1</h2>');
    
    // Replace bold
    text = text.replace(/\*\*(.*?)\*\*/g, '<strong>$1</strong>');
    
    // Replace links
    text = text.replace(/\[(.*?)\]\((.*?)\)/g, '<a href="$2" target="_blank">$1</a>');
    
    // Replace lists
    text = text.replace(/^\d\. (.*$)/gm, '<li>$1</li>');
    text = text.replace(/^- (.*$)/gm, '<li>$1</li>');
    
    // Wrap lists in ul/ol tags
    text = text.replace(/((?:<li>.*<\/li>\n?)+)/g, '<ul>$1</ul>');
    
    // Replace line breaks with paragraphs
    text = text.replace(/\n\n/g, '</p><p>');
    text = '<p>' + text + '</p>';
    
    // Clean up empty paragraphs
    text = text.replace(/<p>\s*<\/p>/g, '');
    
    return text;
}

document.addEventListener('DOMContentLoaded', async () => {
    try {
        // Get user data and preferences from storage
        const { email, authentication_token, showProfilePictures } = await chrome.storage.local.get(['email', 'authentication_token', 'showProfilePictures']);
        
        if (!email || !authentication_token) {
            throw new Error('User not authenticated');
        }

        // Initialize profile picture toggle with saved preference
        const showProfilePicturesToggle = document.getElementById('showProfilePictures');
        if (showProfilePicturesToggle) {
            // Set initial state - default to OFF unless explicitly set to true
            showProfilePicturesToggle.checked = showProfilePictures === true;
            
            // Add event listener
            showProfilePicturesToggle.addEventListener('change', async function() {
                const profilePictures = document.querySelectorAll('.contact-photo img');
                
                if (this.checked) {
                    // When toggled ON, fetch and display profile pictures
                    for (const img of profilePictures) {
                        const row = img.closest('tr');
                        const profileId = row?.getAttribute('data-profile-id');
                        
                        if (profileId) {
                            const profileImageUrl = await getProfileImage(email, authentication_token, profileId);
                            img.setAttribute('data-original-src', profileImageUrl);
                            img.src = profileImageUrl;
                        }
                    }
                } else {
                    // When toggled OFF, just show the default SVG
                    const defaultProfileSvg = `data:image/svg+xml,${encodeURIComponent('<svg width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg"><path d="M12 12C14.21 12 16 10.21 16 8C16 5.79 14.21 4 12 4C9.79 4 8 5.79 8 8C8 10.21 9.79 12 12 12ZM12 14C9.33 14 4 15.34 4 18V20H20V18C20 15.34 14.67 14 12 14Z" fill="currentColor"/></svg>')}`;
                    profilePictures.forEach(img => {
                        img.src = defaultProfileSvg;
                    });
                }
                
                // Save preference to storage
                chrome.storage.local.set({ 'showProfilePictures': this.checked });
                
                // Track the toggle action
                trackUserAction('toggle_profile_pictures', { enabled: this.checked });
            });

            // Trigger the change event to apply initial state
            showProfilePicturesToggle.dispatchEvent(new Event('change'));
        }

        // Send detailed analytics event for page load
        chrome.runtime.sendMessage({
            action: 'sendAnalyticsEvent',
            eventName: 'contact_manager_loaded',
            eventParams: {
                page_type: 'contact_manager',
                authenticated: true,
                timestamp: new Date().toISOString(),
                referrer: document.referrer,
                screen_resolution: `${window.screen.width}x${window.screen.height}`,
                window_size: `${window.innerWidth}x${window.innerHeight}`,
                platform: navigator.platform,
                user_agent: navigator.userAgent
            }
        });

        // Initialize components
        await initializeContactManager();
        await initializeTagManager();
        await updateMultiSelectTagOptions();

        // Initialize AI modal
        const aiModal = setupAiModal();

        // Add AI overlay to each contact photo cell
        const contactPhotoCells = document.querySelectorAll('.contacts-table td:nth-child(2)');
        contactPhotoCells.forEach(cell => {
            const aiOverlay = document.createElement('div');
            aiOverlay.className = 'ai-status-overlay';
            aiOverlay.textContent = 'AI';
            
            const row = cell.closest('tr');
            const contactId = row.getAttribute('data-profile-id');
            
            aiOverlay.onclick = (e) => {
                e.stopPropagation(); // Prevent row selection
                aiModal.show(contactId);
            };
            
            cell.style.position = 'relative';
            cell.appendChild(aiOverlay);
        });

        // Send success analytics
        chrome.runtime.sendMessage({
            action: 'sendAnalyticsEvent',
            eventName: 'contact_manager_initialized',
            eventParams: {
                success: true,
                initialization_time: performance.now(),
                components_loaded: [
                    'contact_manager',
                    'tag_manager',
                    'multi_select'
                ]
            }
        });

    } catch (error) {
        console.error('Error initializing contact manager:', error);
        showError('Failed to initialize contact manager: ' + error.message);

        // Send error analytics
        chrome.runtime.sendMessage({
            action: 'sendAnalyticsEvent',
            eventName: 'contact_manager_error',
            eventParams: {
                error_type: error.name,
                error_message: error.message,
                error_stack: error.stack,
                component: 'contact_manager',
                action: 'initialization'
            }
        });
    }
});

async function initializeContactManager() {
    try {
        // Start performance measurement
        const startTime = performance.now();

        // Load contacts
        await loadContacts();
        
        // Set up event listeners
        setupEventListeners();

        // Calculate initialization time
        const initTime = performance.now() - startTime;

        // Send success analytics
        chrome.runtime.sendMessage({
            action: 'sendAnalyticsEvent',
            eventName: 'contacts_loaded',
            eventParams: {
                success: true,
                load_time: initTime,
                performance_metrics: {
                    initialization_time: initTime,
                    memory_usage: performance.memory?.usedJSHeapSize,
                    navigation_type: performance.navigation?.type
                }
            }
        });

    } catch (error) {
        // Send error analytics
        chrome.runtime.sendMessage({
            action: 'sendAnalyticsEvent',
            eventName: 'contacts_load_error',
            eventParams: {
                error_type: error.name,
                error_message: error.message,
                error_stack: error.stack,
                component: 'contact_manager',
                action: 'load_contacts'
            }
        });
        throw error; // Re-throw to be handled by the main try-catch
    }
}

// Add analytics wrapper for error tracking
function trackError(error, component, action) {
    chrome.runtime.sendMessage({
        action: 'sendAnalyticsEvent',
        eventName: 'contact_manager_error',
        eventParams: {
            error_type: error.name,
            error_message: error.message,
            error_stack: error.stack,
            component,
            action,
            timestamp: new Date().toISOString()
        }
    });
}

// Add analytics wrapper for user actions
function trackUserAction(action, details = {}) {
    chrome.runtime.sendMessage({
        action: 'sendAnalyticsEvent',
        eventName: 'contact_manager_action',
        eventParams: {
            action_type: action,
            ...details,
            timestamp: new Date().toISOString()
        }
    });
}

// Add AI functionality
async function fetchAiSummary(contactId) {
    try {
        const { authentication_token, email } = await chrome.storage.local.get(['authentication_token', 'email']);
        const response = await fetch('https://goai-app.com/summarizeContact', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                contactId,
                authToken: authentication_token, 
                email: email
            })
        });

        if (!response.ok) {
            throw new Error('Failed to fetch AI summary');
        }

        const data = await response.json();
        return data;
    } catch (error) {
        console.error('Error fetching AI summary:', error);
        throw error;
    }
}

function setupAiModal() {
    const modal = document.getElementById('aiModal');
    if (!modal) {
        // Create modal if it doesn't exist
        const modalHtml = `
            <div id="aiModal" class="ai-modal">
                <div class="ai-modal-content">
                    <span class="ai-modal-close">&times;</span>
                    <div class="ai-summary" id="aiSummary">Loading...</div>
                </div>
            </div>
        `;
        document.body.insertAdjacentHTML('beforeend', modalHtml);
    }

    // Get modal elements after ensuring they exist
    const modalElement = document.getElementById('aiModal');
    const closeBtn = modalElement.querySelector('.ai-modal-close');
    const summaryDiv = document.getElementById('aiSummary');

    closeBtn.onclick = () => {
        modalElement.style.display = 'none';
    };

    window.onclick = (event) => {
        if (event.target === modalElement) {
            modalElement.style.display = 'none';
        }
    };

    return {
        show: async (contactId) => {
            modalElement.style.display = 'block';
            summaryDiv.innerHTML = 'Loading...';
            try {
                const data = await fetchAiSummary(contactId);
                summaryDiv.innerHTML = parseMarkdown(data.summary);
            } catch (error) {
                summaryDiv.innerHTML = 'Failed to load AI summary. Please try again later.';
            }
        }
    };
}

// Export functions
export {
    initializeContactManager,
    debounce,
    trackError,
    trackUserAction
};
