// Import error utilities
import { showError, showSuccess } from '../utils/errorUtils.js';
import { loadContacts, applyAdvancedFilters, clearAllFilters, deleteContact, exportCurrentContacts, handleCsvImport } from './tableManager.js';
import { initializeTagManager, openTagManager, fetchAndPopulateTags } from './tagManager.js';

export function setupEventListeners() {
    const elements = {
        importBtn: document.getElementById('importBtn'),
        csvImport: document.getElementById('csvImport'),
        exportBtn: document.getElementById('exportBtn'),
        selectAll: document.getElementById('selectAll'),
        tagManagerBtn: document.getElementById('tagManagerBtn'),
        filterToggle: document.getElementById('filterToggleBtn'),
        filterContent: document.getElementById('filterContent'),
        applyFilters: document.getElementById('applyFilters'),
        clearAdvancedFilters: document.getElementById('clearAdvancedFilters'),
        deleteSelectedBtn: document.getElementById('deleteSelectedBtn'),
        multiSelectControl: document.getElementById('multiSelectControl'),
        automateActionsBtn: document.getElementById('automateActionsBtn'),
        automationModal: document.getElementById('automationModal'),
        closeAutomationModal: document.getElementById('closeAutomationModal'),
        connectionRequestsBtn: document.getElementById('connectionRequestsBtn'),
        sendDMCRBtn: document.getElementById('sendDMCRBtn'),
        postReplyBtn: document.getElementById('postReplyBtn')
    };

    // Set up delete button
    if (elements.deleteSelectedBtn) {
        elements.deleteSelectedBtn.addEventListener('click', async () => {
            const selectedCheckboxes = document.querySelectorAll('.contact-select:checked');
            const selectedUrls = Array.from(selectedCheckboxes).map(checkbox => checkbox.dataset.url);
            
            if (selectedUrls.length === 0) {
                showError('No contacts selected for deletion');
                return;
            }

            const confirmed = await showConfirmationDialog(
                `Are you sure you want to delete ${selectedUrls.length} selected contact${selectedUrls.length === 1 ? '' : 's'}? This action cannot be undone.`
            );

            if (confirmed) {
                await deleteContact(selectedUrls);
            }
        });
    }

    // Set up filter toggle with tag fetching
    if (elements.filterToggle && elements.filterContent) {
        elements.filterToggle.addEventListener('click', async () => {
            elements.filterContent.classList.toggle('hidden');
            elements.filterToggle.classList.toggle('expanded');
            
            // Fetch and populate tags when filter is opened
            if (!elements.filterContent.classList.contains('hidden')) {
                await fetchAndPopulateTags('tagsMultiFilter');
            }
        });
    }

    // Set up filter buttons
    if (elements.applyFilters) {
        elements.applyFilters.addEventListener('click', applyAdvancedFilters);
    }

    if (elements.clearAdvancedFilters) {
        elements.clearAdvancedFilters.addEventListener('click', clearAllFilters);
    }

    // Set up tag manager
    if (elements.tagManagerBtn) {
        elements.tagManagerBtn.addEventListener('click', openTagManager);
    }

    // Set up import/export
    if (elements.importBtn && elements.csvImport) {
        elements.importBtn.addEventListener('click', () => {
            elements.csvImport.click();
        });
        elements.csvImport.addEventListener('change', handleCsvImport);
    }

    if (elements.exportBtn) {
        elements.exportBtn.addEventListener('click', exportCurrentContacts);
    }

    // Set up select all
    if (elements.selectAll) {
        elements.selectAll.addEventListener('change', handleSelectAll);
    }

    // Add automation modal event listeners
    if (elements.automateActionsBtn) {
        elements.automateActionsBtn.addEventListener('click', async () => {
            const selectedCheckboxes = document.querySelectorAll('.contact-select:checked');
            if (selectedCheckboxes.length === 0) {
                showError('No contacts selected');
                return;
            }

            try {
                // Collect the profile URLs of selected contacts
                const selectedUrls = Array.from(selectedCheckboxes).map(checkbox => {
                    const linkedinId = checkbox.dataset.linkedinId;
                    return `https://www.linkedin.com/in/${linkedinId}/`;
                });

                // Store URLs in localStorage
                localStorage.setItem('selectedContactsUrls', JSON.stringify(selectedUrls));

                // Open the Contact Automator in a new tab
                chrome.tabs.create({
                    url: chrome.runtime.getURL('contactAutomator.html')
                });

            } catch (error) {
                console.error('Error launching automation:', error);
                showError('Failed to launch automation: ' + error.message);
            }
        });
    }

    // Add automation action button listeners
    if (elements.connectionRequestsBtn) {
        elements.connectionRequestsBtn.addEventListener('click', handleConnectionRequests);
    }

    if (elements.sendDMCRBtn) {
        elements.sendDMCRBtn.addEventListener('click', handleSendDMCR);
    }

    if (elements.postReplyBtn) {
        elements.postReplyBtn.addEventListener('click', handlePostReply);
    }

    const addTagToSelected = document.getElementById('addTagToSelected');
    const removeTagFromSelected = document.getElementById('removeTagFromSelected');

    if (addTagToSelected) {
        addTagToSelected.addEventListener('click', () => handleMultiSelectTagAction('add'));
    }

    if (removeTagFromSelected) {
        removeTagFromSelected.addEventListener('click', () => handleMultiSelectTagAction('remove'));
    }

    // Update the checkbox change event to fetch tags when showing multi-select control
    document.querySelectorAll('.contact-select').forEach(checkbox => {
        checkbox.addEventListener('change', async () => {
            const selectedCheckboxes = document.querySelectorAll('.contact-select:checked');
            const multiSelectControl = document.getElementById('multiSelectControl');
            const selectedCount = document.getElementById('selectedCount');
            
            if (selectedCheckboxes.length > 0) {
                if (multiSelectControl) {
                    multiSelectControl.style.display = 'block';
                    await fetchAndPopulateTags('multiSelectTag');
                }
                if (selectedCount) {
                    selectedCount.textContent = `${selectedCheckboxes.length} contact${selectedCheckboxes.length === 1 ? '' : 's'} selected`;
                }
            } else {
                if (multiSelectControl) multiSelectControl.style.display = 'none';
            }
        });
    });
}

function handleSelectAll(event) {
    const checkboxes = document.querySelectorAll('.contact-select');
    checkboxes.forEach(checkbox => {
        checkbox.checked = event.target.checked;
    });
    updateMultiSelectControl();
}

function updateMultiSelectControl() {
    const selectedCheckboxes = document.querySelectorAll('.contact-select:checked');
    const multiSelectControl = document.getElementById('multiSelectControl');
    const selectedCount = document.getElementById('selectedCount');
    
    if (selectedCheckboxes.length > 0) {
        if (multiSelectControl) multiSelectControl.style.display = 'block';
        if (selectedCount) selectedCount.textContent = `${selectedCheckboxes.length} contact${selectedCheckboxes.length === 1 ? '' : 's'} selected`;
    } else {
        if (multiSelectControl) multiSelectControl.style.display = 'none';
    }
}

async function handleSendDMCR() {
    try {
        const selectedCheckboxes = document.querySelectorAll('.contact-select:checked');
        if (selectedCheckboxes.length === 0) {
            showError('No contacts selected');
            return;
        }

        // Only collect the profile URLs
        const selectedUrls = Array.from(selectedCheckboxes).map(checkbox => {
            const linkedinId = checkbox.dataset.linkedinId;
            return `https://www.linkedin.com/in/${linkedinId}/`;
        });

        // Store just the URLs in localStorage
        localStorage.setItem('selectedContactsUrls', JSON.stringify(selectedUrls));

        // Open the Contact Automator in a new tab
        chrome.tabs.create({
            url: chrome.runtime.getURL('contactAutomator.html')
        });

        // Close the automation modal
        const automationModal = document.getElementById('automationModal');
        if (automationModal) {
            automationModal.style.display = 'none';
        }

    } catch (error) {
        console.error('Error handling DM/CR:', error);
        showError('Failed to process DM/CR: ' + error.message);
    }
}

async function handleConnectionRequests() {
    try {
        const selectedCheckboxes = document.querySelectorAll('.contact-select:checked');
        if (selectedCheckboxes.length === 0) {
            showError('No contacts selected');
            return;
        }
        console.log('Connection requests automation started');
        showSuccess('Connection requests feature coming soon');
    } catch (error) {
        console.error('Error handling connection requests:', error);
        showError('Failed to process connection requests: ' + error.message);
    }
}

async function handlePostReply() {
    try {
        const selectedCheckboxes = document.querySelectorAll('.contact-select:checked');
        if (selectedCheckboxes.length === 0) {
            showError('No contacts selected');
            return;
        }
        console.log('Post reply automation started');
        showSuccess('Post reply feature coming soon');
    } catch (error) {
        console.error('Error handling post reply:', error);
        showError('Failed to process post reply: ' + error.message);
    }
}

function showConfirmationDialog(message) {
    return new Promise((resolve) => {
        const overlay = document.createElement('div');
        overlay.className = 'confirmation-overlay';
        
        const dialog = document.createElement('div');
        dialog.className = 'confirmation-dialog';
        dialog.innerHTML = `
            <p>${message}</p>
            <div class="dialog-actions">
                <button class="secondary-button" id="cancelDelete">Cancel</button>
                <button class="danger-button" id="confirmDelete">Delete</button>
            </div>
        `;
        
        document.body.appendChild(overlay);
        document.body.appendChild(dialog);
        
        const handleCancel = () => {
            overlay.remove();
            dialog.remove();
            resolve(false);
        };
        
        const handleConfirm = () => {
            overlay.remove();
            dialog.remove();
            resolve(true);
        };
        
        dialog.querySelector('#cancelDelete').addEventListener('click', handleCancel);
        dialog.querySelector('#confirmDelete').addEventListener('click', handleConfirm);
        overlay.addEventListener('click', handleCancel);
    });
}

async function handleMultiSelectTagAction(action) {
    const tagSelect = document.getElementById('multiSelectTag');
    if (!tagSelect) return;

    const tag = tagSelect.value;
    if (!tag) {
        showError('Please select a tag first');
        return;
    }

    const selectedCheckboxes = document.querySelectorAll('.contact-select:checked');
    if (selectedCheckboxes.length === 0) {
        showError('No contacts selected');
        return;
    }

    try {
        const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);

        const contactIds = Array.from(selectedCheckboxes).map(checkbox => checkbox.dataset.linkedinId)
            .filter(id => id);

        if (contactIds.length === 0) {
            throw new Error('No valid contact IDs found');
        }

        const endpoint = action === 'add' ? 'addTag' : 'removeTag';

        const response = await fetch(`https://www.goai-app.com/${endpoint}`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({
                email,
                authToken,
                tag,
                contacts: contactIds
            })
        });

        if (!response.ok) {
            throw new Error(`Failed to ${action} tag`);
        }

        const result = await response.json();
        if (!result.success) {
            throw new Error(result.message || `Failed to ${action} tag`);
        }

        await loadContacts();
        showSuccess(`Successfully ${action}ed tag to ${contactIds.length} contact${contactIds.length === 1 ? '' : 's'}`);
    } catch (error) {
        console.error(`Error ${action}ing tag:`, error);
        showError(`Failed to ${action} tag: ${error.message}`);
    }
}

async function updateSelectedContactsTable() {
    const selectedCheckboxes = document.querySelectorAll('.contact-select:checked');
    const selectedContactsCount = document.getElementById('selectedContactsCount');
    const tbody = document.querySelector('#selectedContactsTable tbody');
    
    if (!tbody || !selectedContactsCount) return;

    selectedContactsCount.textContent = selectedCheckboxes.length;
    tbody.innerHTML = '';

    selectedCheckboxes.forEach(checkbox => {
        const row = checkbox.closest('tr');
        if (!row) return;

        const tooltipData = JSON.parse(row.dataset.tooltipData || '{}');
        const newRow = document.createElement('tr');
        
        newRow.innerHTML = `
            <td>${tooltipData.firstName || ''}</td>
            <td>${tooltipData.lastName || ''}</td>
            <td>${tooltipData.title || ''}</td>
            <td>${tooltipData.company || ''}</td>
            <td>${tooltipData.tags || ''}</td>
            <td>${tooltipData.connected || ''}</td>
            <td><a href="${tooltipData.url}" target="_blank">${tooltipData.url}</a></td>
        `;

        tbody.appendChild(newRow);
    });
}

export {
    handleSelectAll,
    updateMultiSelectControl,
    showConfirmationDialog,
    updateSelectedContactsTable,
    handleConnectionRequests,
    handleSendDMCR,
    handlePostReply
}; 