// Import error utilities
import { showError, showSuccess } from '../utils/errorUtils.js';
import { renderTags, updateTagFilterOptions } from './tagManager.js';

// Add these variables at the top of the file, after the imports
let lastChecked = null;
let isShiftPressed = false;

// Add this function to handle shift-click selection
function handleShiftClick(checkbox, event) {
    if (!lastChecked) {
        lastChecked = checkbox;
        return;
    }

    if (event.shiftKey) {
        const checkboxes = Array.from(document.querySelectorAll('.contact-select'));
        const start = checkboxes.indexOf(lastChecked);
        const end = checkboxes.indexOf(checkbox);
        
        // Use the state of the current clicked checkbox
        const shouldCheck = checkbox.checked;
        
        // Select all checkboxes in between, including start and end points
        const min = Math.min(start, end);
        const max = Math.max(start, end);
        
        for (let i = min; i <= max; i++) {
            checkboxes[i].checked = shouldCheck;
        }
    }

    lastChecked = checkbox;
    updateMultiSelectControl();
}

// Add this new function at the top of the file
async function getProfileImage(email, authToken, profileId) {
    try {
        const response = await fetch(
            `https://www.goai-app.com/getProfilePicture?email=${encodeURIComponent(email)}&authToken=${encodeURIComponent(authToken)}&contactId=${encodeURIComponent(profileId)}`,
            { method: 'GET' }
        );

        if (response.status === 200) {
            const blob = await response.blob();
            return URL.createObjectURL(blob);
        }
        
        return chrome.runtime.getURL('icons/default.jpg');
    } catch (error) {
        console.error('Error fetching profile image:', error);
        return chrome.runtime.getURL('icons/default.jpg');
    }
}

export async function loadContacts() {
    console.log('Loading contacts...');
    const startTime = performance.now();

    try {
        const tbody = document.getElementById('contactsTableBody');
        if (tbody) {
            tbody.innerHTML = '<tr><td colspan="7" class="loading-state">Loading contacts...</td></tr>';
            console.log('Loading state displayed.');
        }

        // Fetch email and authToken from Chrome storage
        const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);
        
        // Fetch contacts from the API
        const response = await fetch(`https://www.goai-app.com/getAllContacts?email=${encodeURIComponent(email)}&authToken=${authToken}`, {
            method: 'GET',
            headers: {
                'Accept': 'application/json'
            }
        });

        if (!response.ok) {
            throw new Error('Failed to fetch contacts');
        }

        const data = await response.json();
        if (!data.success) {
            throw new Error('API response unsuccessful');
        }

        const contacts = data.contacts;
        console.log('Contacts fetched successfully:', contacts);

        // Store total contacts count
        window.totalContactsCount = contacts.length;

        updateTagFilterOptions(contacts);
        await renderContactsTable(contacts);
        
    } catch (error) {
        console.error('Error loading contacts:', error);
        showError('Failed to load contacts: ' + error.message);
    } finally {
        const endTime = performance.now();
        console.log(`Total time taken to load contacts: ${(endTime - startTime).toFixed(2)} ms`);
    }
}

export async function renderContactsTable(contacts) {
    console.log('Rendering contacts table...');
    const tbody = document.getElementById('contactsTableBody');
    const contactCountDisplay = document.getElementById('contactCount');
    
    if (!contacts || contacts.length === 0) {
        if (tbody) tbody.innerHTML = '<tr><td colspan="7" class="no-contacts">No contacts found</td></tr>';
        if (contactCountDisplay) contactCountDisplay.textContent = 'No contacts to display';
        return;
    }

    if (tbody) tbody.innerHTML = '';
    const fragment = document.createDocumentFragment();
    
    // Get the profile picture toggle state
    const { showProfilePictures } = await chrome.storage.local.get(['showProfilePictures']);
    const defaultProfileSvg = `data:image/svg+xml,${encodeURIComponent('<svg width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg"><path d="M12 12C14.21 12 16 10.21 16 8C16 5.79 14.21 4 12 4C9.79 4 8 5.79 8 8C8 10.21 9.79 12 12 12ZM12 14C9.33 14 4 15.34 4 18V20H20V18C20 15.34 14.67 14 12 14Z" fill="currentColor"/></svg>')}`;

    // Get auth credentials for profile pictures
    const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);
    
    for (const contact of contacts) {
        const row = document.createElement('tr');
        
        // Use ContactID from payload instead of calculating from URL
        row.setAttribute('data-profile-id', contact.ContactID);

        const renderedTags = await renderTags(contact.ContactTags);
        
        // Format dates for better display
        const formatDate = (dateString) => {
            if (!dateString) return '-';
            return new Date(dateString).toLocaleString();
        };

        // Create profile image cell with placeholder if pictures are disabled
        let profileImageCell = '';
        const imgSrc = showProfilePictures ? await getProfileImage(email, authToken, contact.ContactID) : defaultProfileSvg;
        profileImageCell = `
            <td class="profile-image-cell">
                <div class="profile-picture-container">
                    <img src="${imgSrc}" 
                         alt="Profile" 
                         class="profile-image"
                         onerror="this.src='${defaultProfileSvg}'">
                        <div class="connection-status-overlay ${contact.existingContact === '1' || contact.existingContact === true ? 'connected' : contact.pendingConnection === 1 ? 'pending' : 'not-connected'}">
                            <i class="fas ${contact.existingContact === '1' || contact.existingContact === true ? 'fa-check' : contact.pendingConnection === 1 ? 'fa-hourglass' : 'fa-times'} icon"></i>
                        </div>
                </div>
            </td>`;

        // Create a clickable profile URL
        const profileUrl = `<a href="${contact.ContactProfileURL}" 
                              target="_blank" 
                              class="profile-link" 
                              title="Open profile in new tab">
                              ${contact.ContactID || 'View Profile'}
                           </a>`;
        
        // Update the connection status cell to use SVG icons
        const connectedIcon = contact.existingContact === '1' || contact.existingContact === true ? `
            <div class="connection-status connected">
                <svg class="icon" width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                    <path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm-2 15l-5-5 1.41-1.41L10 14.17l7.59-7.59L19 8l-9 9z" fill="currentColor"></path>
                </svg>
            </div>
        ` : contact.pendingConnection === 1 ? `
            <div class="connection-status pending">
                <svg class="icon" width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                    <path d="M6 2v6h0.01L6 8.01 10 12l-4 4 0.01 0.01H6V22h12v-5.99h-0.01L18 16l-4-4 4-3.99-0.01-0.01H18V2H6zm10 14.5V20H8v-3.5l4-4 4 4zm-4-5l-4-4V4h8v3.5l-4 4z" fill="#FFA07A"></path>
                </svg>
            </div>
        ` : `
            <div class="connection-status not-connected">
                <svg class="icon" width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                    <path d="M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12z" fill="currentColor"></path>
                </svg>
            </div>
        `;

        // Add engaged status icon
        const engagedIcon = contact.IsEngaged === '1' ? `
            <div class="connection-status connected">
                <svg class="icon" width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                    <path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm-2 15l-5-5 1.41-1.41L10 14.17l7.59-7.59L19 8l-9 9z" fill="currentColor"></path>
                </svg>
            </div>
        ` : `
            <div class="connection-status not-connected">
                <svg class="icon" width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                    <path d="M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12z" fill="currentColor"></path>
                </svg>
            </div>
        `;

        // Determine the profile picture source
        let profilePicSrc;
        if (showProfilePictures && contact.showProfilePictures === true && contact.ContactID) {
            profilePicSrc = await getProfileImage(contact.email, contact.authToken, contact.ContactID);
        } else {
            profilePicSrc = `data:image/svg+xml,${encodeURIComponent('<svg width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg"><path d="M12 12C14.21 12 16 10.21 16 8C16 5.79 14.21 4 12 4C9.79 4 8 5.79 8 8C8 10.21 9.79 12 12 12ZM12 14C9.33 14 4 15.34 4 18V20H20V18C20 15.34 14.67 14 12 14Z" fill="currentColor"/></svg>')}`;
        }

        // Create a safe ID by removing special characters
        const safeId = `activities-cell-${contact.ContactID.replace(/[^a-zA-Z0-9]/g, '_')}`;

        row.innerHTML = `
            <td><input type="checkbox" class="contact-select" 
                data-linkedin-id="${contact.ContactID}"
                data-url="${contact.ContactProfileURL}"></td>
            ${profileImageCell}
            <td>${contact.ContactFirstname || ''}</td>
            <td>${contact.ContactLastname || ''}</td>
            <td>${contact.ContactJobTitle || ''}</td>
            <td>${contact.ContactCurrentCompany || ''}</td>
            <td>${connectedIcon}</td>
            <td>${renderedTags}</td>
            <td id="${safeId}"></td>
            <td>${formatDate(contact.RefreshDatetime)}</td>
        `;

        // Create and append the activities cell using the new function
        const activitiesCell = createActivitiesCell(contact);
        const activitiesCellContainer = row.querySelector(`#${safeId}`);
        if (activitiesCellContainer) {
            activitiesCellContainer.appendChild(activitiesCell);
        }

        fragment.appendChild(row);

        // Add event listeners to checkboxes
        const checkbox = row.querySelector('.contact-select');
        checkbox.addEventListener('click', (e) => {
            if (e.shiftKey && lastChecked) {
                setTimeout(() => {
                    handleShiftClick(checkbox, e);
                }, 0);
            } else {
                lastChecked = checkbox;
                updateMultiSelectControl();
            }
        });

        // Add click handler for activities
        const activitiesCountElement = row.querySelector('.activities-count');
        if (activitiesCountElement) {
            activitiesCountElement.addEventListener('click', (e) => {
                e.stopPropagation();
                showActivitiesModal(JSON.parse(activitiesCountElement.dataset.activities));
            });
        }

        // Add event listeners to profile pictures
        const img = row.querySelector('.profile-image');
        if (img) {
            img.addEventListener('click', () => {
                if (contact.ContactProfileURL) {
                    chrome.tabs.create({ 
                        url: contact.ContactProfileURL,
                        active: false
                    });
                }
            });
        }

        // Add event listeners to profile pictures
        // const img = row.querySelector('.profile-picture.clickable');
        // img.addEventListener('click', (e) => {
        //     const profileUrl = e.target.dataset.profileUrl;
        //     if (profileUrl) {
        //         chrome.tabs.create({ 
        //             url: profileUrl,
        //             active: false
        //         });
        //     }
        // });
    }
    
    if (tbody) tbody.appendChild(fragment);
    
    // Update contact count display with both filtered and total counts
    const totalContacts = window.totalContactsCount || contacts.length;
    const shownContacts = contacts.length;
    
    if (contactCountDisplay) {
        contactCountDisplay.textContent = `Showing ${shownContacts} of ${totalContacts} contacts`;
    }
    
    console.log('Contacts table rendered successfully');
}

function createActivitiesCell(contact) {
    const activitiesCount = contact.activities ? contact.activities.length : 0;
    const td = document.createElement('td');

    if (activitiesCount > 0) {
        // Calculate recent activities (within last 7 days)
        const recentActivities = contact.activities.filter(activity => {
            const activityDate = new Date(activity.postWhen);
            const now = new Date();
            const diffTime = Math.abs(now - activityDate);
            const diffDays = Math.ceil(diffTime / (1000 * 60 * 60 * 24));
            return diffDays <= 7;
        });

        const hasRecentActivity = recentActivities.length > 0;
        const tooltipText = `Total Activities: ${activitiesCount}\nRecent Activities (7 days): ${recentActivities.length}`;

        const div = document.createElement('div');
        div.className = `activities-count${hasRecentActivity ? ' has-recent' : ''}`;
        div.title = tooltipText;
        div.setAttribute('data-activities', JSON.stringify(contact.activities));

        div.innerHTML = `
            <span class="count">${activitiesCount}</span>
            <svg class="icon" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                <path d="M21.99 4c0-1.1-.89-2-1.99-2H4c-1.1 0-2 .9-2 2v12c0 1.1.9 2 2 2h14l4 4-.01-18zM18 14H6v-2h12v2zm0-3H6V9h12v2zm0-3H6V6h12v2z" fill="currentColor"/>
            </svg>
        `;

        td.appendChild(div);
    } else {
        td.textContent = '-';
    }

    return td;
}

function showActivitiesModal(activities) {
    const modal = document.getElementById('activitiesModal');
    const activitiesList = document.getElementById('activitiesList');
    const closeButton = modal.querySelector('.close-button');

    // Clear previous content
    activitiesList.innerHTML = '';

    // Add each activity
    activities.forEach(activity => {
        const [who, when] = activity.whoPostedWhen.split(' • ');
        const activityElement = document.createElement('div');
        activityElement.className = 'activity-item';
        
        // Create context section if it exists
        const contextSection = activity.postContext ? `
            <div class="activity-context">
                <svg class="icon" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                    <path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm1 15h-2v-6h2v6zm0-8h-2V7h2v2z" fill="currentColor"/>
                </svg>
                <span>${activity.postContext}</span>
            </div>
        ` : '';
        
        activityElement.innerHTML = `
            <div class="activity-header">
                <span class="activity-poster">${who}</span>
                <span class="activity-time">${when}</span>
            </div>
            ${contextSection}
            <div class="activity-content">${activity.postContent}</div>
            <div class="activity-footer">
                <div class="activity-stats">
                    <div class="activity-reactions" title="Reactions">
                        <svg class="icon" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                            <path d="M12 21.35l-1.45-1.32C5.4 15.36 2 12.28 2 8.5 2 5.42 4.42 3 7.5 3c1.74 0 3.41.81 4.5 2.09C13.09 3.81 14.76 3 16.5 3 19.58 3 22 5.42 22 8.5c0 3.78-3.4 6.86-8.55 11.54L12 21.35z" fill="currentColor"/>
                        </svg>
                        <span>${activity.postSocialReactions}</span>
                    </div>
                    ${activity.postComments ? `
                        <div class="activity-comments" title="Comments">
                            <svg class="icon" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                                <path d="M20 2H4c-1.1 0-2 .9-2 2v18l4-4h14c1.1 0 2-.9 2-2V4c0-1.1-.9-2-2-2z" fill="currentColor"/>
                            </svg>
                            <span>${activity.postComments}</span>
                        </div>
                    ` : ''}
                </div>
                <a href="${activity.postDirectLink}" target="_blank" class="activity-link">
                    View on LinkedIn
                    <svg class="icon" width="12" height="12" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M19 19H5V5h7V3H5a2 2 0 0 0-2 2v14a2 2 0 0 0 2 2h14a2 2 0 0 0 2-2v-7h-2v7zM14 3v2h3.59l-9.83 9.83 1.41 1.41L19 6.41V10h2V3h-7z" fill="currentColor"/>
                    </svg>
                </a>
            </div>
        `;
        activitiesList.appendChild(activityElement);
    });

    // Show modal
    modal.style.display = 'block';

    // Close modal when clicking close button
    closeButton.onclick = () => {
        modal.style.display = 'none';
    };

    // Close modal when clicking outside
    window.onclick = (event) => {
        if (event.target === modal) {
            modal.style.display = 'none';
        }
    };
}

export async function deleteContact(profileUrls) {
    try {
        // Convert single URL to array if needed
        const urlsArray = Array.isArray(profileUrls) ? profileUrls : [profileUrls];
        
        // Extract just the LinkedIn usernames from URLs
        const profileIds = urlsArray.map(url => {
            // Handle different URL formats
            const matches = url.match(/linkedin\.com\/in\/([^\/]+)/);
            return matches ? matches[1] : null;
        }).filter(id => id); // Filter out any invalid IDs

        if (profileIds.length === 0) {
            throw new Error('No valid profile IDs found');
        }

        console.log('Deleting profiles:', profileIds); // Debug log

        // Get authentication details
        const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);

        // Call the delete endpoint
        const response = await fetch('https://www.goai-app.com/deleteContacts', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({
                email,
                authToken,
                contacts: profileIds
            })
        });

        if (!response.ok) {
            throw new Error('Failed to delete contacts');
        }

        const result = await response.json();
        if (!result.success) {
            throw new Error(result.message || 'Failed to delete contacts');
        }

        // Reload contacts to update the display
        await loadContacts();
        showSuccess(`Successfully deleted ${profileIds.length} contact${profileIds.length === 1 ? '' : 's'}`);
    } catch (error) {
        console.error('Error deleting contacts:', error);
        showError('Failed to delete contacts: ' + error.message);
    }
}

function updateMultiSelectControl() {
    const selectedCheckboxes = document.querySelectorAll('.contact-select:checked');
    const multiSelectControl = document.getElementById('multiSelectControl');
    const selectedCount = document.getElementById('selectedCount');
    
    if (selectedCheckboxes.length > 0) {
        if (multiSelectControl) multiSelectControl.style.display = 'block';
        if (selectedCount) selectedCount.textContent = `${selectedCheckboxes.length} contact${selectedCheckboxes.length === 1 ? '' : 's'} selected`;
    } else {
        if (multiSelectControl) multiSelectControl.style.display = 'none';
    }
}

export async function applyAdvancedFilters() {
    try {
        // Get authentication details
        const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);

        // Build query parameters
        const params = new URLSearchParams({
            email: email,
            authToken: authToken
        });

        // Add basic filters
        const firstName = document.getElementById('firstNameFilter').value.trim();
        const lastName = document.getElementById('lastNameFilter').value.trim();
        const company = document.getElementById('companyFilter').value.trim();
        const jobTitle = document.getElementById('jobTitleFilter').value.trim();

        if (firstName) params.append('firstName', firstName);
        if (lastName) params.append('lastName', lastName);
        if (company) params.append('company', company);
        if (jobTitle) params.append('jobTitle', jobTitle);

        // Add boolean filters
        const existingContact = document.getElementById('existingContactFilter').value;
        const isEngaged = document.getElementById('isEngagedFilter').value;

        if (existingContact !== '') {
            params.append('existingContact', existingContact);
        }
        if (isEngaged !== '') {
            params.append('isEngaged', isEngaged);
        }

        // Add date filters
        const dateFilters = [
            {
                operator: 'connectionRequestOperator',
                value: 'connectionRequestDate',
                paramOperator: 'ConnectionRequestDatetimeOperator',
                paramValue: 'ConnectionRequestDatetimeValue'
            },
            {
                operator: 'lastMessageOperator',
                value: 'lastMessageDate',
                paramOperator: 'LastMessageSentDatetimeOperator',
                paramValue: 'LastMessageSentDatetimeValue'
            },
            {
                operator: 'refreshDateOperator',
                value: 'refreshDate',
                paramOperator: 'RefreshDatetimeOperator',
                paramValue: 'RefreshDatetimeValue'
            }
        ];

        dateFilters.forEach(filter => {
            const operator = document.getElementById(filter.operator).value;
            
            if (operator === 'none') {
                // Special case for "None" option
                params.append(filter.paramOperator, 'none');
            } else if (operator) {
                const date = document.getElementById(filter.value).value;
                if (date) {
                    // Convert local datetime to UTC ISO string
                    const utcDate = new Date(date).toISOString();
                    params.append(filter.paramOperator, operator);
                    params.append(filter.paramValue, utcDate);
                }
            }
        });

        // Add tags filter
        const selectedTags = Array.from(document.getElementById('tagsMultiFilter').selectedOptions)
            .map(option => option.value);
        if (selectedTags.length > 0) {
            params.append('tags', selectedTags.join(','));
        }

        // Make API request
        const response = await fetch(`https://www.goai-app.com/getFilteredContacts?${params.toString()}`);
        
        if (!response.ok) {
            throw new Error('Failed to fetch filtered contacts');
        }

        const data = await response.json();
        if (!data.success) {
            throw new Error('API response unsuccessful');
        }

        // Render the filtered contacts
        await renderContactsTable(data.contacts);
        showSuccess(`Showing ${data.contacts.length} filtered contacts`);
    } catch (error) {
        console.error('Error applying filters:', error);
        showError('Failed to apply filters: ' + error.message);
    }
}

export async function clearAllFilters() {
    // Clear all filter inputs
    document.getElementById('firstNameFilter').value = '';
    document.getElementById('lastNameFilter').value = '';
    document.getElementById('companyFilter').value = '';
    document.getElementById('jobTitleFilter').value = '';
    document.getElementById('existingContactFilter').value = '';
    document.getElementById('isEngagedFilter').value = '';
    document.getElementById('connectionRequestOperator').value = '';
    document.getElementById('connectionRequestDate').value = '';
    document.getElementById('lastMessageOperator').value = '';
    document.getElementById('lastMessageDate').value = '';
    document.getElementById('refreshDateOperator').value = '';
    document.getElementById('refreshDate').value = '';
    document.getElementById('tagsMultiFilter').selectedIndex = -1;
    document.getElementById('selectedTagsDisplay').innerHTML = '';

    // Reload all contacts
    await loadContacts();
    showSuccess('Filters cleared');
}

export async function exportCurrentContacts() {
    try {
        // Get all rows from the current table
        const tbody = document.getElementById('contactsTableBody');
        const rows = tbody.querySelectorAll('tr');
        
        if (rows.length === 0) {
            showError('No contacts to export');
            return;
        }

        // Get the data directly from the table cells
        const contacts = Array.from(rows).map(row => {
            const cells = row.querySelectorAll('td');
            const profileUrl = cells[1].querySelector('.profile-picture').getAttribute('data-profile-url');
            const isConnected = cells[1].querySelector('.connection-status-overlay').classList.contains('connected');
            const isPending = cells[1].querySelector('.connection-status-overlay').classList.contains('pending');
            
            return {
                ContactProfileURL: profileUrl || '',
                ContactFirstname: cells[2]?.textContent || '',
                ContactLastname: cells[3]?.textContent || '',
                ContactJobTitle: cells[4]?.textContent || '',
                ContactCurrentCompany: cells[5]?.textContent || '',
                ContactTags: cells[6]?.textContent || '',
                existingContact: isConnected ? '1' : isPending ? 'P' : '0',
                ContactJSON: '{}',  // Empty object as default
                ConnectionRequestDatetime: '',  // Empty as we don't have this in the table
                LastMessageSentDatetime: '',    // Empty as we don't have this in the table
                IsEngaged: '0',                 // Default to not engaged
                RefreshDatetime: new Date().toISOString()  // Current time as refresh time
            };
        });

        // Convert to CSV
        const headers = [
            'ContactProfileURL',
            'ContactFirstname',
            'ContactLastname',
            'ContactJobTitle',
            'ContactCurrentCompany',
            'ContactTags',
            'existingContact',
            'ContactJSON',
            'ConnectionRequestDatetime',
            'LastMessageSentDatetime',
            'IsEngaged',
            'RefreshDatetime'
        ];

        const csvContent = [
            // Headers
            headers.join(','),
            // Data rows
            ...contacts.map(contact => 
                headers.map(header => {
                    const value = contact[header] || '';
                    // Escape commas and quotes in the value
                    return `"${String(value).replace(/"/g, '""')}"`;
                }).join(',')
            )
        ].join('\n');

        // Create and trigger download
        const blob = new Blob([csvContent], { type: 'text/csv;charset=utf-8;' });
        const url = URL.createObjectURL(blob);
        const link = document.createElement('a');
        link.setAttribute('href', url);
        link.setAttribute('download', `contacts_export_${new Date().toISOString().slice(0,10)}.csv`);
        document.body.appendChild(link);
        link.click();
        document.body.removeChild(link);
        URL.revokeObjectURL(url);

        showSuccess(`Successfully exported ${contacts.length} contacts`);
    } catch (error) {
        console.error('Error exporting contacts:', error);
        showError('Failed to export contacts: ' + error.message);
    }
}

export async function handleCsvImport(event) {
    const file = event.target.files[0];
    if (!file) {
        showError('No file selected');
        return;
    }

    try {
        const csvText = await readFileAsText(file);
        const { headers, data } = parseCSV(csvText);
        
        // Validate and map headers
        const mappedHeaders = validateAndMapHeaders(headers);
        if (!mappedHeaders.ContactProfileURL) {
            showError('CSV must contain a ContactProfileURL column');
            return;
        }

        // Transform data to match expected format
        const transformedData = transformCsvData(data, mappedHeaders);

        // Get authentication details
        const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);

        // Send to API
        const response = await fetch('https://www.goai-app.com/importContacts', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({
                email,
                authToken,
                contacts: transformedData
            })
        });

        if (!response.ok) {
            throw new Error('Failed to import contacts');
        }

        const result = await response.json();
        if (!result.success) {
            throw new Error(result.message || 'Failed to import contacts');
        }

        // Reload contacts to show new data
        await loadContacts();
        showSuccess(`Successfully imported ${transformedData.length} contacts`);
    } catch (error) {
        console.error('Error importing contacts:', error);
        showError('Failed to import contacts: ' + error.message);
    }
}

function readFileAsText(file) {
    return new Promise((resolve, reject) => {
        const reader = new FileReader();
        reader.onload = (e) => resolve(e.target.result);
        reader.onerror = (e) => reject(e);
        reader.readAsText(file);
    });
}

function parseCSV(csvText) {
    const lines = csvText.split(/\r?\n/).map(line => line.trim()).filter(line => line);
    const headers = lines[0].split(',').map(header => header.trim());
    
    const data = [];
    
    // Process each line after headers
    for (let i = 1; i < lines.length; i++) {
        const line = lines[i];
        const values = [];
        let currentValue = '';
        let insideQuotes = false;
        
        // Parse each character to handle quoted values properly
        for (let j = 0; j < line.length; j++) {
            const char = line[j];
            
            if (char === '"') {
                if (insideQuotes && line[j + 1] === '"') {
                    // Handle escaped quotes
                    currentValue += '"';
                    j++;
                } else {
                    // Toggle quote state
                    insideQuotes = !insideQuotes;
                }
            } else if (char === ',' && !insideQuotes) {
                // End of value
                values.push(currentValue.trim());
                currentValue = '';
            } else {
                currentValue += char;
            }
        }
        
        // Add the last value
        values.push(currentValue.trim());
        
        // Create object from headers and values
        if (values.length === headers.length) {
            const row = {};
            headers.forEach((header, index) => {
                row[header] = values[index];
            });
            data.push(row);
        }
    }

    return { headers, data };
}

function validateAndMapHeaders(headers) {
    // Define expected headers and their possible variations
    const headerMappings = {
        'ContactProfileURL': ['url', 'profile', 'linkedin', 'link'],
        'ContactFirstname': ['first', 'firstname', 'given'],
        'ContactLastname': ['last', 'lastname', 'surname', 'family'],
        'ContactJobTitle': ['title', 'job', 'position', 'role'],
        'ContactCurrentCompany': ['company', 'organization', 'employer', 'business'],
        'ContactTags': ['tag', 'tags', 'label'],
        'existingContact': ['connected', 'connection', 'existing'],
        'ConnectionRequestDatetime': ['request', 'connection date', 'connected date'],
        'LastMessageSentDatetime': ['message', 'last message', 'sent'],
        'IsEngaged': ['engaged', 'engagement'],
        'RefreshDatetime': ['refresh', 'updated', 'last update']
    };

    // Create the mapping
    const mappedHeaders = {};
    headers.forEach(header => {
        // Convert header to lowercase and remove special characters for matching
        const normalizedHeader = header.toLowerCase().trim();
        
        for (const [standardHeader, keywords] of Object.entries(headerMappings)) {
            // Check if any of the keywords are present in the header
            if (keywords.some(keyword => normalizedHeader.includes(keyword.toLowerCase()))) {
                mappedHeaders[standardHeader] = header;
                break;
            }
        }
    });

    // Debug logging
    console.log('Header mapping:', {
        original: headers,
        mapped: mappedHeaders
    });

    return mappedHeaders;
}

function transformCsvData(data, mappedHeaders) {
    return data
        // Filter out empty rows (rows where all values are empty)
        .filter(row => Object.values(row).some(value => value && value.trim() !== ''))
        .map((row, index) => {
            const transformedRow = {};
            
            // Map each standard header to its value in the CSV
            for (const [standardHeader, csvHeader] of Object.entries(mappedHeaders)) {
                // Get the raw value from the CSV
                let value = row[csvHeader];
                
                // Remove any surrounding quotes and trim whitespace
                if (typeof value === 'string') {
                    value = value.replace(/^["']|["']$/g, '').trim();
                }

                // Handle specific field types
                switch (standardHeader) {
                    case 'IsEngaged':
                    case 'existingContact':
                        transformedRow[standardHeader] = convertToBoolean(value);
                        break;
                        
                    case 'ConnectionRequestDatetime':
                    case 'LastMessageSentDatetime':
                    case 'RefreshDatetime':
                        transformedRow[standardHeader] = value ? formatDate(value) : '';
                        break;
                        
                    case 'ContactTags':
                        // Ensure tags are properly formatted
                        transformedRow[standardHeader] = value ? value.split(',')
                            .map(tag => tag.trim())
                            .filter(tag => tag)
                            .join(',') : '';
                        break;
                        
                    default:
                        transformedRow[standardHeader] = value || '';
                }
            }

            // Debug logging
            console.log('Row data:', {
                originalRow: row,
                mappedHeaders,
                transformedRow,
                profileUrl: transformedRow.ContactProfileURL
            });

            // Ensure required fields
            if (!transformedRow.ContactProfileURL) {
                throw new Error(`Missing required ContactProfileURL in row ${index + 1}. Headers: ${JSON.stringify(mappedHeaders)}. Row data: ${JSON.stringify(row)}`);
            }

            return transformedRow;
        });
}

function convertToBoolean(value) {
    if (typeof value === 'boolean') return value;
    if (typeof value === 'string') {
        value = value.toLowerCase().trim();
        return value === 'true' || 
               value === 'yes' || 
               value === '1' || 
               value === 'y' || 
               value === 't';
    }
    if (typeof value === 'number') return value === 1;
    return false;
}

function formatDate(dateStr) {
    if (!dateStr) return '';
    
    try {
        // Try parsing as ISO date first
        let date = new Date(dateStr);
        
        // If invalid, try other common formats
        if (isNaN(date.getTime())) {
            // Try DD/MM/YYYY or MM/DD/YYYY
            const parts = dateStr.split(/[\/\-\.]/);
            if (parts.length === 3) {
                // Try both date formats
                date = new Date(parts[2], parts[1] - 1, parts[0]); // DD/MM/YYYY
                if (isNaN(date.getTime())) {
                    date = new Date(parts[2], parts[0] - 1, parts[1]); // MM/DD/YYYY
                }
            }
        }
        
        return isNaN(date.getTime()) ? '' : date.toISOString();
    } catch {
        return '';
    }
}

// Add other table-related functions:
// - updateTagFilterOptions()
// - handleCsvImport()
// - exportContacts()
// - applyAdvancedFilters()
// - clearAllFilters()
// - deleteContact()
// etc... 
