// Import error utilities
import { showError, showSuccess } from '../utils/errorUtils.js';

// Private functions
function setupTagManagerEventListeners() {
    const elements = {
        tagManagerBtn: document.getElementById('tagManagerBtn'),
        addGlobalTag: document.getElementById('addGlobalTag'),
        saveGlobalTags: document.getElementById('saveGlobalTags'),
        cancelGlobalTags: document.getElementById('cancelGlobalTags'),
        exportTags: document.getElementById('exportTags'),
        newGlobalTagInput: document.getElementById('newGlobalTag')
    };

    if (elements.tagManagerBtn) {
        elements.tagManagerBtn.addEventListener('click', openTagManager);
    }
    if (elements.addGlobalTag) {
        elements.addGlobalTag.addEventListener('click', addNewGlobalTag);
    }
    if (elements.saveGlobalTags) {
        elements.saveGlobalTags.addEventListener('click', saveGlobalTagChanges);
    }
    if (elements.cancelGlobalTags) {
        elements.cancelGlobalTags.addEventListener('click', closeTagManager);
    }
    if (elements.exportTags) {
        elements.exportTags.addEventListener('click', exportTagsToFile);
    }
    if (elements.newGlobalTagInput) {
        elements.newGlobalTagInput.addEventListener('keypress', (e) => {
            if (e.key === 'Enter') {
                e.preventDefault();
                addNewGlobalTag();
            }
        });
    }
}

async function loadInitialTags() {
    try {
        const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);
        
        const response = await fetch('https://www.goai-app.com/manageTags', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({
                email,
                authToken,
                action: 'list'
            })
        });

        if (!response.ok) {
            throw new Error('Failed to fetch tags');
        }

        const data = await response.json();
        if (!data.success) {
            throw new Error('API response unsuccessful');
        }

        // Store tags in local storage for quick access
        await chrome.storage.local.set({ 
            globalTags: data.tags.map(t => t.name),
            tagColors: data.tags.reduce((acc, t) => ({ ...acc, [t.name]: t.color }), {})
        });
        
        await renderGlobalTags(data.tags);
    } catch (error) {
        console.error('Error loading tags:', error);
        showError('Failed to load tags: ' + error.message);
        await renderGlobalTags([]);
    }
}

async function renderGlobalTags(tags = []) {
    console.log('Rendering global tags:', tags);
    const tagsContainer = document.getElementById('globalTagsList');
    if (!tagsContainer) return;

    tagsContainer.innerHTML = '';

    const tagArray = Array.isArray(tags) ? tags : [];

    tagArray.forEach(tagObj => {
        const tagElement = document.createElement('div');
        tagElement.className = 'global-tag';
        tagElement.innerHTML = `
            <span style="background-color: ${tagObj.color || '#e1e1e1'}">${tagObj.name}</span>
            <button class="remove-tag" data-tag="${tagObj.name}">&times;</button>
        `;
        tagsContainer.appendChild(tagElement);
    });

    document.querySelectorAll('.remove-tag').forEach(button => {
        button.addEventListener('click', async (e) => {
            const tagName = e.target.dataset.tag;
            if (tagName) {
                await removeTag(tagName);
            }
        });
    });
}

function closeTagManager() {
    const modal = document.getElementById('tagManagerModal');
    if (modal) {
        modal.style.display = 'none';
    }
}

async function addNewGlobalTag() {
    const input = document.getElementById('newGlobalTag');
    const colorInput = document.getElementById('tagColorPicker');
    if (!input || !colorInput) return;

    const tagName = input.value.trim().replace(/[^a-zA-Z0-9]/g, '-');
    const color = colorInput.value;
    
    if (!tagName) return;

    try {
        const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);
        
        const response = await fetch('https://www.goai-app.com/manageTags', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({
                email,
                authToken,
                action: 'add',
                tag: {
                    name: tagName,
                    color: color
                }
            })
        });

        if (!response.ok) {
            throw new Error('Failed to add tag');
        }

        const result = await response.json();
        if (!result.success) {
            throw new Error(result.message || 'Failed to add tag');
        }

        // Refresh the tags list
        await loadInitialTags();
        
        input.value = '';
        colorInput.value = '#e1e1e1';
        showSuccess('Tag added successfully');
    } catch (error) {
        console.error('Error adding tag:', error);
        showError('Failed to add tag: ' + error.message);
    }
}

async function saveGlobalTagChanges() {
    try {
        const tagElements = document.querySelectorAll('.global-tag span');
        if (!tagElements) {
            throw new Error('No tag elements found');
        }

        const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);

        // Get current tag with its color
        const tagSpan = tagElements[0]; // Get the first tag since we're updating one at a time
        const tag = {
            name: tagSpan.textContent.trim(),
            color: tagSpan.style.backgroundColor || '#e1e1e1'
        };

        // Save to API
        const response = await fetch('https://www.goai-app.com/manageTags', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({
                email,
                authToken,
                action: 'update',
                tag: tag // Send single tag object instead of array
            })
        });

        if (!response.ok) {
            throw new Error('Failed to save tag');
        }

        const result = await response.json();
        if (!result.success) {
            throw new Error(result.message || 'Failed to save tag');
        }

        // Update local storage
        const { globalTags = [], tagColors = {} } = await chrome.storage.local.get(['globalTags', 'tagColors']);
        tagColors[tag.name] = tag.color;
        
        await chrome.storage.local.set({ 
            globalTags: [...globalTags, tag.name],
            tagColors
        });

        showSuccess('Tag saved successfully');
        closeTagManager();
        
        // Update tag options in the multi-select
        await updateMultiSelectTagOptions();
    } catch (error) {
        console.error('Error saving tag:', error);
        showError('Failed to save tag: ' + error.message);
    }
}

async function exportTagsToFile() {
    try {
        const { globalTags = [], tagColors = {} } = await chrome.storage.local.get(['globalTags', 'tagColors']);
        
        if (!globalTags.length) {
            showError('No tags to export');
            return;
        }

        const tagConfigData = globalTags.map(tag => ({
            name: tag,
            color: tagColors[tag] || '#e1e1e1'
        }));

        const blob = new Blob([JSON.stringify(tagConfigData, null, 2)], { type: 'text/plain' });
        const url = URL.createObjectURL(blob);
        const a = document.createElement('a');
        a.href = url;
        a.download = 'Tags.txt';
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
        URL.revokeObjectURL(url);

        showSuccess('Tags exported successfully');
    } catch (error) {
        console.error('Error exporting tags:', error);
        showError('Failed to export tags: ' + error.message);
    }
}

async function updateMultiSelectTagOptions() {
    await fetchAndPopulateTags('multiSelectTag');
}

async function removeTag(tagName) {
    try {
        const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);
        
        const response = await fetch('https://www.goai-app.com/manageTags', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({
                email,
                authToken,
                action: 'remove',
                tag: {
                    name: tagName
                }
            })
        });

        if (!response.ok) {
            throw new Error('Failed to remove tag');
        }

        const result = await response.json();
        if (!result.success) {
            throw new Error(result.message || 'Failed to remove tag');
        }

        // Refresh the tags list
        await loadInitialTags();
        showSuccess('Tag removed successfully');
    } catch (error) {
        console.error('Error removing tag:', error);
        showError('Failed to remove tag: ' + error.message);
    }
}

// Exported functions
export async function initializeTagManager() {
    console.log('Initializing tag manager...');
    setupTagManagerEventListeners();
    await loadInitialTags();
}

export function openTagManager() {
    const modal = document.getElementById('tagManagerModal');
    if (modal) {
        modal.style.display = 'block';
    }
}

export function updateTagFilterOptions(contacts) {
    const tagsMultiFilter = document.getElementById('tagsMultiFilter');
    if (!tagsMultiFilter) return;

    const allTags = new Set();
    contacts.forEach(contact => {
        if (contact.ContactTags) {
            const tags = contact.ContactTags.split(',').map(t => t.trim()).filter(t => t);
            tags.forEach(tag => allTags.add(tag));
        }
    });

    const sortedTags = Array.from(allTags).sort();
    tagsMultiFilter.innerHTML = '';
    sortedTags.forEach(tag => {
        const option = document.createElement('option');
        option.value = tag;
        option.textContent = tag;
        tagsMultiFilter.appendChild(option);
    });
}

export async function renderTags(tags) {
    if (!tags) return '';
    const { tagColors = {} } = await chrome.storage.local.get('tagColors');
    const tagArray = tags.split(',').map(t => t.trim()).filter(t => t);
    return tagArray.map(tag => 
        `<span class="tag" style="background-color: ${tagColors[tag] || '#e1e1e1'}">${tag}</span>`
    ).join('');
}

// Export all public functions
export {
    updateMultiSelectTagOptions
};

// Add this new function to fetch and populate tags
export async function fetchAndPopulateTags(selectElementId) {
    try {
        const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);
        
        const response = await fetch('https://www.goai-app.com/manageTags', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({
                email,
                authToken,
                action: 'list'
            })
        });

        if (!response.ok) {
            throw new Error('Failed to fetch tags');
        }

        const data = await response.json();
        if (!data.success) {
            throw new Error('API response unsuccessful');
        }

        // Populate the specified select element
        const selectElement = document.getElementById(selectElementId);
        if (!selectElement) return;

        // Clear existing options
        selectElement.innerHTML = selectElementId === 'multiSelectTag' ? 
            '<option value="">Select a tag...</option>' : '';

        // Add new options
        data.tags.forEach(tag => {
            const option = document.createElement('option');
            option.value = tag.name;
            option.textContent = tag.name;
            selectElement.appendChild(option);
        });

        // Store tags in local storage for other uses
        await chrome.storage.local.set({ 
            globalTags: data.tags.map(t => t.name),
            tagColors: data.tags.reduce((acc, t) => ({ ...acc, [t.name]: t.color }), {})
        });

    } catch (error) {
        console.error('Error fetching tags:', error);
        showError('Failed to fetch tags: ' + error.message);
    }
}