// Import dependencies from messageHandlers.js
import {
    findElement,
    logMessageToAPI,
    isPendingConnection,
    handleConnectButton,
    addNoteAndSendRequest,
    dispatchMessageEvent,
    dispatchConnectionEvent
} from './messageHandlers.js';

// New dedicated function for automated connection requests
export async function sendAutomatedConnectionRequest(message, PROFILE_SELECTORS) {
    try {
        console.group('🤖 Automation - Sending Connection Request');
        console.log('Starting connection request process...');

        if (await isPendingConnection()) {
            throw new Error('Connection request is already pending');
        }

        // Get the top section using PROFILE_SELECTORS
        const topSection = await findElement(
            () => document.querySelector(PROFILE_SELECTORS.topSectionNew),
            5000,
            'Main top profile section'
        );

        if (!topSection) {
            throw new Error('Main top profile section not found');
        }

        await handleConnectButton();
        await addNoteAndSendRequest(message);

        // Log the sent connection message
        await logMessageToAPI(
            window.location.href,
            'connection-message',
            message
        );

        dispatchConnectionEvent(true);
        console.log('✅ Connection request sent successfully');
        console.groupEnd();
        return { success: true };
    } catch (error) {
        console.warn('Error sending automated connection request:', error);
        dispatchConnectionEvent(false, error.message);
        console.groupEnd();
        return { success: false, error: error.message };
    }
}

// New dedicated function for automated direct messages
export async function sendAutomatedDirectMessage(message, PROFILE_SELECTORS) {
    try {
        console.group('🤖 Automation - Sending Direct Message');
        console.log('Starting direct message process...');
        console.log('Message to send:', message);

        chrome.runtime.sendMessage({
            action: 'sendAnalyticsEvent',
            eventName: 'send_linkedin_message_started',
            eventParams: {}
        });

        const topSection = document.querySelector(PROFILE_SELECTORS.topSectionNew);
        if (!topSection) {
            throw new Error('Main top profile section not found');
        }

        const messageButton = await findElement(() => 
            Array.from(topSection.querySelectorAll('span'))
                .find(span => span.innerText === "Message")
        );

        if (!messageButton) {
            throw new Error('Message button not found');
        }

        console.log('Waiting 5 seconds before clicking message button...');
        await new Promise(resolve => setTimeout(resolve, 1000));

        messageButton.click();

        const msgFormContentEditable = await findElement(() => 
            document.querySelector('.msg-form__contenteditable')
        );
        msgFormContentEditable.innerHTML = `<p>${message}</p>`;
        msgFormContentEditable.dispatchEvent(new Event('input', { bubbles: true }));

        await new Promise(resolve => setTimeout(resolve, 1000));

        const sendButton = await findElement(() => 
            document.querySelector('.msg-form__send-button')
        );
        sendButton.removeAttribute('disabled');
        sendButton.click();

        await new Promise(resolve => setTimeout(resolve, 1000));

        const dismissButton = await findElement(() => 
            Array.from(document.querySelectorAll('button'))
                .find(button => button.querySelector('use[href="#close-small"]'))
        );
        dismissButton.click();

        dispatchMessageEvent(true);

        await new Promise(resolve => setTimeout(resolve, 1000));

        // Log the sent direct message
        await logMessageToAPI(
            window.location.href,
            'direct-message',
            message
        );

        // Send success status
        chrome.runtime.sendMessage({
            type: 'LINKEDIN_GOAI_MESSAGE_OUTCOME',
            success: true
        });

        console.log('✅ Direct message sent successfully');
        console.groupEnd();
        return { success: true };
    } catch (error) {
        console.warn('Error sending automated direct message:', error);
        // Send failure status
        chrome.runtime.sendMessage({
            type: 'LINKEDIN_GOAI_MESSAGE_OUTCOME',
            success: false,
            error: error.message
        });
        console.groupEnd();
        return { success: false, error: error.message };
    }
}

// New dedicated function for handling profile visits
export async function handleProfileVisit(PROFILE_SELECTORS) {
    try {
        console.group('🤖 Automation - Visiting Profile');
        console.log('Starting profile visit process...');

        // Check if profile page is loaded
        const topSection = await findElement(
            () => document.querySelector(PROFILE_SELECTORS.topSectionNew),
            5000,
            'Main top profile section'
        );

        if (!topSection) {
            throw new Error('Main top profile section not found');
        }

        // Log the profile visit
        await logMessageToAPI(
            window.location.href,
            'profile-visit',
            ''
        );

        // Send success status
        chrome.runtime.sendMessage({
            type: 'LINKEDIN_GOAI_VISIT_OUTCOME',
            success: true
        });

        console.log('✅ Profile visited successfully');
        console.groupEnd();
        return { success: true };
    } catch (error) {
        console.warn('Error visiting profile:', error);
        // Send failure status
        chrome.runtime.sendMessage({
            type: 'LINKEDIN_GOAI_VISIT_OUTCOME',
            success: false,
            error: error.message
        });
        console.groupEnd();
        return { success: false, error: error.message };
    }
}

// Expose functions globally
window.sendAutomatedConnectionRequest = sendAutomatedConnectionRequest;
window.sendAutomatedDirectMessage = sendAutomatedDirectMessage;
window.handleProfileVisit = handleProfileVisit;
