import { PROFILE_SELECTORS } from './profileHandler.js';
import { API_BASE_URL } from './constants.js';
import { showStatus, hideStatus } from './uiHandler.js';

// Add this at the top of messageHandlers.js
const debugLog = (...args) => {
    console.group('🔍 GoAI Debug:');
    console.log(...args);
    console.groupEnd();
};

// Setup message handlers
function setupMessageHandlers() {
    setupEventListeners();
}

// Setup event listeners for custom events
function setupEventListeners() {
    const eventHandlers = {
        'LINKEDIN_GOAI_START': async (event) => {
            chrome.runtime.sendMessage({
                action: 'sendAnalyticsEvent',
                eventName: 'goai_start_triggered',
                eventParams: {}
            });
            
            try {
                const url = window.location.href;
                const contactId = url.split('/in/')[1]?.replace(/\/$/, '');
                
                const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);
                
                const response = await fetch(`${API_BASE_URL}/getContact?email=${encodeURIComponent(email)}&authToken=${authToken}&contactId=${contactId}`, {
                    method: 'GET',
                    headers: {
                        'Accept': 'application/json'
                    }
                });

                const data = await response.json();
                
                if (data.success && data.contact) {
                    const lastRefresh = new Date(data.contact.RefreshDatetime);
                    const now = new Date();
                    const timeDiff = (now - lastRefresh) / 1000 / 60;
                    
                    if (timeDiff < 2) {
                        showStatus('Already Captured Recently!', 'info', 5000);
                        const profileImage = document.querySelector('[class^="pv-top-card-profile-picture__image"]');
                        if (profileImage) {
                            window.applyScrapedStyle(profileImage);
                        }
                        return { success: true, alreadyCaptured: true, data: data.contact };
                    }
                }

                const scrapeResponse = await chrome.runtime.sendMessage({
                    type: 'START_ANALYSIS',
                    url: url,
                });

                if (scrapeResponse?.success) {
                    chrome.runtime.sendMessage({
                        action: 'sendAnalyticsEvent',
                        eventName: 'goai_start_completed',
                        eventParams: { success: true }
                    });
                }

                return scrapeResponse;
            } catch (error) {
                chrome.runtime.sendMessage({
                    action: 'sendAnalyticsEvent',
                    eventName: 'goai_start_completed',
                    eventParams: { success: false, error: error.message }
                });
                window.showStatus('Capture failed: ' + error.message, 'error');
                window.hideStatus(8000);
                throw error;
            }
        },

        'LINKEDIN_GOAI_START_AUTO': async (event) => {
            chrome.runtime.sendMessage({
                action: 'sendAnalyticsEvent',
                eventName: 'goai_auto_start_triggered',
                eventParams: {
                    autoType: event.detail?.AutoType,
                    hasMessage: !!event.detail?.AutoMessage
                }
            });
            
            try {
                const url = window.location.href;
                const isMessageGeneration = !event.detail?.AutoMessage;
                
                window.ScrapingSettings = {
                    deepActivityScraping: false,
                    autoScrape: true,
                };

                let response;
                if(isMessageGeneration) {
                    const profileData = await window.extractProfileData();
                    console.log('!!!!!!!!!!Profile data extracted:', profileData);

                    // Check if user is a 1st-degree connection
                    const isExistingConnection = profileData.connectionDegree === '1st';
                    
                    if (isExistingConnection) {
                        const generatedResponse = await generateMessages(profileData);
                        displayGeneratedMessages(generatedResponse);   
                        await new Promise(resolve => setTimeout(resolve, 5000));                        
                        clickFirstSendButton(); 
                    } else {
                        const generatedResponse = await generateConnectMessages(profileData);
                        displayGeneratedConnectMessages(generatedResponse); 
                        await new Promise(resolve => setTimeout(resolve, 5000));                        
                        clickFirstConnectButton();    
                    }
                } else {
                    const result = await sendLinkedInMessage(event.detail?.AutoMessage);
                    response = { success: result.success };
                    if (!result.success) {
                        response.error = result.error;
                    }
                }

                document.dispatchEvent(new CustomEvent('LINKEDIN_GOAI_AUTO_COMPLETE', {
                    detail: {
                        ...response,
                        messages: response?.messages || []
                    }
                }));

                chrome.runtime.sendMessage({
                    action: 'sendAnalyticsEvent',
                    eventName: 'goai_auto_start_completed',
                    eventParams: { success: true }
                });
                return response;
            } catch (error) {
                console.warn('Error in automated analysis:', error);
                document.dispatchEvent(new CustomEvent('LINKEDIN_GOAI_AUTO_COMPLETE', {
                    detail: { success: false, error: error.message }
                }));
                throw error;
            }
        }
    };

    // Register event listeners
    Object.entries(eventHandlers).forEach(([event, handler]) => {
        document.addEventListener(event, handler);
    });

    // Setup message handlers
    chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
        console.group('Content Script: Message Received');
        console.log('📥 Message type:', request.type || request.action);
        console.log('📥 Message data:', request);

        const messageHandlers = {
            'ANALYZE_PROFILE': handleAnalyzeProfile,
            'SCRAPE_PROFILE': handleScrapeProfile,
            'LINKEDIN_GOAI_START_AUTO': handleLinkedInGoAiStartAuto
        };

        const handler = messageHandlers[request.type || request.action];
        if (handler) {
            return handler(request, sendResponse);
        }

        console.groupEnd();
        return true;
    });
}

// Add these handler functions
async function handleAnalyzeProfile(request, sendResponse) {
    try {
        const response = await chrome.runtime.sendMessage({
            type: 'START_ANALYSIS',
            url: window.location.href,
        });

        if (response.alreadyCaptured) {
            showStatus(
                `Already captured in Contact Manager!\nLast captured: ${formatDate(response.captureDate)}`, 
                'info',
                5000
            );
            
            const profileImage = document.querySelector('[class^="pv-top-card-profile-picture__image"]');
            if (profileImage) {
                window.applyScrapedStyle(profileImage);
            }
        }

        sendResponse({ success: true });
    } catch (error) {
        console.error('Error in ANALYZE_PROFILE:', error);
        sendResponse({ success: false, error: error.message });
    }
    return true;
}

function handleScrapeProfile(request, sendResponse) {
    const currentUrl = window.location.href;
    
    if (window.isScrapingInProgress && window.lastScrapedUrl === currentUrl) {
        console.log('🚫 Duplicate scrape request prevented');
        sendResponse({ success: false, error: 'Scraping already in progress' });
        return;
    }

    chrome.runtime.sendMessage({ 
        type: 'START_ANALYSIS',
        url: currentUrl 
    }, handleScrapeResponse.bind(null, sendResponse, currentUrl));
    
    return true;
}

function handleLinkedInGoAiStartAuto(request, sendResponse) {
    document.dispatchEvent(new CustomEvent('LINKEDIN_GOAI_START_AUTO', {
        detail: {
            AutoType: request.AutoType,
            AutoMessage: request.AutoMessage
        }
    }));
    sendResponse({ success: true });
    return true;
}

function handleScrapeResponse(sendResponse, currentUrl, response) {
    if (response.alreadyCaptured) {
        handleAlreadyCaptured(response, sendResponse);
        return;
    }

    handleNewScrape(currentUrl, sendResponse);
}

function handleAlreadyCaptured(response, sendResponse) {
    console.log('Profile already captured in CSV');
    showStatus(
        `Already captured in Contact Manager!\nLast captured: ${formatDate(response.captureDate)}`, 
        'info',
        5000
    );
    
    const profileImage = document.querySelector('[class^="pv-top-card-profile-picture__image"]');
    if (profileImage) {
        window.applyScrapedStyle(profileImage);
    }

    sendResponse({ success: true, data: response.data });
}

function handleNewScrape(currentUrl, sendResponse) {
    console.log('🔄 Scraping profile data');
    showStatus('Capturing profile...', 'loading');
    window.isScrapingInProgress = true;
    window.lastScrapedUrl = currentUrl;

    window.extractProfileData()
        .then(handleScrapeSuccess.bind(null, sendResponse))
        .catch(handleScrapeError.bind(null, sendResponse))
        .finally(() => {
            setTimeout(() => {
                window.isScrapingInProgress = false;
            }, 5000);
        });
}

function formatDate(dateString) {
    const date = new Date(dateString);
    return date.toLocaleString();
}

// Add these functions after handleNewScrape

async function handleScrapeSuccess(sendResponse, data) {
    try {
        console.log('✅ Profile data scraped successfully:', data);
        showStatus('Profile captured!', 'success');
        hideStatus(3000);
        
        const profileImage = document.querySelector('[class^="pv-top-card-profile-picture__image"]');
        if (profileImage) {
            console.group('🖼️ Profile Picture Processing');
            console.log('Found profile image element:', profileImage);
            window.applyScrapedStyle(profileImage);
            
            // Upload profile picture if available
            const imageUrl = profileImage.src;
            console.log('Profile image URL:', imageUrl);
            
            if (imageUrl) {
                const contactId = window.location.pathname.split('/')[2];
                console.log('Extracted contact ID:', contactId);
                console.log('Initiating profile picture upload...');
                const uploadResult = await window.uploadProfilePicture(imageUrl, contactId);
                console.log('Upload result:', uploadResult);
            } else {
                console.warn('⚠️ No image URL found in profile picture element');
            }
            console.groupEnd();
        } else {
            console.warn('⚠️ No profile picture element found');
        }
        sendResponse({ success: true, data });
    } catch (error) {
        handleScrapeError(sendResponse, error);
    }
}

function handleScrapeError(sendResponse, error) {
    console.warn('❌ Error scraping profile:', error);
    showStatus('Error capturing profile', 'error');
    hideStatus(3000);
    sendResponse({ success: false, error: error.message });
}

// Setup message container
function setupMessageContainer() {
    const floatingMessageContainer = document.getElementById('floatingMessageContainer');
    if (!floatingMessageContainer) return;

    const closeButton = floatingMessageContainer.querySelector('.goai-close-floating');
    if (closeButton) {
        closeButton.addEventListener('click', () => {
            floatingMessageContainer.style.display = 'none';
        });
    }

    floatingMessageContainer.style.display = 'block';
    hideStatus();

    // Call the new instructions handling setup
    setupInstructionsHandling();

    // Setup show/hide functionality for instructions
    const instructionsContainer = floatingMessageContainer.querySelector('.goai-user-instructions');
    if (instructionsContainer) {
        // Remove any existing event listeners by cloning the container
        const newInstructionsContainer = instructionsContainer.cloneNode(true);
        instructionsContainer.parentNode.replaceChild(newInstructionsContainer, instructionsContainer);

        const showLink = newInstructionsContainer.querySelector('.goai-show-link');
        const hideLink = newInstructionsContainer.querySelector('.goai-hide-link');
        const instructionsText = newInstructionsContainer.querySelector('.instructions-text');
        const editLink = newInstructionsContainer.querySelector('.goai-edit-link');
        const instructionsContent = newInstructionsContainer.querySelector('.instructions-content');

        if (showLink && hideLink && instructionsText && editLink && instructionsContent) {
            showLink.addEventListener('click', function(e) {
                e.preventDefault();
                e.stopPropagation();
                instructionsText.style.display = 'block';
                showLink.style.display = 'none';
                hideLink.style.display = 'inline';
            });

            hideLink.addEventListener('click', function(e) {
                e.preventDefault();
                e.stopPropagation();
                instructionsText.style.display = 'none';
                hideLink.style.display = 'none';
                showLink.style.display = 'inline';
            });

            editLink.addEventListener('click', async function(e) {
                e.preventDefault();
                e.stopPropagation();

                if (editLink.textContent === 'edit') {
                    // Enter edit mode
                    instructionsContent.setAttribute('contenteditable', 'true');
                    instructionsContent.focus();
                    editLink.textContent = 'save';
                } else {
                    // Save mode
                    const newInstructions = instructionsContent.textContent.trim();
                    instructionsContent.setAttribute('contenteditable', 'false');
                    
                    try {
                        const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
                        
                        showStatus('Saving instructions...', 'loading');
                        
                        const response = await fetch(`${API_BASE_URL}/updateInstructions`, {
                            method: 'POST',
                            headers: {
                                'Content-Type': 'application/json',
                                'Accept': 'application/json'
                            },
                            body: JSON.stringify({
                                email,
                                authToken: authentication_token,
                                instructions: newInstructions
                            })
                        });

                        const data = await response.json();
                        
                        if (data.success) {
                            showStatus('Instructions saved!', 'success', 3000);
                            editLink.textContent = 'edit';
                            // Keep instructions hidden and show link visible after saving
                            instructionsText.style.display = 'none';
                            hideLink.style.display = 'none';
                            showLink.style.display = 'inline';
                        } else {
                            throw new Error(data.error || 'Failed to save instructions');
                        }
                    } catch (error) {
                        console.error('Error saving instructions:', error);
                        showStatus('Error saving instructions: ' + error.message, 'error', 5000);
                        // Revert to edit mode on error
                        instructionsContent.setAttribute('contenteditable', 'true');
                        editLink.textContent = 'save';
                    }
                }
            });
        }
    }

    // Setup message send buttons
    floatingMessageContainer.querySelectorAll('.goai-message-send').forEach(sendButton => {
        sendButton.addEventListener('click', async (e) => {
            const messageContainer = e.target.closest('.goai-message');
            const messageText = messageContainer.querySelector('.goai-message-body').textContent;
            
            // 1. Disable the send button
            sendButton.disabled = true;
            sendButton.style.opacity = '0.5';
            sendButton.style.cursor = 'not-allowed';
            
            // 2. Hide reasons and controls
            const reasonsElement = messageContainer.querySelector('.goai-message-reasons');
            const controlsElement = messageContainer.querySelector('.goai-message-controls');
            
            if (reasonsElement) reasonsElement.style.display = 'none';
            if (controlsElement) controlsElement.style.display = 'none';
            
            // 3. Remove all other message containers
            const allMessageContainers = floatingMessageContainer.querySelectorAll('.goai-message');
            allMessageContainers.forEach(container => {
                if (container !== messageContainer) {
                    container.remove();
                }
            });
            
            // 4. Add send status div with icon
            const sendStatusDiv = document.createElement('div');
            sendStatusDiv.className = 'send-status-div sending';
            sendStatusDiv.innerHTML = `
                <span>Sending now! Please wait!</span>
                <svg class="icon" width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                    <path d="M8 12l4-4 4 4H8zm0 2h8v2H8v-2z" fill="currentColor"></path>
                </svg>
            `;
            messageContainer.appendChild(sendStatusDiv);
            
            const result = await window.sendLinkedInMessage(messageText);
            if (result.success) {
                sendStatusDiv.classList.remove('sending');
                sendStatusDiv.classList.add('success');
                sendStatusDiv.innerHTML = `
                    <span>Message Sent!</span>
                    <svg class="icon" width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm-2 15l-5-5 1.41-1.41L10 14.17l7.59-7.59L19 8l-9 9z" fill="currentColor"></path>
                    </svg>
                `;
            } else {
                sendStatusDiv.classList.remove('sending');
                sendStatusDiv.classList.add('error');
                sendStatusDiv.innerHTML = `
                    <span>Failed to send message: ${result.error || 'Unknown error'}</span>
                    <svg class="icon" width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M12 2C6.47 2 2 6.47 2 12s4.47 10 10 10 10-4.47 10-10S17.53 2 12 2zm5 13.59L15.59 17 12 13.41 8.41 17 7 15.59 10.59 12 7 8.41 8.41 7 12 10.59 15.59 7 17 8.41 13.41 12 17 15.59z" fill="currentColor"></path>
                    </svg>
                `;
            }
        });
    });

    // Setup connect buttons
    floatingMessageContainer.querySelectorAll('.goai-message-connect').forEach(connectButton => {
        connectButton.addEventListener('click', async (e) => {
            const messageContainer = e.target.closest('.goai-message');
            const messageText = messageContainer.querySelector('.goai-message-body').textContent;
            
            // 1. Disable the connect button
            connectButton.disabled = true;
            connectButton.style.opacity = '0.5';
            connectButton.style.cursor = 'not-allowed';
            
            // 2. Hide reasons and controls
            const reasonsElement = messageContainer.querySelector('.goai-message-reasons');
            const controlsElement = messageContainer.querySelector('.goai-message-controls');
            
            if (reasonsElement) reasonsElement.style.display = 'none';
            if (controlsElement) controlsElement.style.display = 'none';
            
            // 3. Remove all other message containers
            const allMessageContainers = floatingMessageContainer.querySelectorAll('.goai-message');
            allMessageContainers.forEach(container => {
                if (container !== messageContainer) {
                    container.remove();
                }
            });
            
            // 4. Add send status div with icon
            const sendStatusDiv = document.createElement('div');
            sendStatusDiv.className = 'send-status-div sending';
            sendStatusDiv.innerHTML = `
                <span>Sending now! Please wait!</span>
                <svg class="icon" width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                    <path d="M8 12l4-4 4 4H8zm0 2h8v2H8v-2z" fill="currentColor"></path>
                </svg>
            `;
            messageContainer.appendChild(sendStatusDiv);
            
            const result = await window.sendConnectionRequest(messageText);
            if (result.success) {
                sendStatusDiv.classList.remove('sending');
                sendStatusDiv.classList.add('success');
                sendStatusDiv.innerHTML = `
                    <span>Connection Request Sent!</span>
                    <svg class="icon" width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm-2 15l-5-5 1.41-1.41L10 14.17l7.59-7.59L19 8l-9 9z" fill="currentColor"></path>
                    </svg>
                `;
            } else {
                sendStatusDiv.classList.remove('sending');
                sendStatusDiv.classList.add('error');
                sendStatusDiv.innerHTML = `
                    <span>Failed to send connection request: ${result.error || 'Unknown error'}</span>
                    <svg class="icon" width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M12 2C6.47 2 2 6.47 2 12s4.47 10 10 10 10-4.47 10-10S17.53 2 12 2zm5 13.59L15.59 17 12 13.41 8.41 17 7 15.59 10.59 12 7 8.41 8.41 7 12 10.59 15.59 7 17 8.41 13.41 12 17 15.59z" fill="currentColor"></path>
                    </svg>
                `;
            }
        });
    });

    // Setup message edit feature
    setupMessageEditFeature();

    // Setup modifier buttons with logging
    document.querySelectorAll('.messageModifiers').forEach(container => {
        const label = container.querySelector('.modifier-label');
        const buttons = container.querySelector('.modifier-buttons');
        
        //console.log('Setting up modifier label:', label);
        //console.log('Setting up modifier buttons:', buttons);
        
        // Remove any existing listeners by cloning
        const newLabel = label.cloneNode(true);
        label.parentNode.replaceChild(newLabel, label);
        
        // Click handler for the label
        newLabel.addEventListener('click', (e) => {
            e.preventDefault();
            e.stopPropagation();
            
            console.log('Modifier label clicked');
            console.log('Current display:', buttons.style.display);
            
            // Toggle display
            if (buttons.style.display === 'block') {
                buttons.style.display = 'none';
            } else {
                buttons.style.display = 'block';
                console.log('Set display to block');
            }
            
            console.log('New display:', buttons.style.display);
        });

        // Close buttons when clicking outside
        document.addEventListener('click', (e) => {
            if (!container.contains(e.target)) {
                buttons.style.display = 'none';
            }
        });
    });

    // Update the click handlers for modifier buttons in setupMessageContainer
    document.querySelectorAll('.modifier-btn').forEach(button => {
        button.addEventListener('click', async (e) => {
            // Ignore if button is disabled
            if (button.hasAttribute('disabled')) {
                return;
            }

            e.preventDefault();
            e.stopPropagation();

            // Get the message body from the closest message container
            const messageContainer = button.closest('.goai-message');
            const messageBody = messageContainer.querySelector('.goai-message-body').textContent;

            // Get the revision type by removing emoji and trimming
            const revisionText = button.textContent;
            const revision = revisionText.replace(/[\u{1F300}-\u{1F9FF}]/gu, '').trim();

            // Close the floating container immediately after clicking
            const floatingMessageContainer = document.getElementById('floatingMessageContainer');
            if (floatingMessageContainer) {
                floatingMessageContainer.style.display = 'none';
            }

            try {
                await reviseMessage(messageBody, revision);
            } catch (error) {
                console.error('Error revising message:', error);
                showStatus('Failed to revise message', 'error', 5000);
            }
        });
    });

    // Add drag functionality
    setupDragFunctionality();
}

function setupMessageEditFeature() {
    const floatingMessageContainer = document.getElementById('floatingMessageContainer');
    
    floatingMessageContainer.querySelectorAll('.goai-message-body').forEach(messageDiv => {
        // Remove previous click event listeners
        const newMessageDiv = messageDiv.cloneNode(true);
        messageDiv.parentNode.replaceChild(newMessageDiv, messageDiv);
        
        // Add new click event listener
        newMessageDiv.addEventListener('click', (e) => {
            // Check if already editable
            if (e.target.getAttribute('contenteditable') === 'true') {
                return;
            }

            // Make the div content editable
            e.target.setAttribute('contenteditable', 'true');
            e.target.style.borderColor = '#0a66c2';
            e.target.focus();

            // Add blur event listener
            e.target.addEventListener('blur', () => {
                e.target.removeAttribute('contenteditable');
                e.target.style.borderColor = '#e0e0e0';
            }, { once: true });

            // Prevent immediate blur when clicking
            e.stopPropagation();
        });
    });
}

function setupInstructionsHandling() {
    const instructionsContainer = document.querySelector('.goai-user-instructions');
    if (!instructionsContainer) return;

    const elements = {
        showButton: instructionsContainer.querySelector('.goai-show-link'),
        hideButton: instructionsContainer.querySelector('.goai-hide-link'),
        textDisplay: instructionsContainer.querySelector('.instructions-text'),
        textContent: instructionsContainer.querySelector('.instructions-content'),
        editButton: instructionsContainer.querySelector('.goai-edit-link'),
        editContainer: instructionsContainer.querySelector('.instructions-edit-container'),
        textarea: instructionsContainer.querySelector('.instructions-edit-area'),
        saveButton: instructionsContainer.querySelector('.goai-save-instructions'),
        cancelButton: instructionsContainer.querySelector('.goai-cancel-instructions'),
        statusDiv: instructionsContainer.querySelector('.instructions-status')
    };

    setupInstructionsEventListeners(elements);
    loadInitialInstructions(elements);
}

function setupInstructionsEventListeners(elements) {
    const { showButton, hideButton, textDisplay, editButton, textContent, statusDiv } = elements;

    showButton.addEventListener('click', (e) => {
        e.preventDefault();
        textDisplay.style.display = 'block';
        showButton.style.display = 'none';
        hideButton.style.display = 'inline';
    });

    hideButton.addEventListener('click', (e) => {
        e.preventDefault();
        textDisplay.style.display = 'none';
        hideButton.style.display = 'none';
        showButton.style.display = 'inline';
    });

    editButton.addEventListener('click', async (e) => {
        e.preventDefault();
        if (editButton.textContent === 'edit') {
            // Enter edit mode
            textDisplay.style.border = '2px solid #0a66c2';
            textContent.setAttribute('contenteditable', 'true');
            textContent.focus();
            editButton.textContent = 'save';
            // Hide instructions and update show/hide links
            textDisplay.style.display = 'none';
            hideButton.style.display = 'none';
            showButton.style.display = 'inline';
        } else {
            // Save mode
            const newInstructions = textContent.textContent.trim();
            textContent.setAttribute('contenteditable', 'false');
            textDisplay.style.border = 'none';

            try {
                const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
                
                showStatus('Saving instructions...', 'loading');
                
                const response = await fetch(`${API_BASE_URL}/updateInstructions`, {
                    method: 'POST',
                    headers: {
                        'Content-Type': 'application/json',
                        'Accept': 'application/json'
                    },
                    body: JSON.stringify({
                        email,
                        authToken: authentication_token,
                        instructions: newInstructions
                    })
                });

                const data = await response.json();
                
                if (data.success) {
                    showStatus('Instructions saved!', 'success', 3000);
                    editButton.textContent = 'edit';
                    // Keep instructions hidden and show link visible after saving
                    textDisplay.style.display = 'none';
                    hideButton.style.display = 'none';
                    showButton.style.display = 'inline';
                } else {
                    throw new Error(data.error || 'Failed to save instructions');
                }
            } catch (error) {
                console.error('Error saving instructions:', error);
                showStatus('Error saving instructions: ' + error.message, 'error', 5000);
                // Revert to edit mode on error
                textContent.setAttribute('contenteditable', 'true');
                textDisplay.style.border = '2px solid #0a66c2';
                editButton.textContent = 'save';
            }
        }
    });
}

async function handleInstructionsSave(elements) {
    const { textarea, textContent, editContainer, hideButton, 
            showButton, textDisplay, statusDiv } = elements;
    
    const newInstructions = textarea.value.trim();
    statusDiv.textContent = 'Saving...';
    
    try {
        const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
        
        const response = await fetch(`${API_BASE_URL}/updateInstructions`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Accept': 'application/json',
                'Origin': chrome.runtime.getURL('')
            },
            body: JSON.stringify({
                email,
                authToken: authentication_token,
                instructions: newInstructions
            })
        });

        const data = await response.json();
        if (data.success) {
            textContent.textContent = newInstructions;
            editContainer.style.display = 'none';
            statusDiv.textContent = 'Instructions saved successfully!';
            
            setTimeout(() => {
                textDisplay.style.display = 'none';
                hideButton.style.display = 'none';
                showButton.style.display = 'inline';
                statusDiv.textContent = '';
            }, 500);
        } else {
            throw new Error(data.error || 'Failed to save instructions');
        }
    } catch (error) {
        console.error('Error saving instructions:', error);
        statusDiv.textContent = `Error: ${error.message}`;
    }
}

async function loadInitialInstructions(elements) {
    const { textContent, textarea } = elements;
    
    try {
        const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
        
        const response = await fetch(
            `${API_BASE_URL}/getInstructions?email=${encodeURIComponent(email)}&authToken=${authentication_token}`,
            {
                headers: {
                    'Accept': 'application/json',
                    'Origin': chrome.runtime.getURL('')
                }
            }
        );
        
        const data = await response.json();
        if (data.success) {
            textContent.textContent = data.instructions;
            textarea.value = data.instructions;
        }
    } catch (error) {
        console.error('Error fetching instructions:', error);
        showStatus('Error loading instructions', 'error');
    }
}

// Generate messages for connected profiles
async function generateMessages(profileData) {
    showStatus('Generating messages...', 'loading');
    
    let timer = null; // Define timer at the start of the function
    
    chrome.runtime.sendMessage({
        action: 'sendAnalyticsEvent',
        eventName: 'generate_messages_started',
        eventParams: {}
    });
    
    try {
        const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
        const contactId = extractContactId();
        await checkCredits(email, authentication_token);
        
        let seconds = 0;
        timer = setInterval(() => {
            seconds++;
            const statusIndicator = document.querySelector('.goai-status-indicator');
            if (statusIndicator?.classList.contains('visible')) {
                statusIndicator.textContent = `Generating messages... ${seconds}s`;
            }
        }, 1000);

        const response = await fetch(`${API_BASE_URL}/generateMessages`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Accept': 'application/json'
            },
            body: JSON.stringify({
                email,
                authToken: authentication_token,
                contactId,
                aiPersona: 'sales',
                messageType: profileData.messageType || 'direct-message'
            })
        });

        if (!response.ok) {
            throw new Error(`Server error: ${response.status}`);
        }

        const data = await response.json();
        
        // Clear timer before displaying messages
        if (timer) {
            clearInterval(timer);
            timer = null;
        }
        
        // Display messages
        await displayGeneratedMessages(data);
        
        return data;
    } catch (error) {
        if (timer) {
            clearInterval(timer);
            timer = null;
        }
        console.error('Generate messages error:', error);
        showStatus('Failed to generate messages: ' + error.message, 'error', 5000);
        throw error;
    }
}

// Generate connection request messages
async function generateConnectMessages(profileData) {
    showStatus('Generating connect messages...', 'loading');
    
    chrome.runtime.sendMessage({
        action: 'sendAnalyticsEvent',
        eventName: 'generate_connect_messages_started',
        eventParams: {}
    });
    
    let timer; // Declare timer in outer scope so we can clear it
    
    try {
        const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
        const contactId = extractContactId();
        await checkCredits(email, authentication_token);

        let seconds = 0;
        timer = setInterval(() => {
            seconds++;
            const statusIndicator = document.querySelector('.goai-status-indicator');
            if (statusIndicator?.classList.contains('visible')) {
                statusIndicator.textContent = `Generating connect messages... ${seconds}s`;
            }
        }, 1000);

        const response = await fetch(`${API_BASE_URL}/generateMessages`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Accept': 'application/json'
            },
            body: JSON.stringify({
                email,
                authToken: authentication_token,
                contactId,
                aiPersona: 'sales',
                messageType: profileData.messageType || 'connection-message'
            })
        });

        if (!response.ok) {
            throw new Error(`Server error: ${response.status}`);
        }

        const data = await response.json();
        
        // Display messages and ensure timer is cleared after container is visible
        await displayGeneratedConnectMessages(data);
        if (timer) {
            clearInterval(timer);
            timer = null;
        }
        
        return data;
    } catch (error) {
        if (timer) {
            clearInterval(timer);
            timer = null;
        }
        console.error('Generate connect messages error:', error);
        showStatus('Failed to generate connect messages: ' + error.message, 'error', 5000);
        throw error;
    }
}

// Send LinkedIn message
async function sendLinkedInMessage(message) {
    try {
        chrome.runtime.sendMessage({
            action: 'sendAnalyticsEvent',
            eventName: 'send_linkedin_message_started',
            eventParams: {}
        });

        const topSection = document.querySelector(PROFILE_SELECTORS.topSectionNew);
        if (!topSection) {
            throw new Error('Main top profile section not found');
        }

        const messageButton = await findElement(() => 
            Array.from(topSection.querySelectorAll('span'))
                .find(span => span.innerText === "Message")
        );

        if (!messageButton) {
            throw new Error('Message button not found');
        }

        console.log('Waiting 5 seconds before clicking message button...');
        await new Promise(resolve => setTimeout(resolve, 5000));

        messageButton.click();

        const msgFormContentEditable = await findElement(() => 
            document.querySelector('.msg-form__contenteditable')
        );
        msgFormContentEditable.innerHTML = `<p>${message}</p>`;
        msgFormContentEditable.dispatchEvent(new Event('input', { bubbles: true }));

        await new Promise(resolve => setTimeout(resolve, 1000));

        const sendButton = await findElement(() => 
            document.querySelector('.msg-form__send-button')
        );
        sendButton.removeAttribute('disabled');
        sendButton.click();

        await new Promise(resolve => setTimeout(resolve, 5000));

        const dismissButton = await findElement(() => 
            Array.from(document.querySelectorAll('button'))
                .find(button => button.querySelector('use[href="#close-small"]'))
        );
            dismissButton.click();

        dispatchMessageEvent(true);

        // After successful send
        await logMessageToAPI(
            window.location.href,
            'direct-message',
            message
        );

        return { success: true };
    } catch (error) {
        console.warn('Error sending LinkedIn message:', error);
        dispatchMessageEvent(false, error.message);
        return { success: false, error: error.message };
    }
}

// Send connection request
async function sendConnectionRequest(message) {
    try {
        chrome.runtime.sendMessage({
            action: 'sendAnalyticsEvent',
            eventName: 'send_connection_request_started',
            eventParams: {}
        });
        
        if (await isPendingConnection()) {
            throw new Error('Connection request is already pending');
        }

        await handleConnectButton();
        await addNoteAndSendRequest(message);

        // Log the sent connection message
        await logMessageToAPI(
            window.location.href,
            'connection-message',
            message
        );

        dispatchConnectionEvent(true);
        return { success: true };
    } catch (error) {
        console.warn('Error sending connection request:', error);
        dispatchConnectionEvent(false, error.message);
        return { success: false, error: error.message };
    }
}

// Helper functions
async function checkCredits(email, authToken) {
    const creditsResponse = await fetch(
        `${API_BASE_URL}/getCredits?email=${encodeURIComponent(email)}&authToken=${authToken}`,
        {
            method: 'GET',
            headers: {
                'Content-Type': 'application/json',
                'Accept': 'application/json'
            }
        }
    );
    
    if (!creditsResponse.ok) {
        throw new Error('Failed to fetch credits');
    }
    
    const { credits = 0 } = await creditsResponse.json();
    if (credits <= 0) {
        throw new Error('Out of credits. Please add more credits to continue.');
    }
}

function extractContactId() {
    const url = new URL(window.location.href);
    const pathParts = url.pathname.split('/');
    const contactId = pathParts[2];
    
    if (!contactId) {
        throw new Error('Could not extract contact ID from URL');
    }
    
    return contactId;
}

async function findElement(selector, timeout = 5000, description = '') {
    const startTime = Date.now();
    while (Date.now() - startTime < timeout) {
        const element = selector();
        if (element) return element;
        await new Promise(resolve => setTimeout(resolve, 100));
    }
    throw new Error(`Element not found: ${description} - ${selector.toString()}`);
}

async function isPendingConnection() {
    const pendingButton = Array.from(document.querySelectorAll('.artdeco-button__text'))
        .find(span => span.textContent.trim() === 'Pending');
    return !!pendingButton;
}

async function handleConnectButton() {
    debugLog('Starting handleConnectButton');
    
    // First get the main section - get the first element since querySelectorAll returns a NodeList
    const topSection = document.querySelector(PROFILE_SELECTORS.topSectionChild).parentElement;
    debugLog('topSection selector:', PROFILE_SELECTORS.topSectionChild);
    debugLog('topSection section:', topSection);
    
    if (!topSection) {
        debugLog('❌ Main section not found - skipping automation');
        return null;
    }

    debugLog('✅ Found main section');

    // Look for direct connect button within main section
    const directConnectButton = await findElement(
        () => Array.from(topSection.querySelectorAll('span.artdeco-button__text'))
            .find(span => span.textContent.trim() === 'Connect')
    ).catch(() => null);

    if (directConnectButton) {
        debugLog('✅ Direct connect button found, clicking it');
        directConnectButton.click();
    } else {
        debugLog('Direct connect button not found, trying more actions flow');
        await handleMoreActionsFlow(topSection);
    }
}

async function handleMoreActionsFlow(topSection) {
    debugLog('Starting handleMoreActionsFlow');
    
    const moreButton = await findElement(
        () => {
            const button = topSection.querySelector('button.artdeco-dropdown__trigger');
            return button?.querySelector('span')?.textContent.trim() === 'More' ? button : null;
        }
    );

    if (!moreButton) {
        debugLog('❌ More button not found');
        return null;
    }

    debugLog('✅ More button found, clicking it');
    moreButton.click();

    // Wait a moment for dropdown to appear
    await new Promise(resolve => setTimeout(resolve, 500));

    const connectButton = await findElement(
        () => Array.from(topSection.querySelectorAll('div[role="button"]'))
            .find(el => el.querySelector('span.display-flex.t-normal.flex-1')?.textContent.trim() === 'Connect')
    );

    if (!connectButton) {
        debugLog('❌ Connect button not found in dropdown');
        return null;
    }

    debugLog('✅ Connect button found in dropdown, clicking it');
    connectButton.click();
}

async function addNoteAndSendRequest(message) {
    const addNoteButton = await findElement(
        () => Array.from(document.querySelectorAll('button[aria-label="Add a note"]'))
            .find(el => el.querySelector('span')?.textContent.trim() === 'Add a note')
    );
    addNoteButton.click();

    const messageTextarea = await findElement(
        () => document.querySelector('textarea[name="message"]')
    );
    messageTextarea.value = message;
    messageTextarea.dispatchEvent(new Event('input', { bubbles: true }));

    await new Promise(resolve => setTimeout(resolve, 4000));

    const sendButton = await findElement(
        () => Array.from(document.querySelectorAll('button[aria-label="Send invitation"]'))
            .find(button => button.querySelector('span')?.textContent.trim() === 'Send')
    );

    // Enable the send button before clicking
    debugLog('Removing disabled attribute from send button...');
    sendButton.removeAttribute('disabled');
    
    debugLog('Clicking send button...');
    sendButton.click();
}

function dispatchMessageEvent(success, error = null) {
    document.dispatchEvent(new CustomEvent('LINKEDIN_GOAI_MESSAGE_SENT', {
        detail: { success, error }
    }));
    
    chrome.runtime.sendMessage({
        action: 'sendAnalyticsEvent',
        eventName: 'send_linkedin_message_completed',
        eventParams: { success, error }
    });
}

function dispatchConnectionEvent(success, error = null) {
    document.dispatchEvent(new CustomEvent('LINKEDIN_GOAI_CONNECTION_SENT', {
        detail: { success, error }
    }));
    
    chrome.runtime.sendMessage({
        type: 'LINKEDIN_GOAI_CONNECTION_OUTCOME',
        success,
        error
    });
    
    chrome.runtime.sendMessage({
        action: 'sendAnalyticsEvent',
        eventName: 'send_connection_request_completed',
        eventParams: { success, error }
    });
}

// Display generated messages
async function displayGeneratedMessages(response, isModifierRevision = false) {
    if (!response || (!response.messages && !response.data)) {
        throw new Error('Invalid response from message generation');
    }

    try {
        // Keep showing the loading status unless it's a modifier revision
        if (!isModifierRevision) {
            showStatus('Preparing messages...', 'loading');
        }

        // Fetch instructions from API
        const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
        const instructionsResponse = await fetch(
            `${API_BASE_URL}/getInstructions?email=${encodeURIComponent(email)}&authToken=${authentication_token}`,
            {
                headers: {
                    'Accept': 'application/json',
                    'Origin': chrome.runtime.getURL('')
                }
            }
        );
        
        const instructionsData = await instructionsResponse.json();
        const userInstructions = instructionsData.success ? instructionsData.instructions : 'No instructions found';
        
        const messages = response.messages || response.data;
        
        const floatingMessageContainer = document.getElementById('floatingMessageContainer');
        
        // Reset container state
        floatingMessageContainer.style.display = 'none';
        floatingMessageContainer.innerHTML = '';
        
        // Force a reflow
        void floatingMessageContainer.offsetHeight;
        
        floatingMessageContainer.innerHTML = `
            <button class="goai-close-floating" aria-label="Close messages"></button>
            ${userInstructionsTemplate(userInstructions)}
            <div class="goai-messages-div">
                ${messages.map(({ messageBody, reasons }) => `
                    <div class="goai-message">
                        <div class="goai-message-body">${messageBody}</div>
                        <div class="goai-message-reasons">
                            ${reasons.map(reason => 
                                `<div class="goai-message-reason">${reason}</div>`
                            ).join('')}
                        </div>
                        <div class="goai-message-controls">
                            <div class="messageModifiers">
                                <button class="modifier-label">Change it!</button>
                                <div class="modifier-buttons" style="display: none;">
                                    <button class="modifier-btn shorter">✂️ Shorter</button>
                                    <button class="modifier-btn funnier">😄 Funnier</button>
                                    <button class="modifier-btn less-direct">🎭 Less Direct</button>
                                    <button class="modifier-btn more-direct">🎯 More Direct</button>
                                    <button class="modifier-btn professional">🤝 Professional</button>
                                    <button class="modifier-btn creative">🎨 Creative</button>
                                    <button class="modifier-btn enthusiastic">🌟 Enthusiastic</button>
                                    <button class="modifier-btn casual">🗣 Casual</button>
                                    <button class="modifier-btn rephrase">🔄 Rephrase</button>
                                    <button class="modifier-btn technical">🤓 Technical</button>
                                    <button class="modifier-btn playful">🎪 Playful</button>
                                    <button class="modifier-btn goal-oriented">🎯 Goal-oriented</button>
                                    <button class="modifier-btn data-driven">📊 Data-driven</button>
                                    <button class="modifier-btn custom" disabled>✨ Custom</button>
                                </div>
                            </div>
                            <button class="goai-message-send">Send Message</button>
                        </div>
                    </div>
                `).join('')}
            </div>
        `;

        setupMessageContainer();
        setupMessageHoverEffects();
        
        // Show the container
        floatingMessageContainer.style.display = 'block';
        
        // Wait for the container to be fully rendered
        await new Promise(resolve => requestAnimationFrame(resolve));
        
        // Additional wait to ensure CSS transitions complete
        await new Promise(resolve => setTimeout(resolve, 50));
        
        // Only hide the status after confirming the container is visible
        const isContainerVisible = floatingMessageContainer.offsetHeight > 0;
        if (isContainerVisible) {
            setTimeout(() => {
                hideStatus();
            }, 250);
        } else {
            console.warn('Container not visible after display attempt');
            // Try to recover by forcing display
            floatingMessageContainer.style.display = 'block';
            floatingMessageContainer.style.opacity = '1';
            setTimeout(() => {
                hideStatus();
            }, 1000);
        }

    } catch (error) {
        console.error('Error displaying messages:', error);
        showStatus('Error displaying messages', 'error');
    }
}

// Display generated connect messages
async function displayGeneratedConnectMessages(response) {
    if (!response || !response.messages) {
        throw new Error('Invalid response from connection message generation');
    }

    showStatus('Connection messages generated!', 'success');

    try {
        // Fetch instructions from API
        const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
        const instructionsResponse = await fetch(
            `${API_BASE_URL}/getInstructions?email=${encodeURIComponent(email)}&authToken=${authentication_token}`,
            {
                headers: {
                    'Accept': 'application/json',
                    'Origin': chrome.runtime.getURL('')
                }
            }
        );
        
        const instructionsData = await instructionsResponse.json();
        const userInstructions = instructionsData.success ? instructionsData.instructions : 'No instructions found';
        
        const floatingMessageContainer = document.getElementById('floatingMessageContainer');
        floatingMessageContainer.innerHTML = `
            <button class="goai-close-floating" aria-label="Close messages"></button>
            ${userInstructionsTemplate(userInstructions)}
            <div class="goai-messages-div">
                ${response.messages.map(({ messageBody, reasons }) => `
                    <div class="goai-message">
                        <div class="goai-message-body">${messageBody}</div>
                        <div class="goai-message-reasons">
                            ${reasons.map(reason => 
                                `<div class="goai-message-reason">${reason}</div>`
                            ).join('')}
                        </div>
                        <div class="goai-message-controls">
                            <div class="messageModifiers">
                                <button class="modifier-label">Change it!</button>
                                <div class="modifier-buttons" style="display: none;">
                                    <button class="modifier-btn shorter">✂️ Shorter</button>
                                    <button class="modifier-btn funnier">😄 Funnier</button>
                                    <button class="modifier-btn less-direct">🎭 Less Direct</button>
                                    <button class="modifier-btn more-direct">🎯 More Direct</button>
                                    <button class="modifier-btn professional">🤝 Professional</button>
                                    <button class="modifier-btn creative">🎨 Creative</button>
                                    <button class="modifier-btn enthusiastic">🌟 Enthusiastic</button>
                                    <button class="modifier-btn casual">🗣️ Casual</button>
                                    <button class="modifier-btn rephrase">🔄 Rephrase</button>
                                    <button class="modifier-btn technical">🤓 Technical</button>
                                    <button class="modifier-btn playful">🎪 Playful</button>
                                    <button class="modifier-btn goal-oriented">🎯 Goal-oriented</button>
                                    <button class="modifier-btn data-driven">📊 Data-driven</button>
                                    <button class="modifier-btn custom" disabled>✨ Custom</button>
                                </div>
                            </div>
                            <button class="goai-message-connect">Send Connection Request</button>
                        </div>
                    </div>
                `).join('')}
            </div>
        `;

        setupMessageContainer();
        setupMessageHoverEffects();
        
        // Show the container
        floatingMessageContainer.style.display = 'block';
        
        // Wait for the container to be fully rendered
        await new Promise(resolve => requestAnimationFrame(resolve));
        
        // Only hide the status after confirming the container is visible
        const isContainerVisible = floatingMessageContainer.offsetHeight > 0;
        if (isContainerVisible) {
            setTimeout(() => {
                hideStatus();
            }, 250);
        } else {
            console.warn('Container not visible after display attempt');
            setTimeout(() => {
                hideStatus();
            }, 1000);
        }

    } catch (error) {
        console.error('Error fetching instructions:', error);
        showStatus('Error loading instructions', 'error');
    }
}

// Template for user instructions
function userInstructionsTemplate(userInstructions) {
    return `
        <div class="goai-user-instructions">
            <span class="instructions-label">AI Instructions</span>
            <a href="#" class="goai-show-link">show</a>
            <a href="#" class="goai-hide-link" style="display: none;">hide</a>
            <div class="instructions-text" style="display: none;">
                <span class="instructions-content">${userInstructions}</span>
                <div class="instructions-actions">
                    <a href="#" class="goai-edit-link">edit</a>
                </div>
            </div>
            <div class="instructions-edit-container" style="display: none;">
                <textarea class="instructions-edit-area">${userInstructions}</textarea>
                <div class="instructions-edit-actions">
                    <button class="goai-save-instructions">Save</button>
                    <button class="goai-cancel-instructions">Cancel</button>
                </div>
                <div class="instructions-status"></div>
            </div>
        </div>
    `;
}

// Click the first connect button found
async function clickFirstConnectButton() {
    debugLog('Starting clickFirstConnectButton');
    
    try {
        // Get message text from the first message in container
        debugLog('Getting message text from container...');
        const messageContainer = document.querySelector('.goai-message');
        const messageText = messageContainer?.querySelector('.goai-message-body')?.textContent;

        if (!messageText) {
            throw new Error('Message text not found');
        }

        // Use existing functions to handle the connection flow
        if (await isPendingConnection()) {
            throw new Error('Connection request is already pending');
        }

        await handleConnectButton();
        await addNoteAndSendRequest(messageText);

        // Log the message to the API
        debugLog('Logging message to API...');
        await logMessageToAPI(
            window.location.href,
            'connection-message',
            messageText
        );

        dispatchConnectionEvent(true);
        debugLog('✅ Successfully completed clickFirstConnectButton');
        return true;

    } catch (error) {
        debugLog('❌ Error in clickFirstConnectButton:', error);
        debugLog('Error stack:', error.stack);
        
        // Dispatch error events
        dispatchConnectionEvent(false, error.message);
        return false;
    }
}

// Click the first send button found
async function clickFirstSendButton() {
    try {
        // Get message text from the first message in container
        const messageContainer = document.querySelector('.goai-message');
        const messageText = messageContainer?.querySelector('.goai-message-body')?.textContent;

        if (!messageText) {
            throw new Error('Message text not found');
        }

        // Use existing function to handle sending message
        const result = await sendLinkedInMessage(messageText);

        if (!result.success) {
            throw new Error(result.error || 'Failed to send message');
        }

        // Log the message to the API
        debugLog('Logging message to API...');
        await logMessageToAPI(
            window.location.href,
            'direct-message',
            messageText
        );

        return true;
    } catch (error) {
        debugLog('❌ Error in clickFirstSendButton:', error);
        
        // Dispatch error events
        dispatchMessageEvent(false, error.message);
        return false;
    }
}

async function logMessageToAPI(profileUrl, messageType, messageContent) {
    try {
        const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
        
        const response = await fetch(`${API_BASE_URL}/logMessageSent`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Accept': 'application/json'
            },
            body: JSON.stringify({
                email,
                authToken: authentication_token,
                profileUrl,
                messageType,
                messageContent
            })
        });

        if (!response.ok) {
            console.warn('Failed to log message to API:', response.status);
            return false;
        }
        
        console.log('✅ Successfully logged message to API');
        return true;
    } catch (error) {
        console.error('Error logging message to API:', error);
        return false;
    }
}

// Make it available globally
window.logMessageToAPI = logMessageToAPI;

// Export all necessary functions
export {
    setupMessageHandlers,
    setupMessageContainer,
    setupEventListeners,
    formatDate,
    generateMessages,
    generateConnectMessages,
    sendLinkedInMessage,
    sendConnectionRequest,
    checkCredits,
    extractContactId,
    findElement,
    displayGeneratedMessages,
    displayGeneratedConnectMessages,
    setupMessageEditFeature,
    clickFirstConnectButton,
    clickFirstSendButton,
    isPendingConnection,
    handleConnectButton,
    addNoteAndSendRequest,
    dispatchMessageEvent,
    dispatchConnectionEvent,
    logMessageToAPI,
    reviseMessage
}; 

function setupMessageHoverEffects() {
    console.log('Message hover effects initialized');
} 

// Add this new function to handle message revisions
async function reviseMessage(currentMessage, revision) {
    showStatus('Revising message...', 'loading');
    
    let timer; // Declare timer in outer scope
    
    try {
        const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
        const contactId = extractContactId();
        
        let seconds = 0;
        timer = setInterval(() => {
            seconds++;
            const statusIndicator = document.querySelector('.goai-status-indicator');
            if (statusIndicator?.classList.contains('visible')) {
                statusIndicator.textContent = `Revising message... ${seconds}s`;
            }
        }, 1000);

        const response = await fetch(`${API_BASE_URL}/reviseMessage`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Accept': 'application/json'
            },
            body: JSON.stringify({
                email,
                authToken: authentication_token,
                contactId,
                currentMessage,
                revision
            })
        });

        if (!response.ok) {
            throw new Error(`Server error: ${response.status}`);
        }

        const data = await response.json();
        
        if (data.messages) {
            await displayGeneratedMessages(data, true);
            if (timer) {
                clearInterval(timer);
                timer = null;
            }
        } else {
            throw new Error('No messages received from revision');
        }

        return data;
    } catch (error) {
        if (timer) {
            clearInterval(timer);
            timer = null;
        }
        console.error('Revise message error:', error);
        showStatus('Failed to revise message: ' + error.message, 'error', 5000);
        throw error;
    }
} 

// Add this function to setupMessageContainer
function setupDragFunctionality() {
    const container = document.getElementById('floatingMessageContainer');
    if (!container) return;

    let isDragging = false;
    let currentX;
    let currentY;
    let initialX;
    let initialY;
    let xOffset = 0;
    let yOffset = 0;

    // Remove the separate drag handle creation and instead use the container itself
    container.addEventListener('mousedown', dragStart);
    document.addEventListener('mousemove', drag);
    document.addEventListener('mouseup', dragEnd);

    function dragStart(e) {
        // Only start dragging if clicking the container itself, not its children
        if (e.target !== container) {
            return;
        }

        initialX = e.clientX - xOffset;
        initialY = e.clientY - yOffset;
        
        isDragging = true;
        container.classList.add('dragging');
    }

    function drag(e) {
        if (!isDragging) return;
        
        e.preventDefault();
        
        currentX = e.clientX - initialX;
        currentY = e.clientY - initialY;

        xOffset = currentX;
        yOffset = currentY;

        // Ensure the container stays within viewport bounds
        const rect = container.getBoundingClientRect();
        const viewportWidth = window.innerWidth;
        const viewportHeight = window.innerHeight;

        // Constrain horizontal movement
        if (currentX < 0) currentX = 0;
        if (currentX + rect.width > viewportWidth) {
            currentX = viewportWidth - rect.width;
        }

        // Constrain vertical movement
        if (currentY < 0) currentY = 0;
        if (currentY + rect.height > viewportHeight) {
            currentY = viewportHeight - rect.height;
        }

        setTranslate(currentX, currentY, container);
    }

    function dragEnd(e) {
        if (!isDragging) return;
        
        initialX = currentX;
        initialY = currentY;

        isDragging = false;
        container.classList.remove('dragging');
    }

    function setTranslate(xPos, yPos, el) {
        el.style.transform = `translate(${xPos}px, ${yPos}px)`;
    }
} 
