import { API_BASE_URL } from './constants.js';
import { showStatus, hideStatus } from './uiHandler.js';

// Define global selectors
export const PROFILE_SELECTORS = {
    // Basic profile selectors
    mainSection: 'main.scaffold-layout__main',
    topSectionChild: 'div.top-card-background-hero-image',
    topSectionNew: 'section[data-member-id]',
    name: 'h1.break-words',
    headline: 'div.text-body-medium',
    location: 'span.text-body-small.inline',
    connections: '.dist-value[aria-hidden="true"]',
    followers: '.pv-top-card--list span.t-bold',
    degree: '.t-normal span',    
    pendingConnection: {
        container: 'span.artdeco-button__text',
        text: 'Pending'
    },
    
    // About section (updated)
    about: {
        sections: 'section.artdeco-card.pv-profile-card.break-words',
        headerSpan: 'span[aria-hidden="true"]',
        content: '.display-flex.ph5.pv3',
        text: 'span'
    },
    
    // Experience section
    experience: {
        section: 'section.artdeco-card > div#experience',
        listItem: 'li.artdeco-list__item',
        company: 'span.t-14.t-normal span[aria-hidden="true"]',
        jobTitle: 'div.display-flex span',
        duration: {
            element: 'span.t-14.t-normal.t-black--light',
            text: 'span[aria-hidden="true"]'
        },
        location: {
            element: 'span.t-14.t-normal.t-black--light',
            text: 'span[aria-hidden="true"]'
        },
        multiPosition: '.pvs-list__item-container',
        container: 'section.artdeco-card > div#experience',
        positionCounter: 'div.display-flex.align-items-center.mr1.hoverable-link-text.t-bold',
        multiPositionTitle: 'div.display-flex.align-items-center.mr1.hoverable-link-text.t-bold',
        multiPositionStatus: 'span.t-14.t-normal span[aria-hidden="true"]',
        multiPositionCompanyNameAndTitle: 'div.t-bold span[aria-hidden="true"]', 
        positionCountLink: '#navigation-index-see-all-positions-aggregated'
    },
    
    // Education section (updated)
    education: {
        section: '#education',
        eduItem: 'ul > li:not(li li)',
        school: 'div.display-flex.full-width span[aria-hidden="true"]',
        duration: 'span.t-14.t-normal.t-black--light span[aria-hidden="true"]',
        fieldOfStudy: 'span.t-14.t-normal span[aria-hidden="true"]',
        additionalInfo: 'div.pvs-entity__sub-components'
    },
    
    // Company and School quick view
    quickView: {
        container: '.ph5.pb5',
        items: 'li'
    },
    
    // Profile picture
    profilePicture: {
        primary: 'img[width="200"][height="200"][src*="profile-displayphoto"]',
        fallback: '[class^="pv-top-card-profile-picture__image"]'
    },
    
    // Skills section
    skills: {
        section: '#skills',
        listItem: '.pvs-list__item-container',
        title: '.t-bold span'
    },
    
    // Recommendations section
    recommendations: {
        section: '#recommendations',
        listItem: '.pvs-list__item-container',
        author: '.t-bold span',
        text: '.t-normal span'
    },
    
    // Activity section
    activities: {
        section: '#content_collections',
        listItem: 'li.profile-creator-shared-feed-update__mini-container',
        whoPostedAndWhen: 'a span[aria-hidden="true"]',
        postContent: 'a.flex-1',
        postSocialReactions: 'span[aria-hidden="true"].social-details-social-counts__reactions-count',
        postComments: 'li.social-details-social-counts__comments',
        postContext: 'div[class^="feed-mini-update-content"]'
    },
    
    // Interests section
    interests: {
        section: '#interests',
        listItem: '.pvs-list__item-container',
        title: '.t-bold span'
    },



    // Direct Messages section
    directMessages: {
        mainElement: 'div.msg-convo-wrapper.msg-overlay-conversation-bubble[role="dialog"][aria-label="Messaging"][data-view-name="message-overlay-conversation-bubble-item"]',
        messageElement: 'li.msg-s-message-list__event.clearfix'

    }
};

// Setup profile page observer
function setupProfilePageReadyObserver() {
    // Initialize default scraping settings
    window.ScrapingSettings = {
        deepActivityScraping: false,
        autoScrape: false,
    };

    // Get stored settings and update defaults
    chrome.storage.local.get(['selectors', 'scrapingSettings'], function(result) {
        const scrapingSettings = result.scrapingSettings;
        
        const isAutoScrapeEnabled = scrapingSettings?.autoScrape !== undefined ? 
            scrapingSettings.autoScrape : true;  // Default to true if not set

        console.log('🔄 isAutoScrapeEnabled:', isAutoScrapeEnabled);

        // Check immediately in case the element already exists
        const experienceSection = document.querySelector(PROFILE_SELECTORS.experience.container);
        if (experienceSection) {
            console.log('✅ Experience section found immediately');
            
            // Add generate button
            addGenerateButton();
            
            console.log('🔄 Updating dropdown items');
            updateDropdownItems();
            
            if (isAutoScrapeEnabled) {
                console.log('🔄 Auto-Scraping profile data');
                handleAutoScrape();
            }
            return;
        }

        // If not found immediately, start observing
        const observer = new MutationObserver((mutations, obs) => {
            const experienceSection = document.querySelector(PROFILE_SELECTORS.experience.container);
            if (experienceSection) {
                console.log('✅ Experience section found by observer');
                
                // Add generate button
                addGenerateButton();
                
                console.log('🔄 Updating dropdown items');
                updateDropdownItems();
                
                if (isAutoScrapeEnabled) {
                    console.log('🔄 Auto-Scraping profile data');
                    handleAutoScrape();
                }
                obs.disconnect();
            }
        });

        observer.observe(document.body, {
            childList: true,
            subtree: true
        });
    });
}

// Utility function to get connection status
function getConnectionStatus() {
    const connectionDegree = document.querySelector('.dist-value[aria-hidden="true"]')?.innerText || null;
    return connectionDegree === '1st';
}

// Utility function to format contact data
function formatContactData(profileData, currentUrl, isConnection) {
    const nameParts = profileData.name.split(' ');
    const firstName = nameParts[0];
    const lastName = nameParts.slice(1).join(' ');

    // Get current company from experience array
    const currentCompany = profileData.experience && profileData.experience.length > 0 
        ? profileData.experience[0].Company  // Changed from company to Company to match the property name
        : '';

    // Create a clean version of the full profile data for ContactJSON
    const fullProfileData = {
        name: profileData.name,
        headline: profileData.headline,
        location: profileData.location,
        about: profileData.about,
        experience: profileData.experience,
        education: profileData.education,
        skills: profileData.skills,
        recommendations: profileData.recommendations,
        activities: profileData.activities,
        interests: profileData.interests,
        connectionDegree: profileData.connectionDegree,
        pendingConnection: profileData.pendingConnection,
        profileUrl: currentUrl,
        CurrentCompany: currentCompany,  // Updated to use the experience-based company
        connections: profileData.School || '',
        directMessages: profileData.directMessages || {
            hasMessages: false,
            messageCount: 0,
            messages: []
        }
        
    };

    // Format the contact data according to the API requirements
    return {
        ContactProfileURL: currentUrl,
        ContactFirstname: firstName,
        ContactLastname: lastName,
        ContactJobTitle: profileData.headline || '',
        ContactCurrentCompany: currentCompany,
        ContactTags: '',  // Default empty string for tags
        existingContact: isConnection ? "1" : "0",
        ContactJSON: JSON.stringify(fullProfileData),  // Now includes direct messages
        ConnectionRequestDatetime: '',
        LastMessageSentDatetime: '',    // Can be updated later if needed
        IsEngaged: "0"                  // Default to not engaged
    };
}

// Utility function to send contact data to API
async function sendContactDataToAPI(contactData, email, authToken) {
    console.log('🔄 Sending profile data to API...');
    const addContactResponse = await fetch(`${API_BASE_URL}/addContact`, {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json',
            'Accept': 'application/json'
        },
        body: JSON.stringify({
            email,
            authToken,
            contact: contactData
        })
    });

    if (addContactResponse.status !== 201) {
        const errorData = await addContactResponse.json();
        throw new Error(errorData.error || `Server error: ${addContactResponse.status}`);
    }

    console.log('✅ Profile data sent successfully');
    showStatus('Profile captured successfully!', 'success', 3000);
}

// Add this function to create and add the generate button
function addGenerateButton() {
    if (!document.querySelector('.goai-generate-float-button')) {
        const isConnection = getConnectionStatus();
        
        const generateButton = document.createElement('button');
        generateButton.className = 'goai-generate-float-button';
        if (!isConnection) {
            generateButton.classList.add('connect');
        }
        generateButton.textContent = isConnection ? 'goAi Messages' : 'goAi Connect Messages';
        
        generateButton.addEventListener('click', async () => {
            console.log(`${generateButton.textContent} button clicked`);
            showStatus('Capturing profile...', 'loading');
            
            try {
                const currentUrl = window.location.href;
                const contactId = currentUrl.split('/in/')[1]?.split('/')[0];
                
                if (!contactId) {
                    throw new Error('Could not extract contact ID from URL');
                }

                // Check if contactId contains any forward slashes
                if (contactId.includes('/')) {
                    console.log('Skipping profile capture - invalid contact ID format');
                    return { success: true, skipped: true };
                }

                // Get auth credentials from storage
                const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);
                
                // First, scrape the profile data
                console.log('🔄 Scraping profile data...');
                const profileData = await extractProfileData();
                if (!profileData) {
                    throw new Error('Failed to scrape profile data');
                }

                // Get profile picture URL
                const { profileImage, profilePictureUrl } = await detectProfilePicture();

                // Format the data for the API
                const contactData = formatContactData(profileData, currentUrl, isConnection);

                // Send profile data to API
                await sendContactDataToAPI(contactData, email, authToken);

                // Handle profile picture upload if available
                if (profilePictureUrl) {
                    await uploadProfilePicture(profilePictureUrl, contactId, email, authToken);
                }

                // Apply scraped style if profile image is available
                if (profileImage) {
                    applyScrapedStyle(profileImage);
                }

                // Generate messages based on connection status
                if (isConnection) {
                    console.log('Generating messages for existing contact');
                    const generatedResponse = await window.generateMessages({ existingContact: true });
                    window.displayGeneratedMessages(generatedResponse);
                } else {
                    console.log('Generating connect messages for new contact');
                    const generatedResponse = await window.generateConnectMessages({ existingContact: false });
                    window.displayGeneratedConnectMessages(generatedResponse);
                }

            } catch (error) {
                console.warn('Error:', error);
                showStatus('Error: ' + error.message, 'error', 5000);
            }
        });
        
        document.body.appendChild(generateButton);
    }
}

// Add new function to handle auto-scraping
async function handleAutoScrape() {
    try {
        const url = window.location.href;
        const contactId = url.split('/in/')[1]?.replace(/\/$/, '');
        
        // Check if contactId contains any forward slashes
        if (contactId.includes('/')) {
            console.log('Skipping profile capture - invalid contact ID format');
            return { success: true, skipped: true };
        }

        // Get auth credentials from storage
        const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);
        
        // Check if profile was recently captured
        const response = await fetch(`${API_BASE_URL}/getContact?email=${encodeURIComponent(email)}&authToken=${authToken}&contactId=${contactId}`, {
            method: 'GET',
            headers: {
                'Accept': 'application/json'
            }
        });

        const data = await response.json();
        
        if (data.success && data.contact) {
            const lastRefresh = new Date(data.contact.RefreshDatetime);
            const now = new Date();
            const timeDiff = (now - lastRefresh) / 1000 / 60;
            
            if (timeDiff < 2) {
                console.log('✅ Profile already captured recently');
                showStatus('Already Captured Recently!', 'info', 5000);
                
                const profileImage = document.querySelector('[class^="pv-top-card-profile-picture__image"]');
                if (profileImage) {
                    applyScrapedStyle(profileImage);
                }
                return;
            }
        }

        // If not recently captured, extract and send profile data
        //console.log('🔄 Extracting profile data...');
        showStatus('Capturing profile...', 'loading');
        
        const profileData = await extractProfileData();
        const { profileImage, profilePictureUrl } = await detectProfilePicture();

        const isConnection = getConnectionStatus();
        const contactData = formatContactData(profileData, url, isConnection);
        await sendContactDataToAPI(contactData, email, authToken);

        if (profilePictureUrl) {
            await uploadProfilePicture(profilePictureUrl, contactId, email, authToken);
        }

        if (profileImage) {
            applyScrapedStyle(profileImage);
        }

        return { success: true };

    } catch (error) {
        console.error('Error in auto-scrape:', error);
        showStatus('Auto-capture failed: ' + error.message, 'error', 5000);
    }
}

// Apply visual style to scraped profiles
function applyScrapedStyle(profileElement) {
    //console.log('🎨 Attempting to apply scraped style to:', profileElement);
    
    if (!profileElement) {
        console.warn('❌ Profile element is null or undefined');
        return;
    }

    try {
        profileElement.classList.add('goai-scraped-profile-picture');
        
        if (profileElement.classList.contains('goai-scraped-profile-picture')) {
            //console.log('✅ Successfully added goai-scraped-profile-picture class');
        } else {
            console.warn('❌ Failed to add class despite no errors');
        }

        // Force a DOM reflow
        profileElement.offsetHeight;
    } catch (error) {
        console.error('❌ Error applying scraped style:', error);
    }
}

// Remove visual style from profiles
function removeScrapedStyle(profileElement) {
    if (profileElement) {
        profileElement.classList.remove('goai-scraped-profile-picture');
    }
}


async function uploadProfilePicture(profilePictureUrl, contactId, email, authToken) {
    //console.log('🖼️ Uploading profile picture...');
    try {
        const imgResponse = await fetch(profilePictureUrl);
        if (imgResponse.ok) {
            const blob = await imgResponse.blob();
            const formData = new FormData();
            formData.append('contactId', contactId);
            formData.append('profilePicture', blob, 'profile.jpg');

            const uploadResponse = await fetch(`${API_BASE_URL}/saveProfilePicture?email=${encodeURIComponent(email)}&authToken=${authToken}`, {
                method: 'POST',
                headers: {
                    'Accept': 'application/json'
                },
                body: formData
            });

            if (!uploadResponse.ok) {
                console.warn('Failed to upload profile picture:', uploadResponse.status);
            } else {
                //console.log('✅ Profile picture uploaded successfully');
            }
        }
    } catch (error) {
        console.warn('Error processing profile picture:', error);
    }
}


// Setup profile observers
function setupProfileObservers() {
    chrome.storage.local.get(['selectors', 'scrapingSettings'], function(result) {
        const selectors = result.selectors;
        const scrapingSettings = result.scrapingSettings;
        
        const isAutoScrapeEnabled = scrapingSettings?.autoScrape !== undefined ? 
            scrapingSettings.autoScrape : true;
        
        const experienceSelector = selectors?.profile?.experience || 
            'section.artdeco-card > div#experience';

        // Check for existing elements
        checkExistingElements(experienceSelector, isAutoScrapeEnabled);

        // Setup observer for dynamic content
        setupDynamicContentObserver(experienceSelector, isAutoScrapeEnabled);
    });
}

// Check for existing elements on page load
function checkExistingElements(experienceSelector, isAutoScrapeEnabled) {
    const experienceSection = document.querySelector(experienceSelector);
    const goaiButton = document.querySelector('.goai-float-button');
    
    if (experienceSection && goaiButton) {
        handleExistingElements(goaiButton, isAutoScrapeEnabled);
    }
}

// Handle existing elements
function handleExistingElements(goaiButton, isAutoScrapeEnabled) {
    goaiButton.style.display = 'none';
    
    //console.log(' Updating dropdown items');
    updateDropdownItems();
    
    if (isAutoScrapeEnabled) {
        //console.log('🔄 Auto-Scraping profile data');
        document.dispatchEvent(new CustomEvent('LINKEDIN_GOAI_START'));
    }
}

// Setup observer for dynamic content
function setupDynamicContentObserver(experienceSelector, isAutoScrapeEnabled) {
    const observer = new MutationObserver((mutations, obs) => {
        const experienceSection = document.querySelector(experienceSelector);
        const goaiButton = document.querySelector('.goai-float-button');
        
        if (experienceSection && goaiButton) {
            handleExistingElements(goaiButton, isAutoScrapeEnabled);
            obs.disconnect();
        }
    });

    observer.observe(document.body, {
        childList: true,
        subtree: true
    });
}

// Update dropdown items based on connection status
async function updateDropdownItems() {
    try {
        const connectionDegree = document.querySelector('.dist-value[aria-hidden="true"]')?.innerText || null;
        //console.log('🔍 Connection Degree:', connectionDegree);
        
        const dropdownItems = document.querySelectorAll('.goai-dropdown-item[connected]');
        dropdownItems.forEach(item => {
            const isConnected = item.getAttribute('connected') === 'true';
            item.style.display = connectionDegree === '1st' ? 
                (isConnected ? 'block' : 'none') : 
                (isConnected ? 'none' : 'block');
        });
    } catch (error) {
        console.warn('Error updating dropdown items:', error);
    }
}

// Profile data extraction functions
async function extractProfileData() {
    try {
        const profileData = {
            name: getProfileName(),
            headline: getProfileHeadline(),
            location: getProfileLocation(),
            degree: getProfileDegree(),
            pendingConnection: checkPendingConnection(),
            about: await getAboutSection(),
            experience: [],
            education: await getEducationSection(),
            skills: await getSkillsSection(),
            recommendations: await getRecommendationsSection(),
            activities: await getActivitiesSection(),
            interests: await getInterestsSection(),
            connectionDegree: getConnectionDegree(),
            directMessages: await getDirectMessages()
        };

        // Extract experience data
        const experienceSections = document.querySelectorAll(PROFILE_SELECTORS.experience.section);
        for (const div of experienceSections) {
            const section = div.closest('section.artdeco-card');
            if (section) {
                const employerList = section.querySelectorAll(PROFILE_SELECTORS.experience.listItem);
                
                for (const employer of employerList) {
                    const employerData = await processEmployerData(employer);
                    profileData.experience.push(employerData);
                }
            }
        }

        // Extract company and school
        await extractCompanyAndSchool(profileData);

        return profileData;
    } catch (error) {
        console.error('Error extracting profile data:', error);
        throw new Error('Failed to extract profile data: ' + error.message);
    }
}

// Add the processEmployerData function from profileScraper.js
async function processEmployerData(employer) {
    // Count positions using the specific class
    const positionElements = employer.querySelectorAll(PROFILE_SELECTORS.experience.positionCounter);

    const numberOfPositions = positionElements.length || 1;
    if (numberOfPositions > 1) {
        return processMultiPositionEmployer(employer);
    }

    const companyElement = employer.querySelector(PROFILE_SELECTORS.experience.company);
    const companyText = companyElement?.innerText.trim() || '';
    const [companyName, employmentStatus] = companyText.split(' · ').map(text => text.trim());

    const jobTitleElement = employer.querySelector(PROFILE_SELECTORS.experience.jobTitle);
    const jobTitleText = jobTitleElement?.innerText.trim() || '';

    // Updated duration extraction
    const durationElements = employer.querySelectorAll(PROFILE_SELECTORS.experience.duration.element);
    const durationElement = durationElements[0];  // Get the first duration element
    const durationText = durationElement?.querySelector(PROFILE_SELECTORS.experience.duration.text)?.innerText.trim() || '';

    // Updated location extraction
    const locationElements = employer.querySelectorAll(PROFILE_SELECTORS.experience.location.element);
    const locationElement = locationElements[1];  // Get the second location element
    const locationText = locationElement?.querySelector(PROFILE_SELECTORS.experience.location.text)?.innerText.trim() || '';

    return {
        Company: companyName,
        EmploymentStatus: employmentStatus,
        JobTitle: jobTitleText,
        Duration: durationText,
        Location: locationText,
        NumberOfPositions: numberOfPositions
    };
}

// Add new function to handle multiple positions
function processMultiPositionEmployer(employer) {
    let numberOfPositions;

    // First try to get the number from the "see all positions" link
    const showAllPositionsElement = employer.querySelector(PROFILE_SELECTORS.experience.positionCountLink);
    if (showAllPositionsElement) {
        const innerText = showAllPositionsElement.textContent || '';
        numberOfPositions = innerText.match(/\d+/g)?.join('') || '';
        //console.log('🔍 Number of positions from link:', numberOfPositions);
    } else {
        // Fallback to counting position spans
        const positionElements = employer.querySelectorAll(PROFILE_SELECTORS.experience.positionCounter);
        numberOfPositions = positionElements.length-1 || '?';
        //console.log('🔍 Number of positions from spans:', numberOfPositions);
    }

    // Get company name and job title using the new selector
    const elements = employer.querySelectorAll(PROFILE_SELECTORS.experience.multiPositionCompanyNameAndTitle);
    const companyName = elements[0]?.innerText.trim() || '';
    const jobTitle = elements[1]?.innerText.trim() || '';
    //console.log('🔍 Company Name:', companyName);
    //console.log('🔍 Job Title:', jobTitle);

    // Updated duration extraction
    const durationElements = employer.querySelectorAll(PROFILE_SELECTORS.experience.duration.element);
    const durationElement = durationElements[0];  // Get the first duration element
    const durationText = durationElement?.querySelector(PROFILE_SELECTORS.experience.duration.text)?.innerText.trim() || '';

    // Updated location extraction
    const locationElements = employer.querySelectorAll(PROFILE_SELECTORS.experience.duration.element);
    const locationElement = locationElements[1];  // Get the section element
    const locationText = locationElement?.querySelector(PROFILE_SELECTORS.experience.duration.text)?.innerText.trim() || '';    

    const employmentStatuses = employer.querySelectorAll(PROFILE_SELECTORS.experience.multiPositionStatus);
    //console.log('🔍 Employment Statuses:', employmentStatuses);
    const employmentStatus = employmentStatuses[1]?.innerText.trim() || '';

    return {
        Company: companyName,
        EmploymentStatus: employmentStatus,
        JobTitle: jobTitle,
        Duration: durationText,
        Location: locationText,
        NumberOfPositions: numberOfPositions
    };
}

// Profile section extraction helper functions
function getProfileName() {
    const nameElement = document.querySelector(PROFILE_SELECTORS.name);
    //console.log('🔍 Name:', nameElement);
    //console.log('🔍 Name text:', nameElement?.innerText);
    return nameElement ? nameElement.innerText.trim() : '';
}

function getProfileHeadline() {
    const headlineElement = document.querySelector(PROFILE_SELECTORS.headline);
    return headlineElement ? headlineElement.innerText.trim() : '';
}

function getProfileLocation() {
    const locationElement = document.querySelector(PROFILE_SELECTORS.location);
    return locationElement ? locationElement.innerText.trim() : '';
}

function getAboutSection() {
    const sections = document.querySelectorAll(PROFILE_SELECTORS.about.sections);
    for (const section of sections) {
        const headerSpan = section.querySelector(PROFILE_SELECTORS.about.headerSpan);
        if (headerSpan && headerSpan.innerText.trim() === 'About') {
            const aboutContent = section.querySelector(PROFILE_SELECTORS.about.content);
            if (aboutContent) {
                let aboutText = aboutContent.querySelector(PROFILE_SELECTORS.about.text).innerText.trim();
                return aboutText.replace(/…see more$/, '').trim();
            }
            break;
        }
    }
    return '';
}

function getConnectionDegree() {
    const degreeElement = document.querySelector(PROFILE_SELECTORS.connections);
    return degreeElement ? degreeElement.innerText.trim() : '';
}

async function getEducationSection() {
    const educationSection = document.getElementById(PROFILE_SELECTORS.education.section.slice(1))?.closest('section');
    if (!educationSection) return [];
    const educations = educationSection.querySelectorAll(PROFILE_SELECTORS.education.eduItem);
    return Array.from(educations).map(edu => {
        const school = edu.querySelector(PROFILE_SELECTORS.education.school)?.innerText.trim() || '';
        const fieldOfStudyElement = edu.querySelector(PROFILE_SELECTORS.education.fieldOfStudy);
        const fieldOfStudy = fieldOfStudyElement?.innerText.trim().split('Field Of Study').pop().trim() || '';
        const duration = edu.querySelector(PROFILE_SELECTORS.education.duration)?.innerText.trim() || '';
        const additionalInfo = edu.querySelector(PROFILE_SELECTORS.education.additionalInfo)?.innerText.trim() || '';        
        return { 
            School: school,
            FieldOfStudy: fieldOfStudy,
            Duration: duration,
            AdditionalInfo: additionalInfo
        };
    });
}

async function getSkillsSection() {
    const skillsSection = document.getElementById(PROFILE_SELECTORS.skills.section.slice(1))?.closest('section');
    if (!skillsSection) return [];

    const skills = skillsSection.querySelectorAll(PROFILE_SELECTORS.skills.listItem);
    return Array.from(skills).map(skill => {
        return skill.querySelector(PROFILE_SELECTORS.skills.title)?.innerText.trim() || '';
    });
}

async function getRecommendationsSection() {
    const recommendationsSection = document.getElementById(PROFILE_SELECTORS.recommendations.section.slice(1))?.closest('section');
    if (!recommendationsSection) return [];

    const recommendations = recommendationsSection.querySelectorAll(PROFILE_SELECTORS.recommendations.listItem);
    return Array.from(recommendations).map(rec => {
        const author = rec.querySelector(PROFILE_SELECTORS.recommendations.author)?.innerText.trim() || '';
        const text = rec.querySelector(PROFILE_SELECTORS.recommendations.text)?.innerText.trim() || '';
        return { author, text };
    });
}

async function getActivitiesSection() {
    const activitiesSection = document.getElementById(PROFILE_SELECTORS.activities.section.slice(1))?.closest('section');
    if (!activitiesSection) return [];
    const activities = activitiesSection.querySelectorAll(PROFILE_SELECTORS.activities.listItem);
    
    return Array.from(activities).map(activity => {
        //console.log('Full activity element:', activity);
        const whoPostedWhen = `${activity.querySelector(PROFILE_SELECTORS.activities.whoPostedAndWhen)?.innerText.trim()} ago` || '';
        
        // Extract who and when separately
        const [rawWho, rawWhen] = whoPostedWhen.split(' • ');
        const postWho = rawWho.replace(' posted this', '').replace(' reposted this', '').trim();
        
        // Calculate actual datetime for postWhen
        const calculateDateTime = (timeAgo) => {
            const now = new Date();
            const value = parseInt(timeAgo.match(/\d+/)[0]);
            const unit = timeAgo.match(/[a-z]+/)[0];
            
            switch(unit) {
                case 'h':
                    return new Date(now - value * 60 * 60 * 1000).toISOString();
                case 'd':
                    return new Date(now - value * 24 * 60 * 60 * 1000).toISOString();
                case 'w':
                    return new Date(now - value * 7 * 24 * 60 * 60 * 1000).toISOString();
                case 'mo':
                    // Approximate month as 30 days
                    return new Date(now - value * 30 * 24 * 60 * 60 * 1000).toISOString();
                case 'yr':
                    // Calculate year based on 365 days
                    return new Date(now - value * 365 * 24 * 60 * 60 * 1000).toISOString();
                default:
                    console.log('Unhandled time unit:', unit, 'in timeAgo:', timeAgo);
                    return now.toISOString();
            }
        };
        
        const postWhen = calculateDateTime(rawWhen);
        
        const postContent = activity.querySelector(PROFILE_SELECTORS.activities.postContent)?.getAttribute('aria-label') || '';
        // Remove "View full post" from the beginning of postContent
        const cleanedPostContent = postContent.replace("View full post", "").trim();
        const postSocialReactions = activity.querySelector(PROFILE_SELECTORS.activities.postSocialReactions)?.innerText.trim() || '';
        const postDirectLink = activity.querySelector(PROFILE_SELECTORS.activities.postContent)?.getAttribute('href')?.split('?')[0] || '';
        
        const postContextElement = activity.querySelector(PROFILE_SELECTORS.activities.postContext);
        //console.log('Post context element:', postContextElement);
        const postContext = postContextElement?.innerText.trim() || '';
        //console.log('Post context value:', postContext);
        
        const rawComments = activity.querySelector(PROFILE_SELECTORS.activities.postComments)?.innerText.trim() || '';
        const postComments = rawComments.replace(' comments', '').replace(' comment', '');
        //console.log('Post comments value:', postComments);
        
        return { 
            whoPostedWhen, 
            postWho,
            postWhen,
            postContent: cleanedPostContent, 
            postContext,
            postComments,
            postSocialReactions, 
            postDirectLink             
        };
    });
}

async function getInterestsSection() {
    const interestsSection = document.getElementById(PROFILE_SELECTORS.interests.section.slice(1))?.closest('section');
    if (!interestsSection) return [];

    const interests = interestsSection.querySelectorAll(PROFILE_SELECTORS.interests.listItem);
    return Array.from(interests).map(interest => {
        return interest.querySelector(PROFILE_SELECTORS.interests.title)?.innerText.trim() || '';
    });
}

// Profile capture and upload functions
async function captureAndUploadProfile() {
    const currentUrl = window.location.href;
    
   
    const contactId = currentUrl.split('/in/')[1]?.split('/')[0];
    
    if (!contactId) {
        throw new Error('Could not extract contact ID from URL');
    }

    // Check if contactId contains any forward slashes
    if (contactId.includes('/')) {
        //console.log('Skipping profile capture - invalid contact ID format');
        return { success: true, skipped: true };
    }

    // Get auth credentials from storage
    const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);
    
    // Scrape profile data
    //console.log('🔄 Scraping profile data...');
    const profileData = await extractProfileData();
    if (!profileData) {
        throw new Error('Failed to scrape profile data');
    }

    // Get and process profile picture
    const { profileImage, profilePictureUrl } = await detectProfilePicture();

    const isConnection = getConnectionStatus();
    const contactData = formatContactData(profileData, currentUrl, isConnection);
    await sendContactDataToAPI(contactData, email, authToken);

    if (profilePictureUrl) {
        await uploadProfilePicture(profilePictureUrl, contactId, email, authToken);
    }

    if (profileImage) {
        applyScrapedStyle(profileImage);
    }

    return { success: true };
}

// Profile picture handling functions
async function detectProfilePicture() {
    console.group('🖼️ Profile Picture Detection');
    let profileImage = document.querySelector(PROFILE_SELECTORS.profilePicture.primary);
    
    if (!profileImage) {
        profileImage = document.querySelector(PROFILE_SELECTORS.profilePicture.fallback);
    }

    const profilePictureUrl = profileImage ? profileImage.src : null;
    console.groupEnd();

    return { profileImage, profilePictureUrl };
}

// Extract company and school
async function extractCompanyAndSchool(data) {
    const container = document.querySelector(PROFILE_SELECTORS.quickView.container);
    if (container) {
        const items = container.querySelectorAll(PROFILE_SELECTORS.quickView.items);
        if (items.length >= 1) {
            data.CurrentCompany = items[0].innerText.trim();
        }
        if (items.length >= 2) {
            data.School = items[1].innerText.trim();
        }
    }
}

// Add new helper function for degree
function getProfileDegree() {
    const degreeElement = document.querySelector(PROFILE_SELECTORS.degree);
    return degreeElement ? degreeElement.innerText.trim() : '';
}

// Update getDirectMessages function to include message content
async function getDirectMessages() {
    const mainElement = document.querySelector(PROFILE_SELECTORS.directMessages.mainElement);
    if (!mainElement) {
        //console.log('🔍 No direct messages element found');
        return {
            hasMessages: false,
            messageCount: 0,
            messages: []
        };
    }

    const messageElements = mainElement.querySelectorAll(PROFILE_SELECTORS.directMessages.messageElement);
    const messageCount = messageElements.length;
    
    // Extract text content from each message
    const messages = Array.from(messageElements).map(messageElement => {
        const messageText = messageElement.innerText.trim();
        return {
            text: messageText,
        };
    });
    
    //console.log('🔍 Direct messages found:', messageCount);
    //console.log('🔍 Messages:', messages);

    return {
        hasMessages: messageCount > 0,
        messageCount: messageCount,
        messages: messages
    };
}

// Check if connection is pending
function checkPendingConnection() {
    const topSection = document.querySelector(PROFILE_SELECTORS.topSectionNew);
    if (!topSection) return 0;
    
    // Find all span elements with the class and check their text content
    const spans = topSection.querySelectorAll(PROFILE_SELECTORS.pendingConnection.container);
    for (const span of spans) {
        if (span.textContent.trim() === PROFILE_SELECTORS.pendingConnection.text) {
            return 1;
        }
    }
    return 0;
}

// Export all necessary functions
export {
    setupProfilePageReadyObserver,
    captureAndUploadProfile,
    applyScrapedStyle,
    removeScrapedStyle,
    uploadProfilePicture,
    extractProfileData,
    sendContactDataToAPI,
    handleAutoScrape,
    getConnectionStatus,
    formatContactData
}; 