import { API_BASE_URL, UI_CONSTANTS } from './constants.js';
import { generateMessages, generateConnectMessages } from './messageHandlers.js';
import { captureAndUploadProfile } from './profileHandler.js';

let globalStatusIndicator;

// Initialize UI components
async function initializeUI() {
    // Create status indicator if it doesn't exist
    if (!document.querySelector('.goai-status-indicator')) {
        globalStatusIndicator = createStatusIndicator();
        document.body.appendChild(globalStatusIndicator);
    }

    // Create floating message container
    const floatingMessageContainer = createMessageContainer();
    document.body.appendChild(floatingMessageContainer);
}

// Create and manage status indicator
function createStatusIndicator() {
    const indicator = document.createElement('div');
    indicator.className = 'goai-status-indicator';
    indicator.style.position = 'fixed';
    indicator.style.top = '130px';
    indicator.style.left = '20px';
    indicator.style.zIndex = '10000';
    indicator.style.padding = '8px 16px';
    indicator.style.borderRadius = '4px';
    indicator.style.backgroundColor = '#fff';
    indicator.style.boxShadow = '0 2px 8px rgba(0,0,0,0.2)';
    indicator.style.transition = 'opacity 0.3s ease';
    indicator.style.opacity = '0';
    indicator.style.display = 'none';
    indicator.style.minWidth = '150px';
    indicator.style.maxWidth = '300px';
    indicator.style.whiteSpace = 'normal';
    indicator.style.wordBreak = 'break-word';
    indicator.style.fontSize = '14px';
    indicator.style.lineHeight = '1.4';
    return indicator;
}

// Show status message with improved reliability
function showStatus(message, type = 'info', duration = null) {
    // Create status indicator if it doesn't exist
    if (!document.querySelector('.goai-status-indicator')) {
        globalStatusIndicator = createStatusIndicator();
        document.body.appendChild(globalStatusIndicator);
    }

    const statusIndicator = document.querySelector('.goai-status-indicator');
    if (!statusIndicator) return;

    // Clear any existing timeouts
    if (window.statusTimeout) {
        clearTimeout(window.statusTimeout);
        window.statusTimeout = null;
    }

    // Reset classes and add new ones
    statusIndicator.className = 'goai-status-indicator';
    statusIndicator.classList.add(type);
    
    // Set message and ensure display
    statusIndicator.textContent = message;
    statusIndicator.style.display = 'block';
    statusIndicator.style.opacity = '1';
    
    // Force a reflow to ensure CSS transitions work
    statusIndicator.offsetHeight;
    
    // Make visible
    statusIndicator.classList.add('visible');

    // Auto-hide for success messages
    if (type === 'success' && duration === null) {
        duration = 3000;
    }

    if (duration) {
        window.statusTimeout = setTimeout(() => {
            hideStatus();
        }, duration);
    }
}

// Hide status message with improved reliability
function hideStatus(delay = 0) {
    const statusIndicator = document.querySelector('.goai-status-indicator');
    if (!statusIndicator) return;

    // Clear any existing timeouts
    if (window.statusTimeout) {
        clearTimeout(window.statusTimeout);
        window.statusTimeout = null;
    }

    window.statusTimeout = setTimeout(() => {
        statusIndicator.style.opacity = '0';
        statusIndicator.classList.remove('visible');
        
        setTimeout(() => {
            if (statusIndicator.style.opacity === '0') {
                statusIndicator.style.display = 'none';
            }
        }, 300);
    }, delay);
}

// Create message container
function createMessageContainer() {
    const container = document.createElement('div');
    container.id = 'floatingMessageContainer';
    container.style.display = 'none';
    
    chrome.storage.local.get('userInstructions', function(data) {
        const userInstructions = data.userInstructions || 'No user instructions found';
        container.innerHTML = `
            <button class="goai-close-floating" aria-label="Close messages"></button>
            <div id="messageContent">Loading...</div>
        `;
    });
    
    return container;
}

// Update generate button based on connection status
function updateGenerateButton() {
    const generateButton = document.querySelector('.goai-generate-float-button');
    if (generateButton) {
        const connectionDegree = document.querySelector(UI_CONSTANTS.SELECTORS.CONNECTION_DEGREE)?.innerText;
        
        if (connectionDegree !== null) {
            const isConnection = connectionDegree === '1st';
            generateButton.textContent = isConnection ? 'goAi Messages' : 'goAi Connect Messages';
            
            if (isConnection) {
                generateButton.classList.remove('connect');
            } else {
                generateButton.classList.add('connect');
            }
            
            generateButton.style.display = 'block';
        } else {
            generateButton.style.display = 'none';
        }
    }
}

// Add generate button
function addGenerateButton() {
    if (!document.querySelector('.goai-generate-float-button')) {
        const connectionDegree = document.querySelector(UI_CONSTANTS.SELECTORS.CONNECTION_DEGREE)?.innerText;
        const isConnection = connectionDegree === '1st';
        
        const generateButton = document.createElement('button');
        generateButton.className = 'goai-generate-float-button';
        if (!isConnection) {
            generateButton.classList.add('connect');
        }
        generateButton.textContent = isConnection ? 'goAi Messages' : 'goAi Connect Messages';
        
        generateButton.addEventListener('click', handleGenerateButtonClick);
        document.body.appendChild(generateButton);
    }
}

// Updated handle generate button click
async function handleGenerateButtonClick() {
    showStatus('Capturing profile...', 'loading');
    
    try {
        // Capture and upload profile data
        await captureAndUploadProfile();
        
        // Get connection degree to determine message type
        const connectionDegree = document.querySelector(UI_CONSTANTS.SELECTORS.CONNECTION_DEGREE)?.innerText;
        const isConnection = connectionDegree === '1st';
        
        if (isConnection) {
            console.log('Generating messages for existing contact');
            const generatedResponse = await generateMessages({
                existingContact: true,
                messageType: 'direct-message'
            });
            displayGeneratedMessages(generatedResponse);
        } else {
            console.log('Generating connect messages for new contact');
            const generatedResponse = await generateConnectMessages({
                existingContact: false,
                messageType: 'connection-message'
            });
            displayGeneratedConnectMessages(generatedResponse);
        }

    } catch (error) {
        console.warn('Error:', error);
        showStatus('Error: ' + error.message, 'error', 5000);
    }
}

// Export necessary functions
export {
    initializeUI,
    addGenerateButton,
    updateGenerateButton,
    showStatus,
    hideStatus,
    createStatusIndicator
}; 