// Initialize all modules
async function initialize() {
    try {
        // Define resetUIState first
        window.resetUIState = function() {
            isScrapingInProgress = false;
            lastScrapedUrl = null;
            
            const generateButton = document.querySelector('.goai-generate-float-button');
            if (generateButton) {
                generateButton.style.display = 'none';
            }

            const profileImage = document.querySelector('[class^="pv-top-card-profile-picture__image"]');
            if (profileImage) {
                window.removeScrapedStyle(profileImage);
            }

            const statusIndicator = document.querySelector('.goai-status-indicator');
            if (statusIndicator) {
                statusIndicator.style.display = 'none';
                statusIndicator.classList.remove('visible');
            }

            const floatingMessageContainer = document.getElementById('floatingMessageContainer');
            if (floatingMessageContainer) {
                floatingMessageContainer.style.display = 'none';
            }

            // Re-run observers after reset
            setTimeout(() => {
                window.setupProfilePageReadyObserver();
            }, 1000);

            setTimeout(() => {
                window.updateGenerateButton();
            }, 1500);
        };

        // Dynamically import modules
        const { setupUrlHandlers } = await import(chrome.runtime.getURL('contentPage/urlHandler.js'));
        const { 
            initializeUI, 
            showStatus, 
            hideStatus, 
            updateGenerateButton,
            addGenerateButton
        } = await import(chrome.runtime.getURL('contentPage/uiHandler.js'));
        const {
            setupProfilePageReadyObserver,
            applyScrapedStyle,
            removeScrapedStyle,
            uploadProfilePicture,
            extractProfileData,
            getConnectionStatus,
            sendContactDataToAPI,
            formatContactData,
            PROFILE_SELECTORS
        } = await import(chrome.runtime.getURL('contentPage/profileHandler.js'));
        const {
            generateMessages,
            generateConnectMessages,
            sendLinkedInMessage,
            sendConnectionRequest,
            setupMessageHandlers,
            displayGeneratedMessages,
            displayGeneratedConnectMessages
        } = await import(chrome.runtime.getURL('contentPage/messageHandlers.js'));
        const { initializeConnectionsHandler } = await import(chrome.runtime.getURL('contentPage/connectionsHandler.js'));

        // Setup URL handlers first, passing resetUIState
        setupUrlHandlers(window.resetUIState);

        // Initialize UI components
        await initializeUI();
        
        // Add generate button and setup observers
        addGenerateButton();
        
        // Setup profile page observer
        setupProfilePageReadyObserver();
        
        // Setup message handlers
        setupMessageHandlers();
        
        // Initialize connections handler
        initializeConnectionsHandler();
        
        // Inject styles
        injectStyles();

        // Make functions globally available
        window.showStatus = showStatus;
        window.hideStatus = hideStatus;
        window.updateGenerateButton = updateGenerateButton;
        window.applyScrapedStyle = applyScrapedStyle;
        window.removeScrapedStyle = removeScrapedStyle;
        window.generateMessages = generateMessages;
        window.generateConnectMessages = generateConnectMessages;
        window.sendLinkedInMessage = sendLinkedInMessage;
        window.sendConnectionRequest = sendConnectionRequest;
        window.uploadProfilePicture = uploadProfilePicture;
        window.extractProfileData = extractProfileData;
        window.getConnectionStatus = getConnectionStatus;
        window.sendContactDataToAPI = sendContactDataToAPI;
        window.formatContactData = formatContactData;
        window.setupProfilePageReadyObserver = setupProfilePageReadyObserver;
        window.displayGeneratedMessages = displayGeneratedMessages;
        window.displayGeneratedConnectMessages = displayGeneratedConnectMessages;
        
        console.log('✅ goAi extension initialized successfully');

        // Add message listener for automation
        chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
            if (request.action === 'ping') {
                sendResponse({ success: true });
                return true;
            }

            if (request.action === 'sendMessage') {
                handleAutomatedMessage(request, sendResponse);
                return true; // Keep the message channel open for async response
            }
        });

        // Add listener for profile processing
        chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
            if (request.action === 'processProfile') {
                (async () => {
                    try {
                        console.log('🤖 Processing profile...', request);
                        
                        // Import required functions
                        const { 
                            extractProfileData, 
                            sendContactDataToAPI, 
                            getConnectionStatus, 
                            formatContactData,
                            PROFILE_SELECTORS 
                        } = await import(chrome.runtime.getURL('contentPage/profileHandler.js'));
                        
                        console.log('Imported functions:', { 
                            hasExtractProfileData: !!extractProfileData,
                            hasSendContactDataToAPI: !!sendContactDataToAPI,
                            hasGetConnectionStatus: !!getConnectionStatus,
                            hasFormatContactData: !!formatContactData
                        });

                        // Wait for profile page to load
                        //console.log('Waiting for profile page to load...');
                        await waitForProfileLoad(PROFILE_SELECTORS);
                        //console.log('Profile page loaded, extracting data...');

                        // Extract profile data
                        const profileData = await extractProfileData();
                        //console.log('Extracted profile data:', profileData);
                        
                        if (!profileData) {
                            throw new Error('Failed to extract profile data');
                        }

                        // Get connection status
                        const isConnection = await getConnectionStatus();
                        //console.log('Connection status:', isConnection);

                        // Format contact data
                        const contactData = formatContactData(profileData, window.location.href, isConnection);
                        //console.log('Formatted contact data:', contactData);

                        // Send success response
                        sendResponse({ 
                            success: true, 
                            data: contactData 
                        });

                    } catch (error) {
                        console.error('Error processing profile:', error);
                        sendResponse({ 
                            success: false, 
                            error: error.message 
                        });
                    }
                })();
                return true; // Keep the message channel open for async response
            }
        });
    } catch (error) {
        console.error('❌ Error initializing goAi extension:', error);
    }
}

// Global state
let isScrapingInProgress = false;
let lastScrapedUrl = null;

// Inject required styles
function injectStyles() {
    const link = document.createElement('link');
    link.rel = 'stylesheet';
    link.type = 'text/css';
    link.href = chrome.runtime.getURL('styles.css');
    document.head.appendChild(link);
}

// Update the handleAutomatedMessage function
async function handleAutomatedMessage(request, sendResponse) {
    try {
        const { messageType, message, action } = request;

        console.group('🤖 Automation - Message Received');
        console.log('Message Type:', messageType);
        console.log('Action:', action);
        console.log('Message Content:', message);
        console.log('Current URL:', window.location.href);
        console.groupEnd();

        console.log('🤖 Importing profile selectors...');
        // Import PROFILE_SELECTORS first
        const { PROFILE_SELECTORS } = await import(chrome.runtime.getURL('contentPage/profileHandler.js'));
        console.log('✅ Profile selectors imported');
        
        console.log('🤖 Waiting for profile page to load...');
        await waitForProfileLoad(PROFILE_SELECTORS);
        console.log('✅ Profile page loaded');

        // Import new automated message handlers from automatedActions.js
        const { sendAutomatedConnectionRequest, sendAutomatedDirectMessage } = 
            await import(chrome.runtime.getURL('contentPage/automatedActions.js'));

        let success = false;
        // Use new dedicated functions based on message type
        if (messageType === 'connection-message' || (action === 'sendMessage' && messageType === 'connection-message')) {
            console.log('🤖 Starting automated connection  process...');
            success = await sendAutomatedConnectionRequest(message, PROFILE_SELECTORS);
        } else if (messageType === 'direct-message' || (action === 'sendMessage' && messageType === 'direct-message')) {
            console.log('🤖 Starting automated direct message process...');
            success = await sendAutomatedDirectMessage(message, PROFILE_SELECTORS);
        } else if (messageType === 'visit-profile' || action === 'processProfile') {
            // For visit-profile, we just need to wait for the profile to load, which we've already done
            console.log('✅ Profile visited successfully');
            success = true;
        } else {
            throw new Error(`Unsupported message type: ${messageType || action}`);
        }

        if (!success) {
            console.error('❌ Message automation failed');
            throw new Error('Failed to send message');
        }

        console.log('✅ Automation completed successfully');
        sendResponse({ success: true });
    } catch (error) {
        console.error('❌ Automation error:', error);
        console.error('Error stack:', error.stack);
        sendResponse({ success: false, error: error.message });
    }
}

// Update waitForProfileLoad to accept PROFILE_SELECTORS as a parameter
async function waitForProfileLoad(PROFILE_SELECTORS) {
    //console.log('🔍 Starting profile page load check...');
    return new Promise((resolve) => {
        const startTime = Date.now();
        const timeout = 30000; // 30 second timeout
        const activitiesTimeout = 5000; // 5 second timeout for activities section
        let activitiesStartTime = null;

        const checkPage = () => {
            // Use the passed PROFILE_SELECTORS
            const profileSection = document.querySelectorAll(PROFILE_SELECTORS.mainSection);
            const timeElapsed = Date.now() - startTime;

            if (profileSection) {
                //console.log(`✅ Profile section found after ${timeElapsed}ms`);
                // Check for experience section to ensure full load
                const experienceSection = document.querySelector(PROFILE_SELECTORS.experience.container);
                if (experienceSection) {
                    //console.log('✅ Experience section found - checking activities section');
                    
                    // Start tracking activities section load time
                    if (!activitiesStartTime) {
                        activitiesStartTime = Date.now();
                    }
                    
                    const activitiesSection = document.querySelector(PROFILE_SELECTORS.activities.section);
                    const activitiesTimeElapsed = Date.now() - activitiesStartTime;
                    
                    if (activitiesSection) {
                        //console.log('✅ Activities section found - page fully loaded');
                        resolve();
                    } else if (activitiesTimeElapsed > activitiesTimeout) {
                        //console.log('⚠️ Activities section not found after 5 seconds - proceeding anyway');
                        resolve();
                    } else {
                        //console.log(`🔄 Waiting for activities section... (${activitiesTimeElapsed}ms elapsed)`);
                        setTimeout(checkPage, 500);
                    }
                } else {
                    //console.log('🔄 Waiting for experience section...');
                    setTimeout(checkPage, 500);
                }
            } else if (timeElapsed > timeout) {
                console.error('❌ Profile load timeout after 30 seconds');
                resolve(); // Resolve anyway to prevent hanging
            } else {
                //console.log(`🔄 Profile section not found, checking again... (${timeElapsed}ms elapsed)`);
                setTimeout(checkPage, 500);
            }
        };
        checkPage();
    });
}

// Initialize the extension
initialize();
