const API_BASE_URL = 'https://www.goai-app.com';

async function saveEmailState(email) {
  await chrome.storage.local.set({ pendingLoginEmail: email });
}

async function getEmailState() {
  const { pendingLoginEmail } = await chrome.storage.local.get('pendingLoginEmail');
  return pendingLoginEmail;
}

document.addEventListener('DOMContentLoaded', async () => {
    const emailInput = document.getElementById('emailInput');
    const submitEmailBtn = document.getElementById('submitEmail');
    const codeSection = document.getElementById('codeSection');
    const codeInput = document.getElementById('codeInput');
    const loginButton = document.getElementById('loginButton');
    const messageDiv = document.getElementById('message');
    const logoutLink = document.getElementById('logoutLink');
    const startOverLink = document.getElementById('startOver');

    // Check for pending email verification
    const pendingEmail = await getEmailState();
    if (pendingEmail) {
        emailInput.value = pendingEmail;
        emailInput.disabled = true;
        submitEmailBtn.textContent = 'Code Sent';
        submitEmailBtn.disabled = true;
        codeSection.classList.remove('hidden');
    }

    // Function to display messages
    function displayMessage(text, type) {
        messageDiv.textContent = text;
        messageDiv.className = `message ${type}`;
    }

    // Add after the existing displayMessage function
    function showReferralSection() {
        const referralHtml = `
            <div id="referralSection" class="input-section">
                <input type="text" id="referralInput" placeholder="Enter Sign-Up Code" required>
                <button id="createAccountBtn">Create Account</button>
            </div>
        `;
        
        // Insert after emailSection
        document.getElementById('emailSection').insertAdjacentHTML('afterend', referralHtml);
        
        // Add event listener for create account button
        document.getElementById('createAccountBtn').addEventListener('click', handleCreateAccount);
    }

    async function handleCreateAccount() {
        const email = document.getElementById('emailInput').value.trim();
        const referralCode = document.getElementById('referralInput').value.trim();
        const createAccountBtn = document.getElementById('createAccountBtn');
        const submitEmailBtn = document.getElementById('submitEmail');
        
        if (!referralCode) {
            displayMessage('Please enter a valid sign-up code.', 'error');
            return;
        }

        try {
            createAccountBtn.disabled = true;
            createAccountBtn.textContent = 'Creating...';
            displayMessage('Creating account...', 'info');

            const response = await fetch(`${API_BASE_URL}/createUser`, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify({
                    user_email_address: email,
                    referred_by: referralCode
                })
            });

            const data = await response.json();

            if (data.success || data.error === "User already exists") {
                // Remove referral section
                document.getElementById('referralSection').remove();
                
                // Show success message
                displayMessage('Account created successfully! Please check your email for login code.', 'success');
                
                // Set login state to code_sent and save email
                await Promise.all([
                    chrome.storage.local.set({ loginState: 'code_sent' }),
                    saveEmailState(email)
                ]);
                
                // Update UI to show code input
                emailInput.disabled = true;
                submitEmailBtn.textContent = 'Code Sent';
                submitEmailBtn.disabled = true;
                codeSection.classList.remove('hidden');
            } else {
                throw new Error(data.message || 'Failed to create account');
            }

        } catch (error) {
            console.error('Error:', error);
            createAccountBtn.disabled = false;
            createAccountBtn.textContent = 'Create Account';
            displayMessage(error.message || 'Failed to create account. Please try again.', 'error');
        }
    }

    // Handle email submission
    submitEmailBtn.addEventListener('click', async () => {
        const email = emailInput.value.trim();
        if (email && isValidEmail(email)) {
            try {
                submitEmailBtn.disabled = true;
                submitEmailBtn.textContent = 'Sending...';
                displayMessage('Sending login code...', 'info');

                const response = await fetch(`${API_BASE_URL}/login?email=${encodeURIComponent(email)}`);
                const data = await response.json();
                
                if (data.error) {
                    // Show referral input for account creation
                    showReferralSection();
                    submitEmailBtn.textContent = 'Submit';
                    displayMessage('Email not found. Please enter referral email to create account.', 'info');
                    
                    // Save the signup code request state
                    await chrome.storage.local.set({ 
                        loginState: 'signup_code_request',
                        pendingLoginEmail: email 
                    });
                    return;
                }

                // If we get here, the login was successful
                await saveEmailState(email);
                await chrome.storage.local.set({ loginState: 'code_sent' });

                emailInput.disabled = true;
                submitEmailBtn.textContent = 'Code Sent';
                codeSection.classList.remove('hidden');
                displayMessage('Login code sent. Please check your email.', 'success');
                
            } catch (error) {
                console.error('Error:', error);
                submitEmailBtn.textContent = 'Submit';
                submitEmailBtn.disabled = false;
                displayMessage('An unexpected error occurred. Please try again.', 'error');
            }
        } else {
            displayMessage('Please enter a valid email address.', 'error');
        }
    });

    // Handle code input
    codeInput.addEventListener('input', (e) => {
        e.target.value = e.target.value.replace(/\D/g, '');
        
        if (e.target.value.length === 4) {
            loginButton.classList.remove('hidden');
            // Note: Button remains disabled until terms are accepted
        } else {
            loginButton.classList.add('hidden');
        }
    });

    // Handle login button click
    loginButton.addEventListener('click', async () => {
        const email = emailInput.value.trim();
        const magicCode = codeInput.value.trim();

        try {
            loginButton.disabled = true;
            loginButton.textContent = 'Authenticating...';
            displayMessage('Authenticating...', 'info');

            const response = await fetch(`${API_BASE_URL}/authenticate?email=${encodeURIComponent(email)}&magicCode=${magicCode}`, {
                method: 'GET',
                headers: {
                    'Content-Type': 'application/json',
                    'Accept': 'application/json',
                    'Origin': chrome.runtime.getURL('')
                },
                credentials: 'include'
            });

            if (!response.ok) {
                throw new Error('Authentication failed');
            }

            const data = await response.json();
            loginButton.textContent = 'Success!';
            displayMessage('Authentication successful!', 'success');

            // Start a new analytics session
            chrome.runtime.sendMessage({ action: 'startNewSession' });

            // Send login success event with session data
            chrome.runtime.sendMessage({
                action: 'sendAnalyticsEvent',
                eventName: 'user_login',
                eventParams: {
                    login_type: 'email_code',
                    auth_method: 'magic_link',
                    success: true
                }
            });

            // Clear login state after successful authentication
            await chrome.storage.local.remove(['pendingLoginEmail', 'loginState']);
            await chrome.storage.local.set({ 
                authentication_token: data.authentication_token, 
                email: email,
                loginState: 'authenticated'
            });

            // Try to fetch LinkedIn data first
            try {
                console.log('🔍 Starting LinkedIn profile fetch...');
                displayMessage('Fetching LinkedIn profile...', 'info');
                
                // Set up listener for LinkedIn data
                const linkedinDataPromise = new Promise((resolve, reject) => {
                    const messageListener = (message) => {
                        if (message.type === 'linkedInProfileData') {
                            chrome.runtime.onMessage.removeListener(messageListener);
                            if (message.success) {
                                resolve(message);
                            } else {
                                reject(new Error(message.error));
                            }
                        }
                    };
                    chrome.runtime.onMessage.addListener(messageListener);
                    
                    // Set a timeout for the fetch
                    setTimeout(() => {
                        chrome.runtime.onMessage.removeListener(messageListener);
                        reject(new Error('LinkedIn data fetch timed out'));
                    }, 30000); // 30 second timeout
                });
                
                // Start the fetch
                await chrome.runtime.sendMessage({ action: "getLinkedInProfile" });
                
                // Wait for the data
                const linkedinResponse = await linkedinDataPromise;
                console.log('📦 LinkedIn Response:', linkedinResponse);
                
                // Debug the data structure
                console.log('🔍 Data structure:', {
                    data: linkedinResponse?.data,
                    basicProfile: linkedinResponse?.data?.basicProfile,
                    included: linkedinResponse?.data?.basicProfile?.included
                });
                
                // The data is nested two levels deep in the response
                const miniProfile = linkedinResponse?.data?.data?.basicProfile?.included?.[0];
                if (miniProfile?.publicIdentifier) {
                    console.log('✅ LinkedIn data fetch successful');
                    const publicIdentifier = miniProfile.publicIdentifier;
                    
                    console.log('📝 Found LinkedIn public identifier:', publicIdentifier);
                    const linkedinProfileUrl = `https://www.linkedin.com/in/${publicIdentifier}/`;

                    // Send message to popup to skip LinkedIn profile input
                    chrome.runtime.sendMessage({
                        type: 'skipLinkedInInput',
                        linkedinProfileUrl: linkedinProfileUrl
                    });

                    // Update user with LinkedIn data and profile URL
                    displayMessage('Updating profile information...', 'info');
                    const updateSuccess = await transformAndSendLinkedInData(
                        linkedinResponse.data,
                        email,
                        data.authentication_token,
                        linkedinProfileUrl  // Pass the profile URL
                    );
                    
                    if (updateSuccess) {
                        console.log('✨ Profile update successful!');
                        displayMessage('Profile updated successfully!', 'success');
                        
                        // Store the LinkedIn profile URL
                        await chrome.storage.local.set({ linkedin_profile_url: linkedinProfileUrl });
                        
                        // Hide LinkedIn profile URL input section since we got it automatically
                        const linkedinProfileSection = document.getElementById('linkedinProfileSection');
                        if (linkedinProfileSection) {
                            linkedinProfileSection.classList.add('hidden');
                        }
                        
                        // Show success message with the found URL
                        displayMessage(`Successfully linked with profile: ${linkedinProfileUrl}`, 'success');
                    } else {
                        console.error('❌ Failed to update profile with LinkedIn data');
                        showLinkedInUrlInput(); // Fallback to manual input
                    }
                } else {
                    console.log('⚠️ No public identifier found in LinkedIn data, falling back');
                    showLinkedInUrlInput(); // Fallback to manual input
                }
            } catch (error) {
                console.error('🚨 Error during LinkedIn profile update:', error);
                showLinkedInUrlInput(); // Fallback to manual input
            }

            // Fetch updated credits
            try {
                const creditsResponse = await fetch(`${API_BASE_URL}/getCredits?email=${encodeURIComponent(email)}&authToken=${data.authentication_token}`, {
                    method: 'GET',
                    headers: {
                        'Content-Type': 'application/json',
                        'Accept': 'application/json',
                        'Origin': chrome.runtime.getURL('')
                    },
                    credentials: 'include'
                });
                
                if (creditsResponse.ok) {
                    const creditsData = await creditsResponse.json();
                    await chrome.storage.local.set({ credits: creditsData.credits });
                }
            } catch (creditsError) {
                console.error('Error fetching credits:', creditsError);
            }

            // Send message to parent window
            window.parent.postMessage({ type: 'loginSuccess', email: email }, window.location.origin);

        } catch (error) {
            console.error('Error:', error);
            loginButton.textContent = 'Login';
            loginButton.disabled = false;
            codeInput.value = '';
            loginButton.classList.add('hidden');
            displayMessage('Login failed. Please try again.', 'error');
        }
    });

    // Function to transform LinkedIn data to GoAI format
    async function transformAndSendLinkedInData(linkedinData, email, authToken, linkedinProfileUrl) {
        console.log('🔄 Starting data transformation...');
        console.log('📥 Input data:', JSON.stringify(linkedinData, null, 2));

        try {
            // Extract basic profile data
            const basicProfile = linkedinData?.data?.basicProfile?.data;
            const miniProfile = linkedinData?.data?.basicProfile?.included?.[0];
            const profileView = linkedinData?.data?.profileView?.data;
            
            console.log('🔍 Extracted profile data:', { basicProfile, miniProfile, profileView });
            
            if (!basicProfile || !miniProfile) {
                throw new Error('Missing required LinkedIn profile data');
            }

            // Transform the data to match the exact API format
            const transformedData = {
                email: email,
                authToken: authToken,  
                firstName: miniProfile.firstName,
                lastName: miniProfile.lastName,
                linkedin_profile_url: linkedinProfileUrl,
                LI_occupation: miniProfile.occupation,
                LI_objectUrn: miniProfile.objectUrn,
                LI_publicIdentifier: miniProfile.publicIdentifier,
                LI_premiumSubscriber: basicProfile.premiumSubscriber || false,
                LI_geoLocationName: profileView?.geoLocationName || '',
                LI_geoCountryName: profileView?.geoCountryName || '',
                LI_geoLocationPostalCode: profileView?.geoLocation?.postalCode || '',
                LI_summary: profileView?.summary || ''
            };

            console.log('📤 Transformed data:', transformedData);

            // Send the transformed data to the API
            const response = await fetch(`${API_BASE_URL}/updateUser`, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    'Accept': 'application/json',
                    'Origin': chrome.runtime.getURL('')
                },
                credentials: 'include',
                body: JSON.stringify(transformedData)
            });

            if (!response.ok) {
                const errorText = await response.text();
                console.error('🚫 API Error:', errorText);
                throw new Error(`API request failed with status ${response.status}: ${errorText}`);
            }

            const result = await response.json();
            console.log('✅ API Response:', result);
            return true;
        } catch (error) {
            console.error('❌ Error transforming LinkedIn data:', error);
            return false;
        }
    }

    // Function to show LinkedIn URL input as fallback
    function showLinkedInUrlInput() {
        const linkedinProfileSection = document.getElementById('linkedinProfileSection');
        if (linkedinProfileSection) {
            linkedinProfileSection.classList.remove('hidden');
            displayMessage('Please enter your LinkedIn profile URL manually', 'info');
        }
    }

    // Email validation helper function
    function isValidEmail(email) {
        const emailRegex = /^[^\s@]+@[^\s@]+\.[^\s@]+$/;
        return emailRegex.test(email);
    }

    // Move the logout handler inside the DOMContentLoaded event listener
    if (logoutLink) {
        logoutLink.addEventListener('click', async (e) => {
            e.preventDefault();
            await chrome.storage.local.remove([
                'authentication_token', 
                'email', 
                'profileSetupComplete',
                'pendingLoginEmail',
                'loginState'
            ]);
            // ... rest of logout handling ...
        });
    }

    // Add this new event listener
    startOverLink.addEventListener('click', async (e) => {
        e.preventDefault();
        
        // Clear storage
        await chrome.storage.local.remove([
            'pendingLoginEmail',
            'loginState'
        ]);

        // Reset UI
        emailInput.value = '';
        emailInput.disabled = false;
        codeInput.value = '';
        submitEmailBtn.disabled = false;
        submitEmailBtn.textContent = 'Submit';
        loginButton.classList.add('hidden');
        codeSection.classList.add('hidden');
        
        // Remove referral section if it exists
        const referralSection = document.getElementById('referralSection');
        if (referralSection) {
            referralSection.remove();
        }

        // Clear any messages
        displayMessage('', '');
    });

    // Add this near the top of DOMContentLoaded, after initializing elements
    const { loginState, pendingLoginEmail } = await chrome.storage.local.get(['loginState', 'pendingLoginEmail']);
    
    // Restore signup code request state if needed
    if (loginState === 'signup_code_request' && pendingLoginEmail) {
        emailInput.value = pendingLoginEmail;
        emailInput.disabled = true;
        submitEmailBtn.textContent = 'Submit';
        codeSection.classList.add('hidden');
        showReferralSection();
        displayMessage('Please enter referral email to create account.', 'info');
    }

    // Add terms checkbox handler
    document.getElementById('termsCheckbox').addEventListener('change', function() {
        loginButton.disabled = !this.checked;
    });
}); 