// Add at the top of the file, before the DOMContentLoaded listener
const API_BASE_URL = 'https://www.goai-app.com';

// Add these state-related functions at the top of popup.js
async function saveLoginState(state) {
  await chrome.storage.local.set({ loginState: state });
}

async function getLoginState() {
  const { loginState } = await chrome.storage.local.get('loginState');
  return loginState || 'initial'; // possible states: 'initial', 'code_sent', 'signup_code_request', 'authenticated'
}

// Add this function to check button visibility
function updateButtonVisibility() {
  const isAuthenticated = document.getElementById('postLoginContent')?.classList.contains('hidden') === false;
  
  const buttons = [
    'contactManagerBtn',
    'postGeneratorBtn',
    'reportsBtn',
    'settingsButton',
    'feedbackLink'
  ];
  
  buttons.forEach(buttonId => {
    const button = document.getElementById(buttonId);
    if (button) {
      button.style.display = isAuthenticated ? 'flex' : 'none';
    }
  });

  // Also ensure the bottom container is visible
  const bottomContainer = document.querySelector('.bottom-container');
  if (bottomContainer) {
    bottomContainer.style.display = isAuthenticated ? 'flex' : 'none';
  }
}

// Add this helper function for error handling
function handleError(error, context) {
  console.error(`Error in ${context}:`, error);
  // Optionally show error to user
  const errorMessage = document.getElementById('error-message');
  if (errorMessage) {
    errorMessage.querySelector('p').textContent = `Error: ${error.message}`;
    errorMessage.classList.remove('hidden');
    setTimeout(() => {
      errorMessage.classList.add('hidden');
    }, 3000);
  }
}

// Add these functions at the top of the file after API_BASE_URL declaration
async function checkForUpdates() {
    try {
        const response = await fetch('https://goai-app.com/version-info');
        const versionInfo = await response.json();
        const currentVersion = chrome.runtime.getManifest().version;
        
        if (versionInfo.version !== currentVersion) {
            const updateNotification = document.getElementById('updateNotification');
            const versionInfoElement = document.getElementById('versionInfo');
            versionInfoElement.textContent = `Current: ${currentVersion} \u2192 Latest: ${versionInfo.version}`;
            updateNotification.classList.remove('hidden');
        }
    } catch (error) {
        console.error('Error checking for updates:', error);
    }
}

function setupUpdateListeners() {
    const updateButton = document.getElementById('updateButton');
    const dismissButton = document.getElementById('dismissUpdate');
    const updateNotification = document.getElementById('updateNotification');

    updateButton.addEventListener('click', () => {
        chrome.tabs.create({
            url: 'https://chromewebstore.google.com/detail/goai/hclgifmeonilohjeienjloglbhbjeebb'
        });
        updateNotification.classList.add('hidden');
    });

    dismissButton.addEventListener('click', () => {
        updateNotification.classList.add('hidden');
    });
}

document.addEventListener('DOMContentLoaded', async function() {
  // Initialize all elements
  const startButton = document.getElementById('startButton');
  const loader = document.getElementById('loader');
  const errorMessage = document.getElementById('error-message');
  const loginButton = document.getElementById('loginButton');
  const loginIframe = document.getElementById('loginIframe');
  const postLoginContent = document.getElementById('postLoginContent');
  const linkedinProfileSection = document.getElementById('linkedinProfileSection');
  const userEmailDiv = document.getElementById('userEmail');
  const emailDisplay = document.getElementById('emailDisplay');
  const logoutLink = document.getElementById('logoutLink');
  const creditsDisplay = document.getElementById('creditsDisplay');

  // Check and refresh session if needed
  chrome.runtime.sendMessage({ action: 'checkSession' });

  // Get user and session data
  const { 
    email, 
    authentication_token, 
    profileSetupComplete,
    session_id,
    session_count 
  } = await chrome.storage.local.get([
    'email', 
    'authentication_token', 
    'profileSetupComplete',
    'session_id',
    'session_count'
  ]);

  // Send detailed popup open event with session data
  chrome.runtime.sendMessage({
    action: 'sendAnalyticsEvent',
    eventName: 'popup_opened',
    eventParams: {
      entry_point: 'browser_action',
      is_authenticated: Boolean(authentication_token),
      profile_setup_complete: Boolean(profileSetupComplete),
      session_number: session_count || 1,
      view_type: 'popup',
      screen_resolution: `${window.screen.width}x${window.screen.height}`,
      window_size: `${window.innerWidth}x${window.innerHeight}`,
      platform: navigator.platform,
      user_agent: navigator.userAgent
    }
  });

  // Check if user is already logged in AND has completed profile setup
  const loginState = await getLoginState();
  
  if (!email || !authentication_token) {
    if (loginState === 'signup_code_request') {
      // Show signup code request UI in the iframe
      loginButton.classList.add('hidden');
      loginIframe.classList.remove('hidden');
    } else {
      // Initial login state
      loginButton.classList.remove('hidden');
      loginIframe.classList.add('hidden');
      postLoginContent.classList.add('hidden');
      linkedinProfileSection.classList.add('hidden');
    }
  } else if (loginState === 'code_sent') {
    // Show code input section if code was previously sent
    loginButton.classList.add('hidden');
    loginIframe.classList.remove('hidden');
  } else if (loginState === 'authenticated') {
    // User is authenticated, show post-login content
    loginButton.classList.add('hidden');
    loginIframe.classList.add('hidden');
    displayUserEmail(email);
    
    if (profileSetupComplete) {
      postLoginContent.classList.remove('hidden');
      await updateCreditsDisplay();
    } else {
      linkedinProfileSection.classList.remove('hidden');
    }
    updateButtonVisibility();
    await checkForUpdates();
    setupUpdateListeners();
  }

  // Show login iframe when login button is clicked
  loginButton.addEventListener('click', () => {
    loginButton.classList.add('hidden');
    loginIframe.classList.remove('hidden');
  });

  // Initialize credits if not set
  const { credits } = await chrome.storage.local.get('credits');
  if (typeof credits === 'undefined') {
    await chrome.storage.local.set({ credits: 50 });
  }
  
  // Update credits display
  updateCreditsDisplay();

  // Add credits change listener
  chrome.storage.onChanged.addListener((changes, namespace) => {
    if (changes.credits) {
      updateCreditsDisplay();
    }
  });

  console.log('📂 Extension popup opened for the first time!');

  // Helper function to find and focus existing tab, or create new one
  async function findOrCreateTab(url) {
    const fullUrl = chrome.runtime.getURL(url);
    try {
      // Query for all tabs with the exact URL
      const tabs = await chrome.tabs.query({
        url: fullUrl
      });
      
      if (tabs.length > 0) {
        // If tab exists, focus it
        await chrome.tabs.update(tabs[0].id, { active: true });
        await chrome.windows.update(tabs[0].windowId, { focused: true });
        return;
      }
      
      // If no existing tab, create new one
      await chrome.tabs.create({ url: fullUrl });
    } catch (error) {
      console.error('Error handling tab:', error);
    }
  }

  // Listen for messages from the login iframe
  window.addEventListener('message', async (event) => {
    if (event.origin !== window.location.origin) return;
    console.log('Message received:', event.data);

    const { type, email } = event.data;
    if (type === 'loginSuccess') {
        try {
            console.log('Login success received');
            
            // Hide login iframe and button
            loginIframe.classList.add('hidden');
            loginButton.classList.add('hidden');
            
            // Show user email
            displayUserEmail(email);
            
            // Hide post login content initially
            postLoginContent.classList.add('hidden');
            
            // Check if we already have LinkedIn profile URL in storage
            chrome.storage.local.get(['linkedin_profile_url'], function(result) {
                if (result.linkedin_profile_url) {
                    console.log('LinkedIn profile URL already exists:', result.linkedin_profile_url);
                    // Skip LinkedIn input and show post-login content
                    linkedinProfileSection.classList.add('hidden');
                    postLoginContent.classList.remove('hidden');
                } else {
                    console.log('Showing LinkedIn profile section - no existing URL found');
                    linkedinProfileSection.classList.remove('hidden');
                }
            });

            // Send response to prevent message channel from closing
            event.source?.postMessage({ type: 'loginSuccessReceived' }, event.origin);
            updateButtonVisibility();
        } catch (error) {
            console.error('Error handling login success:', error);
            // Still send a response even if there's an error
            event.source?.postMessage({ type: 'loginSuccessError', error: error.message }, event.origin);
        }
    }
  });

  // Handle LinkedIn profile URL submission
  document.getElementById('submitProfileUrl').addEventListener('click', async () => {
    const profileUrlInput = document.getElementById('profileUrlInput');
    const profileUrl = profileUrlInput.value.trim();
    const linkedInUrlRegex = /^https?:\/\/(www\.)?linkedin\.com\/in\/[a-zA-Z0-9-]+\/?$/;

    if (!linkedInUrlRegex.test(profileUrl)) {
      displayMessage('Please enter a valid LinkedIn profile URL.', 'error');
      return;
    }

    try {
      const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);
      const response = await fetch(`${API_BASE_URL}/updateUser`, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'Accept': 'application/json',
          'Origin': chrome.runtime.getURL('')
        },
        credentials: 'include',
        body: JSON.stringify({ email, authToken, linkedin_profile_url: profileUrl })
      });

      if (!response.ok) {
        throw new Error('Failed to update user profile');
      }

      // Set profile setup complete flag and login state
      await chrome.storage.local.set({ 
        profileSetupComplete: true,
        loginState: 'authenticated'
      });

      displayMessage('Profile updated successfully!', 'success');
      
      // Show the main content after successful profile submission
      setTimeout(() => {
        linkedinProfileSection.classList.add('hidden');
        postLoginContent.classList.remove('hidden');
        updateButtonVisibility();
      }, 1500);
      
    } catch (error) {
      console.error('Error updating profile:', error);
      displayMessage('Failed to update profile. Please try again.', 'error');
    }
  });

  // Function to display messages
  function displayMessage(text, type) {
    const messageDiv = document.getElementById('message');
    messageDiv.textContent = text;
    messageDiv.className = `message ${type}`;
  }

  logoutLink.addEventListener('click', async (e) => {
    e.preventDefault();

    // Send logout analytics before clearing data
    chrome.runtime.sendMessage({
      action: 'sendAnalyticsEvent',
      eventName: 'user_logout',
      eventParams: {
        session_duration: Date.now() - (await chrome.storage.local.get('session_start_time')).session_start_time,
        total_sessions: session_count,
        logout_type: 'user_initiated'
      }
    });

    // Clear all login and session related states
    await chrome.storage.local.remove([
      'authentication_token',
      'email',
      'profileSetupComplete',
      'pendingLoginEmail',
      'loginState',
      'session_id',
      'session_start_time'
    ]);

    // Reset UI elements
    userEmailDiv.classList.add('hidden');
    postLoginContent.classList.add('hidden');
    linkedinProfileSection.classList.add('hidden');
    loginButton.classList.remove('hidden');
    loginIframe.classList.add('hidden');
    
    // Clear any displayed messages
    const messageDiv = document.getElementById('message');
    if (messageDiv) {
        messageDiv.textContent = '';
        messageDiv.className = 'message';
    }

    // Reset email display
    if (emailDisplay) {
        emailDisplay.textContent = '';
    }

    // Hide bottom container and other buttons
    updateButtonVisibility();
  });

  function displayUserEmail(email) {
    emailDisplay.textContent = email;
    userEmailDiv.classList.remove('hidden');
  }

  async function updateCreditsDisplay() {
    try {
      const { email, authentication_token: authToken } = await chrome.storage.local.get(['email', 'authentication_token']);
      const response = await fetch(`${API_BASE_URL}/getCredits?email=${encodeURIComponent(email)}&authToken=${authToken}`, {
        method: 'GET',
        headers: {
          'Content-Type': 'application/json',
          'Accept': 'application/json',
          'Origin': chrome.runtime.getURL('')
        },
        credentials: 'include'
      });
      
      if (!response.ok) {
        throw new Error('Failed to fetch credits');
      }
      const data = await response.json();
      creditsDisplay.textContent = data.credits;
    } catch (error) {
      console.error('Error fetching credits:', error);
      creditsDisplay.textContent = '--';
    }
  }

  // Add session tracking to button clicks
  const trackButtonClick = (buttonId, eventName) => {
    document.getElementById(buttonId)?.addEventListener('click', async () => {
      try {
        chrome.runtime.sendMessage({
          action: 'sendAnalyticsEvent',
          eventName: eventName,
          eventParams: {
            button_id: buttonId,
            session_id: session_id,
            interaction_type: 'click',
            component: 'popup_menu'
          }
        });
        await findOrCreateTab(`${buttonId.replace('Btn', '')}.html`);
      } catch (error) {
        handleError(error, buttonId);
      }
    });
  };

  // Track all main navigation buttons except Post Generator
  ['contactManagerBtn', 'reportsBtn'].forEach(buttonId => {
    trackButtonClick(buttonId, `${buttonId.toLowerCase()}_clicked`);
  });

  // Handle Post Generator button separately since it has a different URL pattern
  document.getElementById('postGeneratorBtn')?.addEventListener('click', async () => {
    try {
      chrome.runtime.sendMessage({
        action: 'sendAnalyticsEvent',
        eventName: 'post_generator_clicked',
        eventParams: {
          button_id: 'postGeneratorBtn',
          session_id: session_id,
          interaction_type: 'click',
          component: 'popup_menu'
        }
      });
      await findOrCreateTab('postCreator.html');
    } catch (error) {
      handleError(error, 'postGeneratorBtn');
    }
  });

  // Handle settings button separately since it has a different URL pattern
  document.getElementById('settingsButton')?.addEventListener('click', async () => {
    try {
      chrome.runtime.sendMessage({
        action: 'sendAnalyticsEvent',
        eventName: 'settings_button_clicked',
        eventParams: {
          button_id: 'settingsButton',
          session_id: session_id,
          interaction_type: 'click',
          component: 'popup_menu'
        }
      });
      await findOrCreateTab('settings.html');
    } catch (error) {
      handleError(error, 'settingsButton');
    }
  });

  // Track feedback link separately due to different behavior
  document.getElementById('feedbackLink')?.addEventListener('click', async (e) => {
    e.preventDefault();
    try {
      chrome.runtime.sendMessage({
        action: 'sendAnalyticsEvent',
        eventName: 'feedback_clicked',
        eventParams: {
          session_id: session_id,
          interaction_type: 'click',
          component: 'popup_menu',
          source: 'feedback_link'
        }
      });
      await findOrCreateTab('thankyou.html');
    } catch (error) {
      handleError(error, 'feedback_link');
    }
  });

  // Track auto-scrape toggle changes
  const autoScrapeToggle = document.getElementById('popupAutoScrape');
  if (autoScrapeToggle) {
    const { scrapingSettings = {} } = await chrome.storage.local.get('scrapingSettings');
    autoScrapeToggle.checked = scrapingSettings.autoScrape !== false;

    autoScrapeToggle.addEventListener('change', async function() {
      chrome.runtime.sendMessage({
        action: 'sendAnalyticsEvent',
        eventName: 'auto_scrape_toggled',
        eventParams: {
          enabled: this.checked,
          session_id: session_id,
          interaction_type: 'toggle',
          component: 'popup_settings'
        }
      });

      const updatedSettings = {
        ...scrapingSettings,
        autoScrape: this.checked
      };
      await chrome.storage.local.set({ scrapingSettings: updatedSettings });
    });
  }

  // Display version number
  const manifest = chrome.runtime.getManifest();
  const versionDisplay = document.getElementById('versionNumber');
  if (versionDisplay) {
    versionDisplay.textContent = `v${manifest.version}`;
  }

  // Add click handlers for header and version number
  document.getElementById('headerLink').addEventListener('click', () => {
    chrome.tabs.create({ url: 'https://www.goai-app.com' });
  });
  
  document.getElementById('versionNumber').addEventListener('click', () => {
    chrome.tabs.create({ url: 'https://www.goai-app.com' });
  });

  // Add cursor pointer style
  document.getElementById('headerLink').style.cursor = 'pointer';
  document.getElementById('versionNumber').style.cursor = 'pointer';

});

async function updateCreditsDisplay() {
  const creditsDisplay = document.getElementById('creditsDisplay');
  const { credits = 0 } = await chrome.storage.local.get('credits');
  creditsDisplay.textContent = credits;
  
  // Add visual indicator if credits are low
  const creditsContainer = creditsDisplay.closest('.credits-display');
  if (credits < 10) {
    creditsContainer.classList.add('low');
  } else {
    creditsContainer.classList.remove('low');
  }
}

// Update the chrome.runtime.onMessage listener
chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
    console.log('Runtime message received:', request);
    
    if (request.type === 'skipLinkedInInput') {
        // Hide LinkedIn profile input section since we already have the URL
        const linkedinProfileSection = document.getElementById('linkedinProfileSection');
        if (linkedinProfileSection) {
            linkedinProfileSection.classList.add('hidden');
        }

        // Show post-login content
        const postLoginContent = document.getElementById('postLoginContent');
        if (postLoginContent) {
            postLoginContent.classList.remove('hidden');
        }

        // Store the LinkedIn profile URL and update login state
        chrome.storage.local.set({ 
            linkedin_profile_url: request.linkedinProfileUrl,
            profileSetupComplete: true
        });
        
        // Update login state to authenticated
        saveLoginState('authenticated');
        
        // Update button visibility
        updateButtonVisibility();
    }

    // Send response to prevent message channel from closing
    sendResponse({ received: true });
    return true;
});

// Separate async function to handle runtime messages
async function handleRuntimeMessage(request) {
    switch (request.type) {
        case 'profileUpdated':
            await updateCreditsDisplay();
            break;
        // Add other cases as needed
    }
}
