const API_BASE_URL = 'https://www.goai-app.com';

// Add these utility functions at the top of the file
function jsonToCSV(data, fields) {
    // Create CSV header
    let csv = fields.join(',') + '\n';
    
    // Add data rows
    data.forEach(row => {
        const values = fields.map(field => {
            const value = row[field];
            // Handle objects (like byType) by stringifying them
            const stringValue = typeof value === 'object' ? JSON.stringify(value) : value;
            // Escape commas and quotes, wrap in quotes if needed
            return typeof value === 'string' && value.includes(',') ? 
                `"${stringValue.replace(/"/g, '""')}"` : 
                stringValue;
        });
        csv += values.join(',') + '\n';
    });
    
    return csv;
}

function downloadCSV(csv, filename) {
    const blob = new Blob([csv], { type: 'text/csv;charset=utf-8;' });
    const link = document.createElement('a');
    link.href = URL.createObjectURL(blob);
    link.download = filename;
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
}

async function fetchReportData(reportType) {
    try {
        // Get credentials from storage
        const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
        
        if (!email || !authentication_token) {
            throw new Error('Authentication required');
        }

        const response = await fetch(
            `${API_BASE_URL}/getReportingData?` + 
            `email=${encodeURIComponent(email)}&` +
            `authToken=${authentication_token}&` +
            `reportType=${reportType}`
        );

        if (!response.ok) {
            throw new Error(`API request failed: ${response.statusText}`);
        }

        const data = await response.json();
        if (!data.success) {
            throw new Error(data.message || 'Failed to fetch report data');
        }

        return data.data;
    } catch (error) {
        console.error(`Error fetching ${reportType} report:`, error);
        throw error;
    }
}

document.addEventListener('DOMContentLoaded', () => {
  chrome.runtime.sendMessage({
    action: 'sendAnalyticsEvent',
    eventName: 'reports_page_loaded',
    eventParams: {}
  });
});

document.addEventListener('DOMContentLoaded', async () => {
    try {
        // Fetch all reports in parallel
        const [profileData, messageGenData, messagesSentData] = await Promise.all([
            fetchReportData('profile-captures'),
            fetchReportData('messages-generated'),
            fetchReportData('messages-sent')
        ]);

        // Update each section with the fetched data
        await updateProfileScrapeStats(profileData);
        await updateMessageStats(messageGenData);
        await updateMessagesSentStats(messagesSentData);
    } catch (error) {
        console.error('Error loading data:', error);
        showError('Failed to load data: ' + error.message);
    }
});

function showError(message) {
    const container = document.querySelector('.container');
    const error = document.createElement('div');
    error.className = 'error';
    error.textContent = message;
    container.insertBefore(error, container.firstChild);
}

async function updateProfileScrapeStats(reportData) {
    // Calculate totals
    const totals = reportData.reduce((acc, day) => {
        acc.total += day.total;
        acc.successful += day.successful;
        acc.failed += day.failed;
        return acc;
    }, { total: 0, successful: 0, failed: 0 });

    // Update stats display
    document.getElementById('total-profiles-scraped').textContent = totals.total.toLocaleString();
    document.getElementById('basic-scrapes').textContent = totals.successful.toLocaleString();
    document.getElementById('deep-scrapes').textContent = totals.failed.toLocaleString();
    document.getElementById('scrape-success-rate').textContent = 
        `${((totals.successful / totals.total) * 100).toFixed(1)}%`;

    // Update graph
    updateDailyScrapeGraph(reportData);
}

async function updateMessageStats(reportData) {
    // Calculate totals
    const totals = reportData.reduce((acc, day) => {
        acc.total += day.total;
        acc.total_messages += day.total_messages;
        // Sum up each message type
        Object.entries(day.byType || {}).forEach(([type, count]) => {
            acc.byType[type] = (acc.byType[type] || 0) + count;
        });
        return acc;
    }, { total: 0, total_messages: 0, byType: {} });

    // Update stats display
    document.getElementById('total-messages-generated').textContent = totals.total.toLocaleString();
    
    // Update message type counts dynamically
    const messageTypes = Object.entries(totals.byType);
    if (messageTypes.length >= 2) {
        document.getElementById('total-connect-messages').textContent = 
            messageTypes[0][1].toLocaleString();
        document.getElementById('total-dm-messages').textContent = 
            messageTypes[1][1].toLocaleString();
    }
    
    document.getElementById('avg-messages-per-contact').textContent = 
        (totals.total_messages / totals.total).toFixed(1);

    // Update graph
    updateDailyMessagesGraph(reportData);
}

function updateDailyScrapeGraph(reportData) {
    const graphBars = document.getElementById('daily-scrapes-graph');
    const graphLabels = document.getElementById('daily-scrapes-labels');
    
    graphBars.innerHTML = '';
    graphLabels.innerHTML = '';
    
    const sortedData = [...reportData].sort((a, b) => new Date(a.date) - new Date(b.date));
    const maxCount = Math.max(...sortedData.map(day => day.total));
    
    sortedData.forEach(day => {
        const barContainer = document.createElement('div');
        barContainer.className = 'bar-container';
        
        if (day.total > 0 && maxCount > 0) {
            // Create a wrapper for side-by-side bars
            const barsWrapper = document.createElement('div');
            barsWrapper.className = 'bars-wrapper';
            
            if (day.successful > 0) {
                const barWrapper = document.createElement('div');
                barWrapper.className = 'bar-wrapper';
                
                const successBar = document.createElement('div');
                successBar.className = 'bar scrape-bar success';
                const successHeight = (day.successful / maxCount) * 100;
                successBar.style.height = `${successHeight}%`;
                successBar.title = `${day.date}: ${day.successful} successful captures`;
                
                barWrapper.appendChild(successBar);
                barsWrapper.appendChild(barWrapper);
            }
            
            if (day.failed > 0) {
                const barWrapper = document.createElement('div');
                barWrapper.className = 'bar-wrapper';
                
                const failBar = document.createElement('div');
                failBar.className = 'bar scrape-bar failed';
                const failHeight = (day.failed / maxCount) * 100;
                failBar.style.height = `${failHeight}%`;
                failBar.title = `${day.date}: ${day.failed} failed captures`;
                
                barWrapper.appendChild(failBar);
                barsWrapper.appendChild(barWrapper);
            }
            
            barContainer.appendChild(barsWrapper);
        }
        
        graphBars.appendChild(barContainer);
        
        const label = document.createElement('div');
        label.className = 'label';
        const date = new Date(day.date);
        label.textContent = `${date.getMonth() + 1}/${date.getDate()}`;
        graphLabels.appendChild(label);
    });
}

function updateDailyMessagesGraph(reportData) {
    const graphBars = document.getElementById('message-generation-graph');
    const graphLabels = document.getElementById('message-generation-labels');
    
    graphBars.innerHTML = '';
    graphLabels.innerHTML = '';
    
    const sortedData = [...reportData].sort((a, b) => new Date(a.date) - new Date(b.date));
    
    const messageTypes = new Set();
    sortedData.forEach(day => {
        if (day.byType) {
            Object.keys(day.byType).forEach(type => messageTypes.add(type));
        }
    });
    
    const legendContainer = document.querySelector('.graph-legend');
    legendContainer.innerHTML = '';
    
    const colorMap = {
        'connection-message': '#057642', // Green
        'direct-message': '#0a66c2'      // Blue
    };
    
    // Create a state object to track visibility
    const visibilityState = {};
    messageTypes.forEach(type => {
        visibilityState[type] = true;
    });
    
    // Store legend items for reference
    const legendItems = [];
    
    [...messageTypes].forEach((type) => {
        const legendItem = document.createElement('div');
        legendItem.className = 'legend-item';
        legendItem.style.cursor = 'pointer'; // Add pointer cursor
        legendItem.innerHTML = `
            <div class="legend-color" data-type="${type}" style="background-color: ${colorMap[type]}"></div>
            <span>${type.replace('-', ' ').replace(/\b\w/g, l => l.toUpperCase())}</span>
        `;
        
        // Add click handler
        legendItem.addEventListener('click', () => {
            visibilityState[type] = !visibilityState[type];
            
            // Toggle opacity of legend item
            legendItem.style.opacity = visibilityState[type] ? '1' : '0.5';
            
            // Toggle visibility of corresponding bars
            const bars = document.querySelectorAll(`.message-bar[data-type="${type}"]`);
            bars.forEach(bar => {
                bar.style.opacity = visibilityState[type] ? '1' : '0';
            });
        });
        
        legendItems.push(legendItem);
        legendContainer.appendChild(legendItem);
    });
    
    const maxCount = Math.max(...sortedData.map(day => 
        day.byType ? Object.values(day.byType).reduce((sum, count) => sum + count, 0) : 0
    ));
    
    sortedData.forEach(day => {
        const barContainer = document.createElement('div');
        barContainer.className = 'bar-container';
        
        if (day.byType && maxCount > 0) {
            // Create a wrapper for side-by-side bars
            const barsWrapper = document.createElement('div');
            barsWrapper.className = 'bars-wrapper';
            
            Object.entries(day.byType).forEach(([type, count]) => {
                if (count > 0) {
                    const barWrapper = document.createElement('div');
                    barWrapper.className = 'bar-wrapper';
                    
                    const bar = document.createElement('div');
                    bar.className = 'bar message-bar';
                    bar.setAttribute('data-type', type);
                    bar.style.backgroundColor = colorMap[type] || '#gray';
                    const height = (count / maxCount) * 100;
                    bar.style.height = `${height}%`;
                    bar.title = `${day.date}: ${count} ${type} messages`;
                    
                    // Add click handler to the bar
                    bar.addEventListener('click', createBarClickHandler(
                        type, 
                        visibilityState, 
                        messageTypes,
                        legendItems
                    ));
                    
                    barWrapper.appendChild(bar);
                    barsWrapper.appendChild(barWrapper);
                }
            });
            
            barContainer.appendChild(barsWrapper);
        }
        
        graphBars.appendChild(barContainer);
        
        const label = document.createElement('div');
        label.className = 'label';
        const date = new Date(day.date);
        label.textContent = `${date.getMonth() + 1}/${date.getDate()}`;
        graphLabels.appendChild(label);
    });
}

async function updateMessagesSentStats(reportData) {
    // Calculate totals
    const totals = reportData.reduce((acc, day) => {
        acc.total += day.total;
        // Sum up each message type
        Object.entries(day.byType || {}).forEach(([type, count]) => {
            acc.byType[type] = (acc.byType[type] || 0) + count;
        });
        return acc;
    }, { total: 0, byType: {} });

    // Calculate success rate (assuming successful messages are counted in total)
    const successRate = ((totals.total / Object.values(totals.byType).reduce((a, b) => a + b, 0)) * 100).toFixed(1);

    // Update stats display
    document.getElementById('total-messages-sent').textContent = totals.total.toLocaleString();
    
    // Update message type counts dynamically
    const messageTypes = Object.entries(totals.byType);
    if (messageTypes.length >= 2) {
        document.getElementById('total-sent-connect').textContent = 
            messageTypes[0][1].toLocaleString();
        document.getElementById('total-sent-dm').textContent = 
            messageTypes[1][1].toLocaleString();
    }
    
    document.getElementById('messages-sent-success-rate').textContent = `${successRate}%`;

    // Update graph
    updateMessagesSentGraph(reportData);
}

function updateMessagesSentGraph(reportData) {
    const graphBars = document.getElementById('sent-messages-graph');
    const graphLabels = document.getElementById('sent-messages-labels');
    const legendContainer = document.getElementById('sent-messages-legend');
    
    graphBars.innerHTML = '';
    graphLabels.innerHTML = '';
    legendContainer.innerHTML = '';
    
    const sortedData = [...reportData].sort((a, b) => new Date(a.date) - new Date(b.date));
    
    const messageTypes = new Set();
    sortedData.forEach(day => {
        if (day.byType) {
            Object.keys(day.byType).forEach(type => messageTypes.add(type));
        }
    });
    
    const colorMap = {
        'connection-message': '#057642', // Green
        'direct-message': '#0a66c2'      // Blue
    };
    
    // Create a state object to track visibility
    const visibilityState = {};
    messageTypes.forEach(type => {
        visibilityState[type] = true;
    });
    
    // Store legend items for reference
    const legendItems = [];
    
    [...messageTypes].forEach((type) => {
        const legendItem = document.createElement('div');
        legendItem.className = 'legend-item';
        legendItem.style.cursor = 'pointer'; // Add pointer cursor
        legendItem.innerHTML = `
            <div class="legend-color" data-type="${type}" style="background-color: ${colorMap[type]}"></div>
            <span>${type.replace('-', ' ').replace(/\b\w/g, l => l.toUpperCase())}</span>
        `;
        
        // Add click handler
        legendItem.addEventListener('click', () => {
            visibilityState[type] = !visibilityState[type];
            
            // Toggle opacity of legend item
            legendItem.style.opacity = visibilityState[type] ? '1' : '0.5';
            
            // Toggle visibility of corresponding bars
            const bars = document.querySelectorAll(`.message-bar[data-type="${type}"]`);
            bars.forEach(bar => {
                bar.style.opacity = visibilityState[type] ? '1' : '0';
            });
        });
        
        legendItems.push(legendItem);
        legendContainer.appendChild(legendItem);
    });
    
    const maxCount = Math.max(...sortedData.map(day => 
        day.byType ? Object.values(day.byType).reduce((sum, count) => sum + count, 0) : 0
    ));
    
    sortedData.forEach(day => {
        const barContainer = document.createElement('div');
        barContainer.className = 'bar-container';
        
        if (day.byType && maxCount > 0) {
            // Create a wrapper for side-by-side bars
            const barsWrapper = document.createElement('div');
            barsWrapper.className = 'bars-wrapper';
            
            Object.entries(day.byType).forEach(([type, count]) => {
                if (count > 0) {
                    const barWrapper = document.createElement('div');
                    barWrapper.className = 'bar-wrapper';
                    
                    const bar = document.createElement('div');
                    bar.className = 'bar message-bar';
                    bar.setAttribute('data-type', type);
                    bar.style.backgroundColor = colorMap[type] || '#gray';
                    const height = (count / maxCount) * 100;
                    bar.style.height = `${height}%`;
                    bar.title = `${day.date}: ${count} ${type} messages`;
                    
                    // Add click handler to the bar
                    bar.addEventListener('click', createBarClickHandler(
                        type, 
                        visibilityState, 
                        messageTypes,
                        legendItems
                    ));
                    
                    barWrapper.appendChild(bar);
                    barsWrapper.appendChild(barWrapper);
                }
            });
            
            barContainer.appendChild(barsWrapper);
        }
        
        graphBars.appendChild(barContainer);
        
        const label = document.createElement('div');
        label.className = 'label';
        const date = new Date(day.date);
        label.textContent = `${date.getMonth() + 1}/${date.getDate()}`;
        graphLabels.appendChild(label);
    });
}

// Add event listeners for export buttons
document.addEventListener('DOMContentLoaded', () => {
    // Message Generation Stats Export
    document.getElementById('export-message-stats').addEventListener('click', async () => {
        try {
            const data = await fetchReportData('messages-generated');
            const fields = ['date', 'total', 'total_messages'];
            
            // Add dynamic message type fields from byType
            const messageTypes = new Set();
            data.forEach(day => {
                if (day.byType) {
                    Object.keys(day.byType).forEach(type => messageTypes.add(type));
                }
            });
            
            // Prepare data with flattened byType values
            const csvData = data.map(day => {
                const row = {
                    date: day.date,
                    total: day.total,
                    total_messages: day.total_messages
                };
                // Add message type counts
                messageTypes.forEach(type => {
                    row[`messages_${type}`] = day.byType?.[type] || 0;
                });
                return row;
            });
            
            // Add message type fields to header
            messageTypes.forEach(type => fields.push(`messages_${type}`));
            
            const csv = jsonToCSV(csvData, fields);
            downloadCSV(csv, `message-generation-stats-${new Date().toISOString().split('T')[0]}.csv`);
        } catch (error) {
            console.error('Error exporting message generation stats:', error);
            showError('Failed to export message generation stats: ' + error.message);
        }
    });

    // Messages Sent Stats Export
    document.getElementById('export-sent-messages').addEventListener('click', async () => {
        try {
            const data = await fetchReportData('messages-sent');
            const fields = ['date', 'total'];
            
            // Add dynamic message type fields from byType
            const messageTypes = new Set();
            data.forEach(day => {
                if (day.byType) {
                    Object.keys(day.byType).forEach(type => messageTypes.add(type));
                }
            });
            
            // Prepare data with flattened byType values
            const csvData = data.map(day => {
                const row = {
                    date: day.date,
                    total: day.total
                };
                // Add message type counts
                messageTypes.forEach(type => {
                    row[`sent_${type}`] = day.byType?.[type] || 0;
                });
                return row;
            });
            
            // Add message type fields to header
            messageTypes.forEach(type => fields.push(`sent_${type}`));
            
            const csv = jsonToCSV(csvData, fields);
            downloadCSV(csv, `messages-sent-stats-${new Date().toISOString().split('T')[0]}.csv`);
        } catch (error) {
            console.error('Error exporting sent messages stats:', error);
            showError('Failed to export sent messages stats: ' + error.message);
        }
    });

    // Profile Capture Stats Export
    document.getElementById('export-profile-scrapes').addEventListener('click', async () => {
        try {
            const data = await fetchReportData('profile-captures');
            const fields = ['date', 'total', 'successful', 'failed'];
            
            const csv = jsonToCSV(data, fields);
            downloadCSV(csv, `profile-capture-stats-${new Date().toISOString().split('T')[0]}.csv`);
        } catch (error) {
            console.error('Error exporting profile capture stats:', error);
            showError('Failed to export profile capture stats: ' + error.message);
        }
    });
});

// Update both updateDailyMessagesGraph and updateMessagesSentGraph functions
// Add this function at bar creation in both graph update functions:

function createBarClickHandler(type, visibilityState, messageTypes, legendItems) {
    return () => {
        // Toggle between showing only this type or showing all types
        const isOnlyTypeVisible = Object.entries(visibilityState)
            .every(([t, visible]) => t === type ? visible : !visible);

        if (isOnlyTypeVisible) {
            // Show all types
            messageTypes.forEach(t => {
                visibilityState[t] = true;
                const bars = document.querySelectorAll(`.message-bar[data-type="${t}"]`);
                bars.forEach(bar => {
                    bar.style.opacity = '1';
                });
                // Update legend items
                legendItems.forEach(item => {
                    if (item.querySelector(`[data-type="${t}"]`)) {
                        item.style.opacity = '1';
                    }
                });
            });
        } else {
            // Show only this type
            messageTypes.forEach(t => {
                visibilityState[t] = (t === type);
                const bars = document.querySelectorAll(`.message-bar[data-type="${t}"]`);
                bars.forEach(bar => {
                    bar.style.opacity = t === type ? '1' : '0';
                });
                // Update legend items
                legendItems.forEach(item => {
                    if (item.querySelector(`[data-type="${t}"]`)) {
                        item.style.opacity = t === type ? '1' : '0.5';
                    }
                });
            });
        }
    };
} 