let aiSetupHasChanges = false;
let scrapingSettingsHasChanges = false;
let linkedinElementsHasChanges = false;

// Add API_BASE_URL constant at the top
const API_BASE_URL = 'https://www.goai-app.com';

// Add these utility functions near the top of the file
function showSuccess(message) {
  const saveStatus = document.getElementById('saveStatus');
  saveStatus.textContent = message;
  saveStatus.className = 'save-status show success';
  
  // Hide after 3 seconds
  setTimeout(() => {
    saveStatus.className = 'save-status';
  }, 3000);
}

function showError(message) {
  const saveStatus = document.getElementById('saveStatus');
  saveStatus.textContent = message;
  saveStatus.className = 'save-status show error';
  
  // Hide after 5 seconds
  setTimeout(() => {
    saveStatus.className = 'save-status';
  }, 5000);
}

document.addEventListener('DOMContentLoaded', async function() {
  try {
    // Initialize collapsible sections first
    initializeCollapsibleSections();
    
    // Expand AI Setup section by default
    const aiSetupSection = document.getElementById('aiSetup');
    if (aiSetupSection) {
      aiSetupSection.classList.remove('collapsed');
      const content = aiSetupSection.querySelector('.section-content');
      if (content) {
        content.style.maxHeight = content.scrollHeight + 'px';
      }
    }
    
    // Then load the rest of the content
    await loadAISetupContent();

    // Set up scraping settings handlers
    const deepActivityScraping = document.getElementById('deepActivityScraping');
    const deepScrapingSettings = document.getElementById('deepScrapingSettings');

    // Load saved scraping settings
    await loadScrapingSettings();

    // Toggle deep scraping settings visibility
    if (deepActivityScraping && deepScrapingSettings) {
        deepActivityScraping.addEventListener('change', function() {
            deepScrapingSettings.style.display = this.checked ? 'block' : 'none';
            saveScrapingSettings();
        });
    }

    // Set up event listeners
    setupAISetupListeners();

    // Add save button event listeners
    document.getElementById('saveAISetup').addEventListener('click', saveAISetupContent);

  } catch (error) {
    console.error('Error initializing settings:', error);
    showError('Failed to initialize settings: ' + error.message);
  }
});

async function loadAISetupContent() {
  try {
    // Get values from storage with proper error handling
    const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
    
    if (!email || !authentication_token) {
      console.error('Missing email or authentication token');
      showError('Please log in again to access your settings');
      return;
    }

    // Get the textarea elements
    const instructionsElem = document.getElementById('instructions');
    const userInstructionsElem = document.getElementById('userInstructions');
    const aboutYouElem = document.getElementById('aboutYou');
    const aboutProductElem = document.getElementById('aboutProduct');
    const aiPersonaSelect = document.getElementById('aiPersona');

    try {
      // Get instructions from API
      const instructionsResponse = await fetch(
        `${API_BASE_URL}/getInstructions?email=${encodeURIComponent(email)}&authToken=${authentication_token}`,
        {
          headers: {
            'Accept': 'application/json'
          }
        }
      );
      
      if (!instructionsResponse.ok) {
        throw new Error(`HTTP error! status: ${instructionsResponse.status}`);
      }
      
      const instructionsData = await instructionsResponse.json();
      if (instructionsData.success) {
        userInstructionsElem.value = instructionsData.instructions || '';
      }

      // Get About You and About Product from API
      const aboutFilesResponse = await fetch(
        `${API_BASE_URL}/getAboutFiles?email=${encodeURIComponent(email)}&authToken=${authentication_token}`,
        {
          headers: {
            'Accept': 'application/json'
          }
        }
      );

      if (!aboutFilesResponse.ok) {
        throw new Error(`HTTP error! status: ${aboutFilesResponse.status}`);
      }

      const aboutData = await aboutFilesResponse.json();
      if (aboutData.success) {
        aboutYouElem.value = aboutData.aboutYou || '';
        aboutProductElem.value = aboutData.aboutProduct || '';
      }

      // Get AI Persona from storage (or set default)
      const { aiPersona } = await chrome.storage.local.get('aiPersona');
      if (aiPersona) {
        aiPersonaSelect.value = aiPersona;
      }

      // Update instructions based on persona
      await updateInstructionsWithUserInput();

      // Load existing promo instructions
      const response = await fetch(`${API_BASE_URL}/getInstructions?email=${email}&authToken=${authentication_token}&instructionsType=promo`);
      const promoCTAResponse = await fetch(`${API_BASE_URL}/getInstructions?email=${email}&authToken=${authentication_token}&instructionsType=promo-cta`);
      
      if (response.ok && promoCTAResponse.ok) {
        const promoData = await response.json();
        const promoCTAData = await promoCTAResponse.json();
        
        // Update the text areas with the fetched data
        document.getElementById('promoInstructions').value = promoData.instructions || '';
        document.getElementById('promoCta').value = promoCTAData.instructions || '';
      }

      // Clear unsaved state only on initial load
      if (!aiSetupHasChanges) {
        clearUnsavedStyles();
      }

      updatePromptBuilder();

    } catch (error) {
      console.error('Error loading content from API:', error);
      showError(`Failed to load settings from server: ${error.message}`);
      
      // Fall back to storage if API fails
      const { aboutYou, aboutProduct, userInstructions } = await chrome.storage.local.get([
        'aboutYou',
        'aboutProduct',
        'userInstructions'
      ]);

      if (aboutYou) aboutYouElem.value = aboutYou;
      if (aboutProduct) aboutProductElem.value = aboutProduct;
      if (userInstructions) userInstructionsElem.value = userInstructions;
    }

  } catch (error) {
    console.error('Error loading AI setup content:', error);
    showError('Failed to load AI setup content: ' + error.message);
  }
}

function updatePromptBuilder() {
  const aboutYou = document.getElementById('aboutYou').value;
  const aboutProduct = document.getElementById('aboutProduct').value;
  const instructions = document.getElementById('instructions').value;
  const promptBuilder = document.getElementById('promptBuilder');
  
  // Create a combined prompt
  const combinedPrompt = `Instructions:\n${instructions}\n\nAbout Me:\n${aboutYou}\n\nAbout Product/Service:\n${aboutProduct}`;
  promptBuilder.value = combinedPrompt;
  
  // Don't clear the unsaved state here
}

// Update event listeners to include instructions textarea
document.getElementById('aboutYou').addEventListener('input', updatePromptBuilder);
document.getElementById('aboutProduct').addEventListener('input', updatePromptBuilder);
document.getElementById('instructions').addEventListener('input', updatePromptBuilder);

// Add these functions for collapsible sections
function initializeCollapsibleSections() {
  const sections = document.querySelectorAll('.settings-section');
  
  sections.forEach(section => {
    const header = section.querySelector('.section-header');
    const content = section.querySelector('.section-content');
    
    // Set initial state
    if (section.classList.contains('collapsed')) {
      content.style.maxHeight = '0';
    } else {
      content.style.maxHeight = content.scrollHeight + 'px';
    }
    
    header.addEventListener('click', () => {
      console.log('Section header clicked'); // Debug log
      section.classList.toggle('collapsed');
      
      if (section.classList.contains('collapsed')) {
        content.style.maxHeight = '0';
      } else {
        content.style.maxHeight = content.scrollHeight + 'px';
      }
    });
  });
}

// Add these new functions to handle unsaved changes
function markAsUnsaved(textarea) {
  if (!textarea.classList.contains('unsaved')) {
    textarea.classList.add('unsaved');
    aiSetupHasChanges = true;
  }
}

function clearUnsavedStyles() {
  const aiSetupSection = document.getElementById('aiSetup');
  if (!aiSetupSection) return;

  const formElements = aiSetupSection.querySelectorAll('.unsaved');
  formElements.forEach(element => {
    element.classList.remove('unsaved');
  });
  aiSetupHasChanges = false;
}

function setupAISetupListeners() {
  // Get all input elements in the AI Setup section
  const aiSetupSection = document.getElementById('aiSetup');
  if (!aiSetupSection) return;

  // Add listeners to all form elements
  const formElements = aiSetupSection.querySelectorAll('input, select, textarea');
  formElements.forEach(element => {
    element.addEventListener('input', () => {
      markAsUnsaved(element);
    });
  });
}

// Update the Model Settings section
function setupModelSettingsListeners() {
  const inputs = document.querySelectorAll('#modelSettings input, #modelSettings select');
  
  inputs.forEach(input => {
    const newInput = input.cloneNode(true);
    input.parentNode.replaceChild(newInput, input);
    
    newInput.addEventListener('input', () => {
      markElementAsUnsaved(newInput);
      updateModelSettingsUI();
    });
    
    newInput.addEventListener('change', () => {
      markElementAsUnsaved(newInput);
      updateModelSettingsUI();
    });
  });
}

// Update the Scraping Settings section
function setupScrapingSettingsListeners() {
  const inputs = document.querySelectorAll('#scrapingSettings input');
  
  inputs.forEach(input => {
    const newInput = input.cloneNode(true);
    input.parentNode.replaceChild(newInput, input);
    
    newInput.addEventListener('input', () => {
      markElementAsUnsaved(newInput);
    });
    
    newInput.addEventListener('change', () => {
      markElementAsUnsaved(newInput);
    });
  });
}

// Generic functions for handling unsaved changes
function markElementAsUnsaved(element) {
  if (element.type === 'checkbox') {
    // For toggle switches, add unsaved class to the parent toggle-switch div
    const toggleSwitch = element.closest('.toggle-switch');
    if (toggleSwitch && !toggleSwitch.classList.contains('unsaved')) {
      toggleSwitch.classList.add('unsaved');
    }
  } else if (!element.classList.contains('unsaved')) {
    element.classList.add('unsaved');
  }

  const section = element.closest('.settings-section');
  if (section) {
    switch(section.id) {
      case 'modelSettings':
        modelSettingsHasChanges = true;
        break;
      case 'scrapingSettings':
        scrapingSettingsHasChanges = true;
        break;
    }
  }
}

function clearUnsavedStylesForSection(sectionId) {
  const inputs = document.querySelectorAll(`#${sectionId} input, #${sectionId} select, #${sectionId} textarea`);
  inputs.forEach(input => {
    if (input.type === 'checkbox') {
      // For toggle switches, remove unsaved class from the parent toggle-switch div
      const toggleSwitch = input.closest('.toggle-switch');
      if (toggleSwitch) {
        toggleSwitch.classList.remove('unsaved');
      }
    } else {
      input.classList.remove('unsaved');
    }
  });
  
  switch(sectionId) {
    case 'modelSettings':
      modelSettingsHasChanges = false;
      break;
    case 'scrapingSettings':
      scrapingSettingsHasChanges = false;
      break;
  }
}

async function saveLinkedInElements() {
  try {
    const selectors = {
      profile: {
        name: document.getElementById('nameSelector').value,
        tagline: document.getElementById('taglineSelector').value,
        location: document.getElementById('locationSelector').value,
        followers: document.getElementById('followersSelector').value,
        connections: document.getElementById('connectionsSelector').value,
        about: document.getElementById('aboutSelector').value,
        experience: document.getElementById('experienceSelector').value,
        multiPosition: document.getElementById('multiPositionSelector').value
      },
      activity: {
        post: document.getElementById('postSelector').value,
        timestamp: document.getElementById('timestampSelector').value,
        content: document.getElementById('contentSelector').value,
        reactions: document.getElementById('reactionsSelector').value,
        comments: document.getElementById('commentsSelector').value
      }
    };

    await chrome.storage.local.set({ selectors });
    clearUnsavedStylesForSection('linkedinElements');
    showSuccess('LinkedIn elements saved successfully');
  } catch (error) {
    console.error('Error saving LinkedIn elements:', error);
    showError('Failed to save LinkedIn elements: ' + error.message);
  }
}

// Update updateInstructionsWithUserInput function to use API
async function updateInstructionsWithUserInput() {
  try {
    const instructionsElem = document.getElementById('instructions');
    const userInstructionsElem = document.getElementById('userInstructions');
    const aiPersonaSelect = document.getElementById('aiPersona');
    
    // Get auth credentials
    const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
    
    // Get instructions from API based on persona
    const response = await fetch(
      `${API_BASE_URL}/getInstructions?email=${encodeURIComponent(email)}&authToken=${authentication_token}&persona=${aiPersonaSelect.value}`,
      {
        headers: {
          'Accept': 'application/json'
        }
      }
    );

    if (!response.ok) {
      throw new Error(`HTTP error! status: ${response.status}`);
    }

    const data = await response.json();
    if (data.success) {
      // Update instructions element with the received instructions
      instructionsElem.value = data.instructions || '';
      
      // If there's user input, append it to the instructions
      if (userInstructionsElem.value) {
        instructionsElem.value += '\n\nUser Instructions:\n' + userInstructionsElem.value;
      }
    } else {
      throw new Error(data.message || 'Failed to get instructions');
    }
  } catch (error) {
    console.error('Error updating instructions with user input:', error);
    showError('Failed to update instructions: ' + error.message);
  }
}

// Add this function to update the UI display values for model settings
function updateModelSettingsUI() {
  // Update temperature display
  const temperatureValue = document.getElementById('temperatureValue');
  if (temperatureValue) {
    temperatureValue.textContent = document.getElementById('temperature').value;
  }

  // Update top p display
  const topPValue = document.getElementById('topPValue');
  if (topPValue) {
    topPValue.textContent = document.getElementById('topP').value;
  }

  // Update frequency penalty display
  const frequencyPenaltyValue = document.getElementById('frequencyPenaltyValue');
  if (frequencyPenaltyValue) {
    frequencyPenaltyValue.textContent = document.getElementById('frequencyPenalty').value;
  }

  // Update presence penalty display
  const presencePenaltyValue = document.getElementById('presencePenaltyValue');
  if (presencePenaltyValue) {
    presencePenaltyValue.textContent = document.getElementById('presencePenalty').value;
  }
}

async function loadScrapingSettings() {
    try {
        const { scrapingSettings } = await chrome.storage.local.get('scrapingSettings');
        if (scrapingSettings) {
            // Set autoScrape to true by default if it's not set
            document.getElementById('autoScrape').checked = 
                scrapingSettings.autoScrape !== undefined ? scrapingSettings.autoScrape : true;
            
            const deepActivityScraping = document.getElementById('deepActivityScraping');
            const deepScrapingSettings = document.getElementById('deepScrapingSettings');
            
            // Set the checkbox state
            deepActivityScraping.checked = scrapingSettings.deepActivityScraping;
            
            // Show/hide settings based on checkbox state
            if (deepScrapingSettings) {
                deepScrapingSettings.style.display = deepActivityScraping.checked ? 'block' : 'none';
            }

            document.getElementById('numPostsScrape').value = scrapingSettings.numPostsScrape;
            document.getElementById('autoScroll').checked = scrapingSettings.autoScroll;
            document.getElementById('maxPosts').value = scrapingSettings.maxPosts;
        } else {
            // Set default values if no settings exist
            document.getElementById('autoScrape').checked = true;
            document.getElementById('deepActivityScraping').checked = false;
            document.getElementById('deepScrapingSettings').style.display = 'none';
        }
    } catch (error) {
        console.error('Error loading scraping settings:', error);
        showError('Failed to load scraping settings: ' + error.message);
    }
}

async function saveScrapingSettings() {
    try {
        const scrapingSettings = {
            autoScrape: document.getElementById('autoScrape').checked,
            deepActivityScraping: document.getElementById('deepActivityScraping').checked,
            numPostsScrape: parseInt(document.getElementById('numPostsScrape').value),
            autoScroll: document.getElementById('autoScroll').checked,
            maxPosts: parseInt(document.getElementById('maxPosts').value)
        };

        await chrome.storage.local.set({ scrapingSettings });
        clearUnsavedStylesForSection('scrapingSettings');
        showSuccess('Scraping settings saved successfully');
    } catch (error) {
        console.error('Error saving scraping settings:', error);
        showError('Failed to save scraping settings: ' + error.message);
    }
}

// Modify saveAISetupContent function
async function saveAISetupContent() {
  try {
    const { email, authentication_token } = await chrome.storage.local.get(['email', 'authentication_token']);
    
    if (!email || !authentication_token) {
      showError('Please log in again to save your settings');
      return;
    }

    // Get all changed form elements
    const aiSetupSection = document.getElementById('aiSetup');
    const changedElements = aiSetupSection.querySelectorAll('.unsaved');
    
    if (changedElements.length === 0) {
      showSuccess('No changes to save');
      return;
    }

    const responses = [];
    const aboutData = { email, authToken: authentication_token };
    let hasAboutChanges = false;

    // Process each changed element
    for (const element of changedElements) {
      const value = element.value;
      const elementId = element.id;

      switch (elementId) {
        case 'userInstructions':
          responses.push(
            fetch(`${API_BASE_URL}/updateInstructions`, {
              method: 'POST',
              headers: { 'Content-Type': 'application/json' },
              body: JSON.stringify({
                email,
                authToken: authentication_token,
                instructionsType: 'user',
                instructions: value
              })
            })
          );
          break;

        case 'aboutYou':
          aboutData.aboutYou = value;
          hasAboutChanges = true;
          break;

        case 'aboutProduct':
          aboutData.aboutProduct = value;
          hasAboutChanges = true;
          break;

        case 'promoInstructions':
          responses.push(
            fetch(`${API_BASE_URL}/updateInstructions`, {
              method: 'POST',
              headers: { 'Content-Type': 'application/json' },
              body: JSON.stringify({
                email,
                authToken: authentication_token,
                instructionsType: 'promo',
                instructions: value
              })
            })
          );
          break;

        case 'promoCta':
          responses.push(
            fetch(`${API_BASE_URL}/updateInstructions`, {
              method: 'POST',
              headers: { 'Content-Type': 'application/json' },
              body: JSON.stringify({
                email,
                authToken: authentication_token,
                instructionsType: 'promo-cta',
                instructions: value
              })
            })
          );
          break;
      }
    }

    // If about fields changed, add the updateAboutFiles request
    if (hasAboutChanges) {
      responses.push(
        fetch(`${API_BASE_URL}/updateAboutFiles`, {
          method: 'POST',
          headers: { 'Content-Type': 'application/json' },
          body: JSON.stringify(aboutData)
        })
      );
    }

    // Wait for all requests to complete
    const results = await Promise.all(responses);
    
    // Check if all requests were successful
    const allSuccessful = results.every(response => response.ok);
    
    if (allSuccessful) {
      // Save to local storage only the changed values
      const storageUpdate = {};
      changedElements.forEach(element => {
        storageUpdate[element.id] = element.value;
      });
      await chrome.storage.local.set(storageUpdate);

      clearUnsavedStyles();
      showSuccess('Settings saved successfully');
    } else {
      throw new Error('Some settings failed to save');
    }
  } catch (error) {
    console.error('Error saving AI setup content:', error);
    showError('Failed to save AI setup content: ' + error.message);
  }
}

async function clearContacts() {
    if (confirm('Are you sure you want to delete all contacts?')) {
        try {
            await window.csvUtils.writeCSV([]);
            alert('Contacts Cleared!');
        } catch (error) {
            console.error('Error clearing contacts:', error);
            showError('Failed to clear contacts: ' + error.message);
        }
    }
}

document.addEventListener('DOMContentLoaded', () => {
  chrome.runtime.sendMessage({
    action: 'sendAnalyticsEvent',
    eventName: 'settings_page_loaded',
    eventParams: {}
  });
});
